include "Common_generic_commands.thrift"
include "Generic_types.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd
 
service CIM_generic_commands extends Common_generic_commands.Common_generic_commands
{
    /**
     * Read barcode
     *
     * Return the barcode data that is read
     */
    binary barcode_read(    /** Maximum duration of the operation in seconds*/
                            1: i32  timeout_in_sec)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error          generic_xcept,
                    /** Timeout has been reached*/
                    2: Generic_types.Timeout_error          ex1,
                    /** Invalid argument*/
                    3: Generic_types.Invalid_argument_error ex2);
               
    /**
     * Prints a bitmap image using printer attached to the terminal.
     *
     * Note: This command is only supported in S4 product.
     */
    void print_bitmap_image(    /** Data of the bitmap image to be printed */
                                1: binary bitmap_data)
        throws( /** Bad file format */
                1: Generic_types.Bad_file_format                bad_file_format_xcept
                /** Paper out */
                2: Generic_types.Printer_paper_out              paper_out_xcept
                /** Temperature too hot */
                3: Generic_types.Printer_temperature_too_hot    temp_too_hot_xcept
                /** Cover open */
                4: Generic_types.Printer_cover_open             cover_open_xcept
                /** Cutter error */
                5: Generic_types.Printer_cutter_error           cutter_err_xcept
                /** Printer not plugged */
                6: Generic_types.Printer_not_plugged            printer_not_plugged_xcept
                /** General printer error */
                7: Generic_types.Printer_error                  printer_err_xcept
                /** Generic exception */
                8: Generic_types.Generic_error                  generic_xcept);

    /**
     * Instructs the printer to cut the paper.
     *
     * Note: This command is only supported in S4 product.
     */
    void paper_cut()
        throws( /** Printer not plugged */
                1: Generic_types.Printer_not_plugged    printer_not_plugged_xcept
                /** General printer error */
                2: Generic_types.Printer_error          printer_err_xcept
                /** Generic exception */
                3: Generic_types.Generic_error          generic_xcept);
		
    /**
     * Feeds empty lines (paper) into the printer.
     * 
     * Note: This command is only supported in S4 product.
     */
    void paper_feed(    /** Number of lines to feed */
                        1: i32 number_of_lines)
        throws( /** Paper out */
                1: Generic_types.Printer_paper_out              paper_out_xcept
                /** Temperature too hot */
                2: Generic_types.Printer_temperature_too_hot    temp_too_hot_xcept
                /** Cover open */
                3: Generic_types.Printer_cover_open             cover_open_xcept
                /** Cutter error */
                4: Generic_types.Printer_cutter_error           cutter_err_xcept
                /** Printer not plugged */
                5: Generic_types.Printer_not_plugged            printer_not_plugged_xcept
                /** General printer error */
                6: Generic_types.Printer_error                  printer_err_xcept
                /** Generic exception */
                7: Generic_types.Generic_error                  generic_xcept);

    /**
     * Displays pre-defined text on the vacuum fluroscent display.
     *
     * The pre-defined text is displayed in following manner on 2 LED lines:<br>
     * "Morpho"<br>
     * "S4 Product"
     *
     * Note: This command is only supported in S4 product.
     */
    void VFD_display_text()
        throws( /** VFD not connected */
                1: Generic_types.VFD_not_connected VFD_not_connected_xcept
                /** VFD internal error */
                2: Generic_types.VFD_error VFD_err_xcept
                /** Generic exception */
                3: Generic_types.Generic_error generic_xcept);

    /**
     * Get anti-tamper mechanism status (intrusion or not).
     *
     * Return  status of anti-tamper mechanism 0 : No intrusion, 1 : Intrusion
     *
     * Note: This command is only supported in S4 product.
     */
    i32 get_anti_tamper_status()
        throws( /** Anti-tamper not found */
                1: Generic_types.Anti_tamper_not_found  Anti_tamper_not_found_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * Reset the anti-tamper mechanism status (to no intrusion).
     *
     * Note: This command is only supported in S4 product.
     */
    void reset_anti_tamper_status()
        throws( /** Anti-tamper not found */
                1: Generic_types.Anti_tamper_not_found  Anti_tamper_not_found_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * Get anti-tamper battery status (low battery or not).
     *
     * Return the battery status of anti-tamper  0 : Low battery, 1 : Battery is not low
     *
     * Note: This command is only supported in S4 product.
     */
    i32 get_anti_tamper_battery_status()
        throws( /** Anti-tamper not found */
                1: Generic_types.Anti_tamper_not_found  Anti_tamper_not_found_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * Get WiFi connection status (connected or not).
     *
     * Return the connection status of WiFi  0 : Not connected, 1 : Connected
     */
    i32 get_WiFi_conn_status()
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * Get VFD connection status (connected or not).
     *
     * Return the connection status of VFD  0 : Not connected, 1 : Connected
     *
     * Note: This command is only supported in S4 product.
     */
    i32 get_VFD_conn_status()
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * Get Barcode connection status (connected or not).
     *
     * Return the connection status of Barcode  0 : Not connected, 1 : Connected
     */
    i32 get_barcode_conn_status()
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * Get printer connection status (connected or not).
     *
     * Return the connection status of Printer. It may contain multiple status values.
     * Can be extracted by bit operations with values from Generic_types.Printer_error_code enumeration. 
     *
     * Note: This command is only supported in S4 product.
     */
    i32 get_printer_conn_status()
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * Get printer Serial number.
     *
     * Return the serial number of Printer.
     *
     * Note: This command is only supported in S4 & SONIC product.
     */
    string get_printer_serial_no()
        throws( /** Printer not plugged */
                1: Generic_types.Printer_not_plugged printer_not_plugged_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * Get Barcode reader Serial number.
     *
     * Return the Serial number of Barcode reader.
     */
    string get_barcode_serial_no()
        throws( /** Barcode reader not connected */
                1: Generic_types.BCR_not_connected BCR_not_connected_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);
                
    /** 
     * Write printer Serial number.
     *
     * Note: This command is only supported in S4 & SONIC product.
     */
    void write_printer_serial_num(  /** Serial number to write */
                                    1: binary serial_num)
        throws( /** Printer not plugged */
                1: Generic_types.Printer_not_plugged printer_not_plugged_xcept
                /** Printer bad serial number */
                2: Generic_types.Printer_bad_serial printer_bad_serial_xcept
                /** General printer error */
                3: Generic_types.Printer_error printer_err_xcept
                /** Generic exception */
                4: Generic_types.Generic_error generic_xcept);

    /** 
     * Get Printer Firmware version.
     *
     * Return the Firmware version of Printer.
     *
     * Note: This command is only supported in S4 & SONIC product.
     */
    string get_printer_firmware_version()
        throws( /** Printer not plugged */
                1: Generic_types.Printer_not_plugged printer_not_plugged_xcept
                /** General printer error */
                2: Generic_types.Printer_error printer_err_xcept
                /** Generic exception */
                3: Generic_types.Generic_error generic_xcept);
}