// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2012 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Common_factory_commands.thrift
 *
 * CHANGELOG
 * 04 nov 2015  - Create file from MA5G and CIM factory commands
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 09 Jan 2017 - Add exception 'Generic_types.Invalid_argument_error' for command 'factory_retrieve_touchpoints' 
 * 24 Mar 2017  - PRGCA Product support added 
 * 02 Jul 2018  - MA_VP_MD Product support added 
 */
// </doc_history>
 
include "Generic_types.thrift"
include "Factory_types.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd
 
service Common_factory_commands
{
    /**
     * Get factory information from terminal board or sensor board.
     *
     *
     * Return Requested information
     */
    Factory_types.Factory_info factory_get_info(    /** Board to retrieve information from */
                                                    1: Factory_types.Factory_board_type board_type,
                                                    /** List of requested information */
                                                    2: set<Factory_types.Factory_info_type> info_type)
        throws (
                /** Invalid parameter*/
                1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);
                
    /**
     * Set factory information in flash memory for terminal or sensor board
     */
    void factory_set_info(  /** Board for which information will be set */
                            1: Factory_types.Factory_board_type board_type,
                            /**
                             * Structure that contains factory parameter to set. <br>
                             * The structure can contain several parameters.
                             */
                            2: Factory_types.Factory_info parameter)
        throws (
                /** Invalid parameter*/
                1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);
                
    /**
     * <product>Y;;Y;;Y;;Y;;Y;;Y;;Y</product>
     * Change the state of terminal's back LEDs
     */
    void MMI_set_leds(  /** State of LEDs to apply (ON or OFF) */
                        1: Factory_types.MMI_LED_state leds_state)
        throws (
                /** Invalid parameter*/
                1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);
                
    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Test Touchscreen calibration testing
     *
     * Return result of calibration testing 0 : success, 1 : failure
     */
    bool factory_test_LCD_calibration()
        throws (
                /** Generic exception */
                1: Generic_types.Generic_error generic_xcept
        );
        
    /**
     * Read data from external device
     *
     *
     * Data are read as they are
     */
    binary factory_read_external_device(   /** Target device */
                                            1: Factory_types.Factory_external_device device_type,
                                            /** File to read */
                                            2: string filename)
        throws (
                /** Invalid parameter */
                1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept,
                /** Device unmounted */
                2: Factory_types.Fac_device_unmounted_error device_unmounted_xcept,
                /** Cannot read file */
                3: Factory_types.Fac_read_file_error read_file_xcept
                /** Generic exception */
                4: Generic_types.Generic_error generic_xcept);

    /**
     * Write data parameter into external device
     *
     *
     * Data are put as they are
     */
    void factory_write_external_device(    /** Target device */
                                            1: Factory_types.Factory_external_device device_type,
                                            /** File to read */
                                            2: string filename,
                                            /** Data to write */
                                            3: binary data)
        throws (
                /** Invalid parameter*/
                1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept,
                /** Device unmounted */
                2: Factory_types.Fac_device_unmounted_error device_unmounted_xcept,
                /** Cannot write file */
                3: Factory_types.Fac_write_file_error write_file_xcept
                /** Generic exception */
                4: Generic_types.Generic_error generic_xcept);
                
    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Write LCD Calibration
     *
     *
     * Data are put as they are
     */
    void factory_write_LCD_calibration(    /** Data to write in sensor parameter */
                                           1: binary data)
        throws (
                /** Cannot write file */
                1: Factory_types.Fac_write_file_error write_file_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Read LCD Calibration
     *
     *
     * Data are read as they are
     */
    binary factory_read_LCD_calibration()
        throws (
                /** Cannot read file */
                1: Factory_types.Fac_read_file_error read_file_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Read microphone
     */
    binary factory_read_microphone(    /** Record duration */
                                        1: i32 duration_in_sec,
                                        /** Record frame rate */
                                        2: i32 frame_rate_in_Hz)
        throws (
                /** Invalid parameter */
                1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept,
                /** ALSA error */
                2: Factory_types.Fac_alsa_error alsa_error_xcept
                /** Generic exception */
                3: Generic_types.Generic_error generic_xcept);
                
    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     * Get COTS version
     *
     * Return string containing COTS' versions.
     * Returned string respects json format: {"AT90"="XX.XX", "SIO"="XX.XX", ... }
     */
    string factory_COTS_get_version()
        throws (
                /** Generic exception */
                1: Generic_types.Generic_error generic_xcept
        );
        
    /**
     * <product>Y;;Y;;Y;;Y;;Y;;N;;Y</product>    
     * Read fuse
     *
     * Return list of values of requested fuse from iMX6 CPU.
     */
    map<Factory_types.Factory_fuse_imx6, i32> factory_read_fuse(/** list of fuse to read */
                                                                1: set<Factory_types.Factory_fuse_imx6> list_fuse_address)
        throws (
                /** Invalid parameter*/
                1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);
 
        
    /**
     * <product>N;;N;;N;;N;;N;;Y;;N</product>    
     * Read fuse
     *
     * Return list of values of requested fuse from CPU.
     */
    map<string, string> factory_read_cpu_fuse(/** list of fuse to read, input is list of fuse name */
                                         1: set<string> list_fuse_address)
        throws (
                /** Invalid parameter*/
                1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);
                 
    /**
     * Load upgrade policy in terminal
     *
     * This function is used to upgrade terminal's Firmware upgrade policy. <br>
     * The upgrade policy is used by the terminal before upgrade process. <br>
     *
     * Return upgrade policy status.
     */
    Factory_types.Factory_upgrade_policy_status upgrade_load_policy(    /** Raw content of the policy file */
                                                                        1:  Factory_types.Upgrade_policy_file file_data)
        throws (    /** Incompatible version*/
                    1: Factory_types.Fac_firmware_upgrade_incompatible_version_error incompatible_version_xcept,
                    /** Incompatible product*/
                    2: Factory_types.Fac_firmware_upgrade_incompatible_product_error incompatible_product_xcept,
                    /** Incompatible hardware revision */
                    3: Factory_types.Fac_firmware_upgrade_incompatible_hardware_revision_error incompatible_hardware_revision_xcept
                    /** Generic exception */
                    4: Generic_types.Generic_error generic_xcept);

    /**
     * Start upgrade process
     *
     * This function is used to start upgrade process if upgrade policy is loaded and it is OK. <br>
     * Terminal will restart and boot from backup Firmware.
     */
    void start_upgrade()
        throws (    /** Upgrade policy not uploaded*/
                    1: Factory_types.Fac_firmware_upgrade_policy_not_uploaded_error policy_not_uploaded_xcept,
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Calibrate terminal LCD touchscreen
     *
     *
     * This function performs LCD touchscreen calibration.
     */
    bool factory_calibrate_LCD()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;N;;N;;N;;N</product>
     * Set the brightness level of LCD backlight
     *
     *
     * This function sets LCD backlight brightness
     */
    bool factory_set_LCD_brightness(    /** Brightness value in percentage */
                                        1: i32  brightness_value)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept
                    /** Invalid parameter set */
                    2: Generic_types.Invalid_argument_error invalid_arg_xcept);

    /**
     * <product>N;;N;;N;;N;;N;;N;;N</product>    
     * Set the brightness level of LED
     *
     * This function sets brightness of the blue LEDs
     */
    bool factory_set_LED_brightness(    /** Brightness value in percentage */
                                        1: i32  brightness_value)
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept
                    /** Invalid parameter set */
                    2: Generic_types.Invalid_argument_error invalid_arg_xcept);
 
    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Retrieve touch point co-ordinates of LCD
     *
     *
     * This function performs retrieve list of touchpoints with their co-ordinates
     */
    list<Factory_types.Touchpoint_coordinate> factory_retrieve_touchpoints(
                                              /** No of touchpoints to retrieve */
                                              1: i32 no_touchpoint,
                                              /** Duration in seconds during which the touchpoint retrieve is running(valid range is from 0 to 2147483) */
                                              2: i32 duration_in_sec)
        throws (
                /** Time out */
                1: Generic_types.Timeout_error timeout_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept
		/** Invalid parameter*/
		3: Generic_types.Invalid_argument_error invalid_argument_xcept);
}

