// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2013 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file QA_commands.thrift
 *
 * CHANGELOG
 * 28 may 2013  - Added commands 'cbi_simulation_files_load' and 'cbi_simulation_files_delete_all'
 * 13 nov 2013  - Added commands 'mfu_get_stats' and 'mfu_set_stats'
 * 18 dec 2013  - Added commands 'mfu_get_size', 'mfu_set_size' and 'mfu_synchronize_statistics'
 * 26 Dec 2013  - Added exception 'Generic_error' in 'throws' section of every command
 * 20 feb 2014  - Add namespace for java and C# generation
 * 29 jan 2015  - MorphoWave Product support changes
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 20 july 2016 - Changed simulation thrift command for new CBI/MSI SDK
 *                renamed cbi_simulation_files_load to simulation_files_load 
 *                renamed cbi_simulation_files_delete_all to simulation_files_delete_all
 * 30 Sept 2016 - added 'cbi_simulation_files_load' & 'cbi_simulation_files_delete_all' command to avoid conflict in thrift merge
 * 24 Mar 2017  - PRGCA Product support added  
 * 20 Sept 2017  - MAXT removed 'simulation_files_load' command from list of QA commands
 * 03 Jul 2018  - MA_VP_MD Product support added 
 * 17 June 2019 - added 'get_life_cycle_counters' command to get life cycle counter value.
 */
// </doc_history>

include "MA5G_generic_commands.thrift"
include "Generic_types.thrift"
include "QA_types.thrift"
include "File_types.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

service MA5G_QA_commands extends MA5G_generic_commands.MA5G_generic_commands
{
    /**
     * <product>Y;;Y;;Y;;N;;N;;N;;N</product>
     * Loads files required for sensor simulation mode 
     * <note>For MorphoWave terminal, only one file is required.</note>
     */
    void cbi_simulation_files_load( /** Simulation files to load */
                                    1: QA_types.Cbi_simulation_files files)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;N;;N;;N</product>
     * Delete all simu record soft links required for sensor simulation mode
     */
    void cbi_simulation_files_delete_all()
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;N;;Y;;Y;;N</product>    
     * Delete all simu record soft links required for sensor simulation mode
     */
    void simulation_files_delete_all()
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;N</product>
     * Sets the MFU statistics in CBI sensor for the specified users
     */
    void mfu_set_stats (/** Database identifier */
                        1: byte db_index,
                        /** MAP<User ID, stats> */
                        2: map<string, i32> user_id_stats)
        throws (/** Invalid parameter*/
                1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                /** The specified user ID has not been found in the database.*/
                2: Generic_types.Inexistent_user_id_error inexistant_user_xcept
                /** Generic exception */
                3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;N</product>
     * Gets the MFU statistics in CBI sensor for the specified users
     *
     * Return a map of UserIDs and their corresponding statistics.<br>
     *         If any user is not found, it will not be present in the map.
     */
    map<string, i32> mfu_get_stats (/** Database identifier */
                                    1: byte db_index,
                                    /** List User ID */
                                    2: set<string> user_id_UTF8)
        throws (/** Invalid parameter*/
                1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                /** The specified user ID has not been found in the database.*/
                2: Generic_types.Inexistent_user_id_error inexistant_user_xcept
                /** Generic exception */
                3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;N</product>
     * Gets the current MFU size from CBI sensor
     *
     * Return the current size of the MFU in the sensor
     */
    i32 mfu_get_size ()
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;N;;N</product>
     * Sets the MFU size in CBI sensor
     */
    void mfu_set_size ( /** Size of MFU to be set in sensor */
                        1: i32 size)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;N</product>
     * Synchronize MFU statistics on the CBI sensor
     */
    void mfu_synchronize_statistics ()
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**************************
     * logFile**********
    *****************/

    File_types.File_chunk logfile_get_tar()

    void logfile_flush();

    bool logfile_set_enabled(1: bool enabled);

    bool logfile_get_enabled();

    void logfile_set_filter(1: string filter_str);

    void logfile_delete();

    /**
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     * Get the life cycle counters values
     * Return a map of Life_cycle_counter_types and their corresponding Value.<br>
     */
    map<QA_types.Life_cycle_counter_types, i32> get_life_cycle_counters (1: set<QA_types.Life_cycle_counter_types> life_cycle_counter_list)
        throws (/** Invalid parameter*/
                1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);
}
