// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2015 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Sensor_commands.thrift
 *
 * CHANGELOG
 * 26 Jan 2015  - First draft created
 * 27 Jan 2015  - Multi response help added, reboot command added and comment changes
 * 21 Mar 2015  - Pairing command added
 * 26 Mar 2015  - renamed reset_MA5G_terminal_ID to sensor_forget_pairing added
 * 31 Mar 2015  - renamed sensor_DB_delete_records to sensor_DB_delete_users, sensor_DB_delete_all_records to sensor_DB_delete_all_users
 * 21 Apr 2015  - Added Sensor_file_error error in Sensor_get_file process 
 * 05 Aug 2015  - 'sensor_file_load', 'sensor_start_upgrade' and 'sensor_get_upgrade_status' command added 
 */
// </doc_history>
include "Sensor_types.thrift"

namespace cpp Sensor_cmd
namespace java com.morpho.Sensor

service Sensor_commands
{
    /**
     * Captures user biometrics and returns biometric templates, and returns images.
     *
     *
     * The template is calculated after number of acquisitions. <br><br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * Similar to the <a href=#SampleBioIdentify>C++ sample code</a> provided for the method <code>bio_identify</code>
     */
    Sensor_types.Enroll_reply bio_enroll(   /** Enrollment parameter to use */
                                            1: Sensor_types.Enroll_param enroll_param)
        throws (/** sensor used by another process */
                1: Sensor_types.Sensor_in_use_error err_sensor_used
                /** sensor operation cancelled  */
                2: Sensor_types.Sensor_cancel_error err_sensor_cancel
                /** parameter not set properly  */
                3: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor not found  */
                4: Sensor_types.Sensor_device_not_found_error err_sensor_not_found
                /** MorphoWave sensor not found  */
                5: Sensor_types.Sensor_device_time_out_error err_sensor_time_out
                /** MorphoWave sensor power supply error  */
                6: Sensor_types.Sensor_power_supply_error err_sensor_power_supply
                /** MorphoWave sensor internal error  */
                7: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                8: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor license not available in system */
                9: Sensor_types.Sensor_license_missing_error err_sensor_license_missing
                /** MorphoWave sensor coding error */
                10: Sensor_types.Sensor_coding_error err_sensor_coding
                /** MorphoWave sensor FVP error */
                11: Sensor_types.Sensor_FVP_error err_sensor_FVP
                /** MorphoWave sensor compression error */
                12: Sensor_types.Sensor_compression_error err_sensor_compression
                /** MorphoWave sensor acquisition error */
                13: Sensor_types.Sensor_acquisition_error err_sensor_acquisition
                /** False finger error */
                14: Sensor_types.Sensor_acquisition_status_false_finger_detected_error err_sensor_false_finger
                /** Moist finger error */
                15: Sensor_types.Sensor_acquisition_status_moist_finger_error err_sesor_moist_finger
                /** MorphoWave sensor acquisition status unknown error */
                16: Sensor_types.Sensor_acquisition_status_unknown_error err_status_unknown
                /** MorphoWave sensor security error */
                17: Sensor_types.Sensor_security_error err_sensor_security
                /** SDK data conversion error */
                18: Sensor_types.Sensor_convertion_error err_sensor_convertion
                /** MorphoWave sensor memory error */
                19: Sensor_types.Sensor_memory_error err_sensor_memory
                /** MorphoWave sensor SDK received parameter with null value */
                20: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** MorphoWave sensor USB went to deadlock state */
                21: Sensor_types.Sensor_fatal_error err_sensor_fatal        
                /** Generic exception */
                22: Sensor_types.Sensor_error generic_xcept);

    /**
     * Identify an user using its biometric record against the sensor database
     *
     *
     * <b>Warning:</b> If @a Control_param::enable_intermediate_replies is true, you will need a modified version of the Thrift client that supports
     *          the reception of several T_REPLY messages for one command.<br><br>
     * <b id="SampleBioIdentify">Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * <pre>
     * void Sensor_commandsClient::bio_identify(::Sensor_cmd::Control_reply& _return, const ::Sensor_cmd::Control_param& control_param)
     * {
     *     send_bio_identify(control_param);
     *     <span style="color:green">// recv_bio_identify(_return); // Original line in the generated code</span>
     * &nbsp;
     *     <span style="color:green">// BEGIN: Custom section for receiving and processing intermediate data</span>
     *     uint32_t loop_id = 0;
     *     while (true)
     *     {
     *         <span style="color:green">// receive intermediate response</span>
     *         recv_bio_identify(_return);
     * &nbsp;
     *         <span style="color:green">// check for final_result status</span>
     *         if (_return.__isset.final_result)
     *         {
     *             break; <span style="color:green">// if final_result callback, break from loop</span>
     *         }
     *         else if (_return.__isset.Control_intermediate_result    ) <span style="color:green">// check for interested callback</span>
     *         {
     *             _return.__isset.Control_intermediate_result     false; <span style="color:green">// reset the callback flag</span>
     *             if (Multiresponse_handler::Control_intermediate_result)
     *             {
     *                 <span style="color:green">// process callback data</span>
     *                 Multiresponse_handler::Control_intermediate_result(_return.Control_intermediate_result);
     *             }
     *         }
     *     }
     *     <span style="color:green">// END: Custom section for receiving and processing intermediate data</span>
     * }
     * </pre>
     */     
    Sensor_types.Control_reply bio_identify(/** control parameter to use */
                                            1: Sensor_types.Control_param control_param)
        throws (/** sensor used by another process */
                1: Sensor_types.Sensor_in_use_error err_sensor_used
                /** sensor operation cancelled  */
                2: Sensor_types.Sensor_cancel_error err_sensor_cancel
                /** parameter not set properly  */
                3: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor not found  */
                4: Sensor_types.Sensor_device_not_found_error err_sensor_not_found
                /** MorphoWave sensor not found  */
                5: Sensor_types.Sensor_device_time_out_error err_sensor_time_out
                /** MorphoWave sensor power supply error  */
                6: Sensor_types.Sensor_power_supply_error err_sensor_power_supply
                /** MorphoWave sensor internal error  */
                7: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                8: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor license not available in system */
                9: Sensor_types.Sensor_license_missing_error err_sensor_license_missing
                /** MorphoWave sensor coding error */
                10: Sensor_types.Sensor_coding_error err_sensor_coding
                /** MorphoWave sensor FVP error */
                11: Sensor_types.Sensor_FVP_error err_sensor_FVP
                /** MorphoWave sensor compression error */
                12: Sensor_types.Sensor_compression_error err_sensor_compression
                /** MorphoWave sensor acquisition error */
                13: Sensor_types.Sensor_acquisition_error err_sensor_acquisition
                /** False finger error */
                14: Sensor_types.Sensor_acquisition_status_false_finger_detected_error err_sensor_false_finger
                /** Moist finger error */
                15: Sensor_types.Sensor_acquisition_status_moist_finger_error err_sesor_moist_finger
                /** MorphoWave sensor acquisition status unknown error */
                16: Sensor_types.Sensor_acquisition_status_unknown_error err_status_unknown
                 /** no record found in identification */
                17: Sensor_types.Sensor_acquisition_status_no_hit_error err_no_hit
                /** MorphoWave sensor security error */
                18: Sensor_types.Sensor_security_error err_sensor_security
                /** SDK data conversion error */
                19: Sensor_types.Sensor_convertion_error err_sensor_convertion
                /** MorphoWave sensor memory error */
                20: Sensor_types.Sensor_memory_error err_sensor_memory
                /** MorphoWave sensor SDK received parameter with null value */
                21: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** MorphoWave sensor USB went to deadlock state */
                22: Sensor_types.Sensor_fatal_error err_sensor_fatal        
                /** Generic exception */
                23: Sensor_types.Sensor_error generic_xcept);

    /**
     * Authenticates an user using its biometric against its database references or biometric record references
     *
     *
     * <b>Warning:</b> If @a Control_param::enable_intermediate_replies is true, you will need a modified version of the Thrift client that supports
     *          the reception of several T_REPLY messages for one command.<br><br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * Similar to the <a href=#SampleBioIdentify>C++ sample code</a> provided for the method <code>bio_identify</code>
     */
    Sensor_types.Control_reply bio_authenticate(/** control parameter to use */
                                                1: Sensor_types.Control_param control_param)
        throws (/** sensor used by another process */
                1: Sensor_types.Sensor_in_use_error err_sensor_used
                /** sensor operation cancelled  */
                2: Sensor_types.Sensor_cancel_error err_sensor_cancel
                /** parameter not set properly  */
                3: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor not found  */
                4: Sensor_types.Sensor_device_not_found_error err_sensor_not_found
                /** MorphoWave sensor not found  */
                5: Sensor_types.Sensor_device_time_out_error err_sensor_time_out
                /** MorphoWave sensor power supply error  */
                6: Sensor_types.Sensor_power_supply_error err_sensor_power_supply
                /** MorphoWave sensor internal error  */
                7: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                8: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor license not available in system */
                9: Sensor_types.Sensor_license_missing_error err_sensor_license_missing
                /** MorphoWave sensor coding error */
                10: Sensor_types.Sensor_coding_error err_sensor_coding
                /** MorphoWave sensor FVP error */
                11: Sensor_types.Sensor_FVP_error err_sensor_FVP
                /** MorphoWave sensor compression error */
                12: Sensor_types.Sensor_compression_error err_sensor_compression
                /** MorphoWave sensor acquisition error */
                13: Sensor_types.Sensor_acquisition_error err_sensor_acquisition
                /** False finger error */
                14: Sensor_types.Sensor_acquisition_status_false_finger_detected_error err_sensor_false_finger
                /** Moist finger error */
                15: Sensor_types.Sensor_acquisition_status_moist_finger_error err_sesor_moist_finger
                /** MorphoWave sensor acquisition status unknown error */
                16: Sensor_types.Sensor_acquisition_status_unknown_error err_status_unknown
                /** no record found in identification */
                17: Sensor_types.Sensor_acquisition_status_no_hit_error err_no_hit
                /** MorphoWave sensor security error */
                18: Sensor_types.Sensor_security_error err_sensor_security
                /** SDK data conversion error */
                19: Sensor_types.Sensor_convertion_error err_sensor_convertion
                /** MorphoWave sensor memory error */
                20: Sensor_types.Sensor_memory_error err_sensor_memory
                /** MorphoWave sensor SDK received parameter with null value */
                21: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** MorphoWave sensor USB went to deadlock state */
                22: Sensor_types.Sensor_fatal_error err_sensor_fatal
                /** Generic exception */
                23: Sensor_types.Sensor_error generic_xcept);

    /**
     * Cancel current delayed command
     */
    void cancel_operation()
        throws (/** MorphoWave sensor internal error  */
                1: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                2: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor memory error */
                3: Sensor_types.Sensor_memory_error err_sensor_memory                        
                /** parameter not set properly  */
                4: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** Generic exception */
                5: Sensor_types.Sensor_error generic_xcept
        );

    /**
     * Adds or modify one or more user records in the database
     *
     *
     * Only the fields which are present in the Bio_record structure will be modified.
     *
     * <b>Warning:</b> users list should be limited to 100 entries otherwise sensor may misbehave.
     */
    void sensor_DB_set_users(   /** Mapping of User IDs (UTF8 string) to their database fields*/
                                1: map<string, list<Sensor_types.Bio_record> > users
                                /**
                                  * If set to true,  checks  on  reference  templates  are performed:  same  templates  cannot  be  used  twice,<br>
                                  * and  the  person  must  not  be  already  enrolled.<br>
                                  * If set to false, these checks are not performed.<br>
                                  * This  option  is  useful  to  reduce  the  time  taken  to  fill  large databases.<br>
                                  * In this case, the database coherence must be previously checked.
                                  */
                                2: bool   enable_similarity_check)
        throws (/** Database creation failed  */
                1: Sensor_types.Sensor_database_creation_error err_database_creation_failed
                /** Invalid template format  */
                2: Sensor_types.Sensor_invalid_template_error err_invalid_template
                /** User already exist in database with same id  */
                3: Sensor_types.Sensor_user_exist_error err_user_exists
                /** Biometric data is invalid  */
                4: Sensor_types.Sensor_biometric_data_error err_biometric_data
                /** parameter not set properly  */
                5: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor internal error  */
                6: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                7: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor coding error */
                8: Sensor_types.Sensor_coding_error err_sensor_coding
                /** MorphoWave sensor FVP error */
                9: Sensor_types.Sensor_FVP_error err_sensor_FVP
                /** MorphoWave sensor compression error */
                10: Sensor_types.Sensor_compression_error err_sensor_compression
                /** MorphoWave sensor security error */
                11: Sensor_types.Sensor_security_error err_sensor_security
                /** SDK data conversion error */
                12: Sensor_types.Sensor_convertion_error err_sensor_convertion
                /** MorphoWave sensor memory error */
                13: Sensor_types.Sensor_memory_error err_sensor_memory
                /** MorphoWave sensor SDK received parameter with null value */
                14: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** MorphoWave user record exceed the database size */
                15: Sensor_types.Sensor_user_record_exceed_database_size_error err_sensor_database_size_exceed                    
                /** Generic exception */
                16: Sensor_types.Sensor_error generic_xcept);

    /**
     * Deletes one or more users from the sensor database
     *
     *
     * <b>Warning:</b> If a user is not found, no error is reported.
     */
    map<string, bool> sensor_DB_delete_users(   /** User IDs (UTF8 string) of users to delete*/
                                                1: set<string> user_IDs_UTF8)
        throws (/** Database creation failed  */
                1: Sensor_types.Sensor_database_creation_error err_database_creation_failed
                /** MorphoWave sensor internal error  */
                2: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                3: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor memory error */
                4: Sensor_types.Sensor_memory_error err_sensor_memory
                /** parameter not set properly  */
                5: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** Generic exception */
                6: Sensor_types.Sensor_error generic_xcept);

    /**
     * Deletes all users from the sensor database
     */
    void sensor_DB_delete_all_users()
        throws (/** Database creation failed  */
                1: Sensor_types.Sensor_database_creation_error err_database_creation_failed
                /** MorphoWave sensor internal error  */
                2: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                3: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor memory error */
                4: Sensor_types.Sensor_memory_error err_sensor_memory
                /** Generic exception */
                5: Sensor_types.Sensor_error generic_xcept);

    /**
     * Start sensor software upgrade. The sensor upgrade file must be send prior to this command through <code>Sensor_commands.sensor_file_load</code>  command
     */
    void sensor_start_upgrade()
        throws (/** FOTF sensor incompatible version error  */
                1: Sensor_types.Sensor_upgrade_incompatible_version_error err_sensor_upgrade
                /** FOTF sensor internal error  */
                2: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                3: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** Generic exception */
                4: Sensor_types.Sensor_error generic_xcept);
        
   /**
     * Get sensor software upgrade status. This command will only applicable after <code>Sensor_commands.sensor_start_upgrade</code> command
     *
     * Return a binary value <b>true</b> for success and <b>false</b> for fail will be received.
     */
    bool sensor_get_upgrade_status()
        throws (/** FOTF sensor internal error  */
                1: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                2: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** Generic exception */
                3: Sensor_types.Sensor_error generic_xcept);

    /**
     * Checks presence of one or more user records in sensor database.
     *
     *
     * <b>Warning:</b> The list of user ID shall be limited to 100 items max.
     *
     *
     * Return a map of UserIDs and their corresponding status.<br>
     *         If a user_IDs is not found, its corresponding status will be false.
     */
    map<string, bool> sensor_DB_check_users(/** List of User IDs to return*/
                                            1: set<string>  user_IDs_UTF8)
        throws (/** Database creation failed  */
                1: Sensor_types.Sensor_database_creation_error err_database_creation_failed
                /** Biometric data is invalid  */
                2: Sensor_types.Sensor_biometric_data_error err_biometric_data
                /** MorphoWave sensor internal error  */
                3: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                4: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** Parameter not set properly  */
                6: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor memory error */
                7: Sensor_types.Sensor_memory_error err_sensor_memory
                /** MorphoWave sensor SDK received parameter with null value */
                8: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** Generic exception */
                9: Sensor_types.Sensor_error generic_xcept);

    /**
     * Returns the list of all user IDs (UTF8 string) present in sensor database
     */
    set<string> sensor_DB_get_user_IDs( )
        throws (/** Database creation failed  */
                1: Sensor_types.Sensor_database_creation_error err_database_creation_failed
                /** Biometric data is invalid  */
                2: Sensor_types.Sensor_biometric_data_error err_biometric_data
                /** MorphoWave sensor internal error  */
                3: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                4: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor memory error */
                5: Sensor_types.Sensor_memory_error err_sensor_memory
                /** MorphoWave sensor SDK received parameter with null value */
                6: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** Generic exception */
                7: Sensor_types.Sensor_error generic_xcept);    

    /**
     * Gets the user's database status
     *
     * Return the database capacity and number of records.
     */
    Sensor_types.DB_status sensor_DB_get_status()
        throws (/** Database creation failed  */
                1: Sensor_types.Sensor_database_creation_error err_database_creation_failed
                /** MorphoWave sensor internal error  */
                2: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                3: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor memory error */
                4: Sensor_types.Sensor_memory_error err_sensor_memory
                /** Generic exception */
                5: Sensor_types.Sensor_error generic_xcept);

    /**
     * Returns a list of values corresponding to requested parameters
     */
    list<Sensor_types.Variant> sensor_get_params(   /** List of UTF8 string representing names of requested parameters */
                                                    1: list<string> param_list_UTF8)
        throws (/** parameter not set properly  */
                1: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch                    
                /** Command not available error  */
                2: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor SDK received parameter with null value */
                3: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** Generic exception */
                4: Sensor_types.Sensor_error generic_xcept);        

    /**
     * Set requested parameters with given values
     */
    void sensor_set_params( /**
                             * List of key-value pairs.<br>
                             * Parameters are represented by a UTF8 string
                             */
                            1: map<string, Sensor_types.Variant> param_map)
        throws (/** parameter not set properly  */
                1: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor internal error  */
                2: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                3: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** Generic exception */
                4: Sensor_types.Sensor_error generic_xcept);

    /**
     * Forget sensor pairing, used in factory to pairing after testing, MorphoWave specific command
     */
    void sensor_forget_pairing( )
        throws (/** parameter not set properly  */
                1: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor not found  */
                2: Sensor_types.Sensor_device_not_found_error err_sensor_not_found
                /** MorphoWave sensor not found  */
                3: Sensor_types.Sensor_device_time_out_error err_sensor_time_out
                /** MorphoWave sensor power supply error  */
                4: Sensor_types.Sensor_power_supply_error err_sensor_power_supply
                /** MorphoWave sensor internal error  */
                5: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                6: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor license not available in system */
                7: Sensor_types.Sensor_license_missing_error err_sensor_license_missing
                /** MorphoWave sensor coding error */
                8: Sensor_types.Sensor_coding_error err_sensor_coding
                /** MorphoWave sensor FVP error */
                9: Sensor_types.Sensor_FVP_error err_sensor_FVP
                /** MorphoWave sensor compression error */
                10: Sensor_types.Sensor_compression_error err_sensor_compression
                /** MorphoWave sensor acquisition error */
                11: Sensor_types.Sensor_acquisition_error err_sensor_acquisition
                /** MorphoWave sensor acquisition status unknown error */
                12: Sensor_types.Sensor_acquisition_status_unknown_error err_status_unknown
                /** MorphoWave sensor security error */
                13: Sensor_types.Sensor_security_error err_sensor_security
                /** Generic exception */
                14: Sensor_types.Sensor_error generic_xcept);

    /**
     * Get sensor information 
     */        
    Sensor_types.Sensor_info sensor_get_info(   /** List of requested information */
                                                1: set<Sensor_types.Sensor_info_type> info_type)
        throws (/** parameter not set properly  */
                1: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** Command not available error  */
                2: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor SDK received parameter with null value */
                3: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** Generic exception */
                4: Sensor_types.Sensor_error generic_xcept);

    /**
     * Loads files required for sensor simulation mode
     */
    void sensor_simulation_files_load(  /** Simulation files to load */
                                        1: Sensor_types.Sensor_simulation_files files)
        throws (/** parameter not set properly  */
                1: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** Command not available error  */
                2: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor SDK received parameter with null value */
                3: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** Generic exception */
                4: Sensor_types.Sensor_error generic_xcept);

    /**
     * Delete all files required for sensor simulation mode
     */
    void sensor_simulation_files_delete_all()
        throws (/** parameter not set properly  */
                1: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** Command not available error  */
                2: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor SDK received parameter with null value */
                3: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** Generic exception */
                4: Sensor_types.Sensor_error generic_xcept);

    /**
     * Retrieve a file from the sensor.<br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * <pre>
     *  void Sensor_commands::sensor_file_get( ::Sensor_cmd::File_chunk& _return, const  ::Sensor_cmd::File_details& file_details)
     *  {
     *      send_sensor_file_get(file_details);
     *      <span style="color:green">// recv_sensor_file_get(_return); // Original line in the generated code</span>
     * &nbsp;
     *      <span style="color:green">// BEGIN: Custom section for receiving and processing intermediate data</span>
     *      while (true)
     *      {
     *          recv_sensor_file_get(_return); <span style="color:green">// Multiple reception in loop</span>
     * &nbsp;
     *          <span style="color:green">// Following is the client's callback assumed to take necessary action on the received data,
     *          //  e.g., keep writing the received partial data to the file until the last chunk of data is received.</span>
     *          Multiresponse_handler::client_cb_sensor_file_get(_return);
     * &nbsp;
     *          if(_return.is_last) <span style="color:green">// Exit the loop if the data received was last of all the multiple data received, i.e., no more data to be received</span>
     *          {
     *              break;
     *          }
     *      }
     *      <span style="color:green">// END: Custom section for receiving and processing intermediate data</span>
     *  }
     * </pre>
     */     
    Sensor_types.File_chunk sensor_file_get(/** File details of the file to obtain
                                             * (<code>Sensor_types.File_details.name_UTF8</code> shall be ignored except where multiple files exist)
                                             */
                                            1: Sensor_types.File_details file_details)
        throws(/** parameter not set properly  */
                1: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor internal error  */
                2: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                3: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor SDK received parameter with null value */
                4: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** To large log file to download */
                5: Sensor_types.Sensor_file_error err_sensor_file
                /** Generic exception */
                6: Sensor_types.Sensor_error generic_xcept);

    /**
     * Send a file to sensor.<br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * <pre>
     *  void Sensor_commands::sensor_file_load(const  ::Sensor_cmd::File_details& file_details, ::Sensor_cmd::File_chunk& chunk)
     *  {
     *      <span style="color:green">// send_sensor_file_load(file_details, chunk);
     *      // recv_sensor_file_load(); // Original line in the generated code</span>
     * &nbsp;
     *      <span style="color:green">// BEGIN: Custom section for receiving and processing intermediate data</span>
     *      Sensor_types.File_chunk actual_chunk; <span style="color:green">// a non-const chunk required</span>
     *      while (true)
     *      {     
     *          <span style="color:green">// Following is the client's callback assumed to take necessary action on the received data,
     *          //  e.g., keep sending partial data to the file till the last chunk of data.</span>
     *          Multiresponse_handler::client_cb_sensor_file_load(actual_chunk);
     * &nbsp;
     *          recv_sensor_file_load(); <span style="color:green">// required to see if exception was received -- so that further sending is stopped</span>
     *          if(actual_chunk.is_last) <span style="color:green">// Exit the loop if the data received was last of all the multiple data received, i.e., no more data to be received</span>
     *          {
     *              break;
     *          }
     *      }
     *      <span style="color:green">// END: Custom section for receiving and processing intermediate data</span>
     *  }
     * </pre>
     */     
    void    sensor_file_load(   /** File details of the file to be send
                                 * (<code>Sensor_types.File_details.name_UTF8</code> shall be ignored except where multiple files exist)
                                 */
                                1: Sensor_types.File_details      file_details,
                                /** The chunk of the file */
                                2: Sensor_types.File_chunk chunk)
        throws (/** parameter not set properly  */
                1: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** FOTF sensor internal error  */
                2: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                3: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** FOTF sensor SDK recieved parameter with null value */
                4: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** To large log file to download */
                5: Sensor_types.Sensor_file_error err_sensor_file
                /** Generic exception */
                6: Sensor_types.Sensor_error generic_xcept);

    /**
     * When the command returns, sensor is going to reboot
     */
    void sensor_reboot()
        throws (/** sensor used by another process */
                1: Sensor_types.Sensor_in_use_error err_sensor_used
                /** parameter not set properly  */
                2: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor not found  */
                3: Sensor_types.Sensor_device_not_found_error err_sensor_not_found
                /** MorphoWave sensor not found  */
                4: Sensor_types.Sensor_device_time_out_error err_sensor_time_out
                /** MorphoWave sensor power supply error  */
                5: Sensor_types.Sensor_power_supply_error err_sensor_power_supply
                /** MorphoWave sensor internal error  */
                6: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                7: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor license not available in system */
                8: Sensor_types.Sensor_license_missing_error err_sensor_license_missing
                /** MorphoWave sensor FVP error */
                9: Sensor_types.Sensor_FVP_error err_sensor_FVP
                /** MorphoWave sensor acquisition error */
                10: Sensor_types.Sensor_acquisition_error err_sensor_acquisition
                /** MorphoWave sensor acquisition status unknown error */
                11: Sensor_types.Sensor_acquisition_status_unknown_error err_status_unknown
                /** MorphoWave sensor security error */
                12: Sensor_types.Sensor_security_error err_sensor_security
                /** MorphoWave sensor SDK recieved parameter with null value */
                13: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** MorphoWave service not exist in system */
                14: Sensor_types.Sensor_FOTF_service_not_exist_error err_service_not_exist
                /** Generic exception */
                15: Sensor_types.Sensor_error generic_xcept);

   /**
     * Store pairing data, stores host data 
     *
     */
    void store_pairing_data(/** host data */
                            1: binary host_data)       
        throws (/** sensor used by another process */
                1: Sensor_types.Sensor_in_use_error err_sensor_used
                /** sensor operation canceled  */
                2: Sensor_types.Sensor_cancel_error err_sensor_cancel
                /** parameter not set properly  */
                3: Sensor_types.Sensor_parameter_mismatch_error err_sensor_param_mismatch
                /** MorphoWave sensor not found  */
                4: Sensor_types.Sensor_device_not_found_error err_sensor_not_found
                /** MorphoWave sensor not found  */
                5: Sensor_types.Sensor_device_time_out_error err_sensor_time_out
                /** MorphoWave sensor power supply error  */
                6: Sensor_types.Sensor_power_supply_error err_sensor_power_supply
                /** MorphoWave sensor internal error  */
                7: Sensor_types.Sensor_internal_error err_sensor_internal
                /** Command not available error  */
                8: Sensor_types.Sensor_command_not_available_error err_sensor_command_not_available
                /** MorphoWave sensor license not available in system */
                9: Sensor_types.Sensor_license_missing_error err_sensor_license_missing
                /** MorphoWave sensor acquisition error */
                10: Sensor_types.Sensor_acquisition_error err_sensor_acquisition
                /** MorphoWave sensor acquisition status unknown error */
                11: Sensor_types.Sensor_acquisition_status_unknown_error err_status_unknown
                /** MorphoWave sensor security error */
                12: Sensor_types.Sensor_security_error err_sensor_security
                /** MorphoWave sensor SDK received parameter with null value */
                13: Sensor_types.Sensor_null_parameter_error err_sensor_null_param
                /** SSL certificate already exist in MorphoWave service */
                14: Sensor_types.Sensor_certificate_already_exist_error err_certificate
                /** SSL certificate not valid for MorphoWave service */
                15: Sensor_types.Sensor_certificate_not_valid_error err_certificate_not_valid
                /** SSL certificate dose not contains private key */
                16: Sensor_types.Sensor_certificate_private_key_not_available_error err_private_key
                /** Generic exception */
                17: Sensor_types.Sensor_error generic_xcept);
}
