/**
 * That file contains the commands used by MA5G SDK FAB specifically for MA5G products (not common to all products)
 */

// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2012 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Factory_commands.thrift
 *
 * CHANGELOG
 * 23 nov 2012  - Initiate header
 * 10 dec 2012  - Change namespace to comply MPH coding rules.
 * 11 dec 2012  - Change service name to comply MPH coding rules.
 * 19 dec 2012  - Put together Factory_sensor_read/write commands
 * 21 jan 2013  - Added factory_test_LCD_calibration command for testing touchscreen.
 * 30 jan 2013  - Changed parameter type of "factory_get_info", "factory_erase_info" and "MMI_set_leds".
 * 12 feb 2013  - Add Invalide parameter excpetion to MMI_set_leds command.
 *              - Add Invalide parameter excpetion to Factory get/set/erase info commands.
 *              - Add Invalide parameter excpetion to Factory sensor write/read parameter commands.
 *              - Remove Factory_sensor_read_camera_parameter and factory_sensor_write_camera_parameter.
 * 18 feb 2013  - Create Factory_read_external_device and Factory_write_external_device commands.
 * 22 feb 2013  - Add file not found exception to Factory_sensor_read_parameter commands.
 * 25 feb 2013  - Create Factory_get_GPI_state and Factory_set_GPO_state.
 * 08 mar 2013  - Add Remove Licence(s) commands.
 * 18 mar 2013  - Use generic timeout exception in factory_check_wiegand.
 * 29 mar 2013  - Add factory_read_microphone command.
 * 04 apr 2013  - Add factory_check_serial command.
 *              - Add factory_get_anti_intrusion_switch_state command.
 * 26 apr 2013  - Change parameter of "Factory_check_wiegand" command.
 * 29 may 2013  - Add factory_wiegand_LED_out_get_state and factory_wiegand_LED_in_set_state commands.
 * 03 jun 2013  - Add factory_COTS_get_version command.
 * 20 jun 2013  - Add factory_config_params_name_get_all command.
 * 24 jun 2013  - Add factory_read_fuse and factory_write_fuse commands.
 *              - Change output type and rename factory_config_parameter_name_get_all to factory_config_parameter_get_all.
 * 26 jun 2013  - Rename factory_set_GPO_state's parameter duration_in_sec to duration_in_ms.
 *              - Change factory_get_info and factory_erase_info's parameters to use factory_info_type enums.
 * 20 jul 2013  - Modified 'licence' to 'license' in commands for spell correction.
 * 30 jul 2013  - Change factory_check_serial return type and parameters.
 * 14 aug 2013  - Added Firmware upgrade command.
 * 09 oct 2013  - Change output type and parameter of factory_wiegand_LED_out_get_state command.
 * 18 oct 2013  - Add frame rate and audio format to factory_read_micro command.
 * 24 dec 2013  - Add factory_calibrate_LCD command.
 * 26 dec 2013  - Added exception 'Generic_error' in 'throws' section of every command
 * 10 jan 2014  - Added 'factory_retrieve_touchpoints' command.
 * 20 feb 2014  - Add namespace for java and C# generation  
 * 28 feb 2014  - Added 'factory_certificate_image_store' command
 * 24 mar 2014  - Added commands 'factory_test_MMI', and 'start_upgrade_via_USB'
 *              - Added product support notes  
 *              - Removed unused command 'factory_wiegand_LED_in_set_state'
 * 19 jun 2014  - Command 'MMI_set_leds' supported in MALITE/S
 * 01 jul 2014  - Removed Internal_types.thrift
 * 14 jul 2014  - Command start_upgrade_via_USB throws Generic_types.Invalid_argument_error
 * 29 jan 2015  - FACP Product support changes
 * 23 mar 2015  - Added command 'factory_clear_pairing_data' , removed factory_reset_MA5G_ID
 * 28 apr 2015  - Added exception for 'factory_clear_pairing_data' 
 * 04 nov 2015  - Change java namespace for coherency
 * 11 may 2016  - Remove command "factory_sensor_write_parameter"  , not be use in factory and new sensor manager will not handle 
 * 11 may 2016  - Remove command "factory_sensor_read_parameter"  , not be use in factory and new sensor manager will not handle 
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 25 Jul 2016  - Added 'factory_read_temperature' and 'factory_read_ambient_light_level' commands
 * 5  Aug 2016  - Update exception description for 'factory_read_temperature' and 'factory_read_ambient_light_level' commands
 * 07 Sep 2016 -  change return type for command 'factory_read_ambient_light_level'
 * 13 Sep 2016 -  change return type for command 'factory_read_temperature'
 * 25 Sep 2016  - Added 'factory_format_SD_card' command
 * 25 Sep 2016  - update 'factory_read_temperature' description
 * 15 nov 2016  - Added 'factory_open_factory_secure_channel' command
 * 24 Mar 2017  - PRGCA Product support added 
 * 20 Sept 2017  - Added 'simulation_files_load' command to Factory commands
 * 24 Oct 2017  - Added 'factory_read_sensor_temperature' command
 * 03 Jul 2018  - MA_VP_MD Product support added 
 */
// </doc_history>
 
include "Generic_types.thrift"
include "Factory_types.thrift"
include "QA_types.thrift"
include "Common_factory_commands.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd
 
service MA5G_factory_commands extends Common_factory_commands.Common_factory_commands
{
    /**
     * Erase factory information from flash memory for terminal or sensor board
     */
    void factory_erase_info(    /** Board for which information will be erased */
                                1: Factory_types.Factory_board_type board_type,
                                /** List of the factory parameter to erase */
                                2: set<Factory_types.Factory_info_type> parameter_list)
        throws (    /** Invalid parameter*/
                    1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);
    
    /**
     * Get GPIO state
     *
     * Return requested GPIO state
     */
    Factory_types.Factory_GPIO_state factory_get_GPIO_state(    /** Requested GPIO */
                                                                1: Factory_types.Factory_GPIO_index GPIO_index)
        throws (    /** Invalid parameter*/
                    1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Set GPO state
     *
     * Send state to requested GPO
     */
    void factory_set_GPO_state(    /** Requested GPO */
                                   1: Factory_types.Factory_GPIO_index GPO_index,
                                   /** State to send*/
                                   2: Factory_types.Factory_GPIO_state GPO_state,
                                   /** Duration of signal to send */
                                   3: i32 duration_in_ms)
        throws (    /** Invalid parameter*/
                    1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * Check wiegand and serial
     *
     * Return true if check wiegand work
     */
    bool factory_check_wiegand(    /** Time out */
                                     1: i32 timeout_in_sec)
        throws (    /** Time out */
                    1: Generic_types.Timeout_error timeout_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);
    
    /**
     * Remove a license
     */
    void license_remove(    /** License name */
                            1: string       license_name)
        throws( /** Invalid parameter (license doesn't exist) */
                1: Generic_types.Invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * Remove all licenses
     */
    void license_remove_all()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Check serial
     *
     * Send binary input parameter to serial output and return binary received from serial input
     */
    bool factory_check_serial(     /** Timeout */
                                   1: i32 timeout_in_sec)
        throws (    /** Invalid parameter*/
                    1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept,
                    /** Timeout */
                    2: Generic_types.Timeout_error timeout_xcept
                    /** Generic exception */
                    3: Generic_types.Generic_error generic_xcept);

    /**
     * Get anti intrusion switch state
     */
    Factory_types.Anti_intrusion_switch_state factory_get_anti_intrusion_switch_state()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept
        );

    /**
     * Get LED Wiegand state
     *
     *
     * Return requested LED Wiegand state
     */
    Factory_types.Factory_Wiegand_LED_state factory_wiegand_LED_out_get_state(  /** Timeout in second */
                                                                                1: i32 timeout_in_sec)
        throws (    /** Time out */
                    1: Generic_types.Timeout_error timeout_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);
                    
    /**
     * Get factory config all parameters
     *
     * Return all config parameters name and value instead of config_get_all_params_name which filter some parameter
     */
    map<string, string> factory_config_params_get_all()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);
     
    /**
     * <product>N;;N;;N;;N;;Y;;Y;;N</product>        
     * Read temperature level in milli degree celsius
     */
    i32 factory_read_temperature()
        throws (	/** Cannot read temperature. Unable to access temperature sensor file */
					1: Factory_types.Fac_read_file_error read_file_xcept
					/** Generic exception */
					2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     * Read sensors' temperature level in milli degree celsius
     */
    map<Factory_types.Factory_temperature_sensors, i32> factory_read_sensor_temperature(/** list of temperature sensors to read */
                                                                                        1: set<Factory_types.Factory_temperature_sensors> t_sensor)
        throws (       /** Cannot read temperature. Unable to access temperature sensor file */
                       1: Factory_types.Fac_read_file_error read_file_xcept
                       /** Invalid parameter */
                       2: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                       /** Generic exception */
                       3: Generic_types.Generic_error generic_xcept);
                    
    /**
     * <product>N;;N;;N;;N;;Y;;N;;N</product>    
     * Read ambient light level
     */
    i32 factory_read_ambient_light_level()
        throws (    /** Cannot read ambient light level. Unable to access light sensor file */
					1: Factory_types.Fac_read_file_error read_file_xcept
					/** Generic exception */
					2: Generic_types.Generic_error generic_xcept);                              
                    
    /**
     * <product>Y;;Y;;Y;;Y;;Y;;N;;Y</product>        
     * Write fuse
     *
     * Write requested fuse from iMX6 CPU.
     * WARNING All set to fuse is irreversible and run the risk to break definitely terminal's hardware
     */
    void factory_write_fuse(     /** list of fuse and value to write*/
                                 1: map<Factory_types.Factory_fuse_imx6, i32> secure_fuse)
        throws (    /** Invalid parameter*/
                    1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;N;;N;;Y;;N</product>        
     * Write fuse
     *
     * Write requested fuse from CPU.
     * WARNING All set to fuse is irreversible and run the risk to break definitely terminal's hardware
     */
    void factory_write_cpu_fuse(     /** list of fuse and value to write, first argument is name of fuse and second is value to write*/
                                 1: map<string, string> secure_fuse)
        throws (    /** Invalid parameter*/
                    1: Factory_types.Fac_invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);


    /**
     * Stores given certificate image file
     *
     * This function stores given certificate image of the various certification performed.
     */
    void factory_certificate_image_store(   /** Certificate image data */
                                            1: binary picture_data)
        throws( /** Invalid argument */
                1: Generic_types.Invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;Y;;Y;;N;;N;;N;;Y</product>
     * Test/play an MMI configuration.
     */
    void factory_test_MMI(  /** The MMI config structure to test/play */
                            1: Factory_types.MMI MMIs)
        throws (    /** Invalid argument */
                    1: Generic_types.Invalid_argument_error invalid_argument_xcept
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;Y;;Y;;Y;;Y</product>
     * Start the Firmware upgrade via USB.
     */
    void start_upgrade_via_USB( /** Path to the upgrade package */
                                1: string path_to_upgrade_pkg)
        throws (    /** Incompatible version*/
                    1: Factory_types.Fac_firmware_upgrade_incompatible_version_error incompatible_version_xcept,
                    /** Incompatible product*/
                    2: Factory_types.Fac_firmware_upgrade_incompatible_product_error incompatible_product_xcept,
                    /** Incompatible hardware revision */
                    3: Factory_types.Fac_firmware_upgrade_incompatible_hardware_revision_error incompatible_hardware_revision_xcept
                    /** Invalid parameter*/
                    4: Generic_types.Invalid_argument_error     invalid_argument_xcept
                    /** Generic exception */
                    5: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;Y;;N;;N;;N</product>
     * Reset MA5G terminal ID from MorphoWave PC configuration
     *
     */
    void factory_clear_pairing_data(/** To reset host data in terminal **/
                                    1:bool host_data,
                                    /** To reset Sensor Data **/ 
                                    2:bool sensor_data)
        throws (    /** error while clear host pairing data*/
                    1:Factory_types.Fac_clear_pairing_host_data_error host_pairing_data_xcept,
                    /** error while clear sensor pairing data*/
                    2:Factory_types.Fac_clear_pairing_sensor_data_error sensor_pairing_data_xcept,
                    /** Generic exception */
                    3: Generic_types.Generic_error generic_xcept);
    /**
     * <product>N;;N;;N;;N;;Y;;N;;N</product>    
     * Format SD card
     *
     * This command only deletes rootfs_data directory from SD card. Terminal reboot is mandatory after this command.
     */
    void factory_format_SD_card()
        throws (    /** SD card not found */
                    1: Generic_types.SD_card_not_found    sd_card_not_found_xcept,
                    /** Generic exception */
                    2: Generic_types.Generic_error generic_xcept);
    /**
     * Open Factory communication channel over SSL/TLS
     *
     * This command opens communication channel secured by SSL/TLS to serve Factory commands. Terminal reboot is mandatory to close the channel.
     */
    void factory_open_factory_secure_channel()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;N;;N;;Y;;Y</product>    
     * Reset life cycle counters
     */
    void reset_life_cycle_counters()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);
                    
        /**
     * <product>N;;N;;N;;N;;Y;;Y;;N</product>    
     * Loads files required for sensor simulation mode
     */

    void simulation_files_load( /** Simulation files to load */
                                    1: QA_types.Simulation_files files)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

}
