// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2012 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Internal_commands.thrift
 *
 * CHANGELOG
 * 23 nov 2012   - Initiate header
 * 10 dec 2012  - Change namespace to comply MPH coding rules.
 * 11 dec 2012  - Change service name to comply MPH coding rules.
 * 31 dec 2012  - Added command to perform complete authentication flow to be used by GUI application.
 * 18 jan 2013  - Added command to get terminal information.
 * 05 mar 2013  - Added command 'data_export_to_file' to export data
 * 07 mar 2013  - Removed command 'user_DB_export_to_file', 
 *                now user database can be exported through command 'data_export_to_file'
 * 14 mar 2013  - Added exception in command 'data_export_to_file', exception:'DB_invalid_passphrase_error'
 *              - Added exception in command 'user_DB_import_from_file', exception:'Invalid_directory_path_error'
 * 25 mar 2013  - Added command 'send_event'
 *              - Updated command 'data_export_to_file', added exception 'No_space_left_on_device'
 *              - Updated command 'user_DB_import_from_file', added exception 'Not_compatible_with_current_license' 
 *              - Added command 'search_user_ID_list'
 *              - Added command 'convert_binary_number_to_string' .
 * 05 apr 2013  - Updated command 'biofinger_complete_authentication', removed arguments exept user id
 * 18 apr 2013  - Added command 'set_fkey' for setting function key in FW for GUI during TNA operations. 
 * 07 may 2013  - Added command 'convert_templates' for template conversion for card encoding.
 * 22 may 2013  - Added command 'execute_USB_script' to execute ILV commands listed in file on USB drive.
 * 23 may 2013  - Removed command 'convert_binary_number_to_string'
 * 14 jun 2013  - Added command 'send_keypad_passthrough'
 * 19 jun 2013  - Added command 'external_db_set_id'
 * 23 jul 2013  - Added command 'check_license_of_module'
 * 27 jul 2013  - Added command 'cls_keys_update'
 * 02 aug 2013  - Included Internal_types.thrift file
 * 05 aug 2013  - Renamed command 'search_user_list' now searchs user by ID, first name or last name 
 * 06 aug 2013  - Changed return value of 'search_user_list' to return list of users 
 * 06 aug 2013  - Added command 'set_sensor_low_consumption_state' 
 * 06 aug 2013  - Added exception 'Invalid_directory_path_error' in command 'cls_keys_update'
 * 29 aug 2013  - Updated command 'cls_keys_update', added argument 'filepath_UTF8' and 
 *                added exeption 'Invalid_file'
 * 29 aug 2013  - Updated command 'data_export_to_file', added argument 'key_type'
 * 29 aug 2013  - Updated command 'data_export_to_file', removed argument 'key_type'
 * 05 sep 2013  - Added command 'cls_reset'
 * 26 dec 2013  - Added exception 'Generic_error' in 'throws' section of every command
 * 20 feb 2014  - Add namespace for java and C# generation  
 * 24 mar 2014  - Added product support notes
 * 30 jun 2014  - Added command MMI_set_configs/MMI_get_configs
 * 03 jul 2014  - Edited command MMI_set_configs/MMI_get_configs, used Factory types
 * 11 jul 2014  - command data_export_to_file throws Generic_types.Invalid_argument_error
 * 15 sep 2014  - command set_fkey throws Generic_types.Invalid_argument_error
 * 29 jan 2015  - MorphoWave Product support changes   
 * 11 feb 2015  - Command set_sensor_low_consumption_state now supported in MorphoWave Product support
                - Added command get_user_details
 * 04 mar 2015  - Changed return type of 'get_user_details'
 * 08 jun 2015  - Added command 'config_get_param_details
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 07 Sep  2016 - Added command set_video_play_state
 * 28 Nov  2016 - Rename command 'set_video_play_state'  to 'set_video_play_auto_brightness_state'
 * 24 Mar 2017  - PRGCA Product support added
 * 22 June 2017 - Added exception 'Invalid_argument_error' in command 'cls_keys_update'
 * 24 July 2017 - Move 'Invalid_argument_error' in command 'cls_keys_update' exception at the end of the existing ones for backward compatibility. 
 * 31 Oct 2017  - Remove support for PRGCA product 
 * 19 jan 2018  - Added command 'set_sensor_low_consumption_mode'
 * 02 Jul 2018  - MA_VP_MD Product support added 
 * 5 July 2018  - Added command 'play_touch_sound'
 * 28 Sept 2018  - Added command 'convert_templates_advanced'
 */
// </doc_history>

include "MA5G_generic_commands.thrift"
include "Generic_types.thrift"
include "Config_types.thrift"
include "Biofinger_types.thrift"
include "Db_types.thrift"
include "Cls_types.thrift"
include "Internal_types.thrift"
include "Factory_types.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

/*
 * User control trigger
 */
typedef Db_types.User_rule_trigger_check User_control_trigger

service MA5G_internal_commands extends MA5G_generic_commands.MA5G_generic_commands
{
    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Exports the data to file
     *
     *
     * This command export data files in specified path.<br>
     *
     * While exporting the user database, this command ciphers all the records of the database using the specified passphrase and write them in specific filepath.<br>
     * Those records can only be used by @a user_DB_import_from_file command.
     * Warning: Modifying the database during an exportation will not corrupt the exported result, but the modification is not guaranteed
     *          to be present in the exported data.
     */
    void data_export_to_file(   /** Data export options */
                                1: Internal_types.Data_export_options export_option
                            )
                    throws (                                
                                /** Invalid directory path */
                                1: Internal_types.Invalid_directory_path_error  invalid_dir_ex,
                                /** The passphrase does not match.*/
                                2: Internal_types.DB_invalid_passphrase_error invalid_passphrase_xcept,
                                /** No space left on device to export */
                                3: Internal_types.No_space_left_on_device no_space_left_xcept
                                /** Generic exception */
                                4: Generic_types.Generic_error generic_xcept
                                /** Invalid parameter*/
                                5: Generic_types.Invalid_argument_error     invalid_argument_xcept
                            );

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Imports ciphered database records
     *
     *
     * This command un-cipher and store all records contained in a file in the terminal database. The ciphered records shall have been
     * obtained using command DB_export and with the same passphrase.
     *
     *
     * Note: This command will clear the database before importing the new records.
     *
     * <b>Warning</b>: Modifying the database during an importation will not corrupt the database, but the modification may be lost.
     */
    void user_DB_import_from_file(  /**
                                     * Passphrase used to un-cipher the database records. You shall use the same passphrase that have
                                     * been specified for the export.
                                     */
                                    1: string   passphrase_UTF8,
                                    /** Filepath where to obtain the ciphered database records*/
                                    2: string   filepath_UTF8)
                throws  (                            
                            /** The passphrase does not match.*/
                            1: Internal_types.DB_invalid_passphrase_error invalid_passphrase_xcept,
                            /** Invalid path */
                            2: Internal_types.Invalid_directory_path_error  invalid_dir_ex,
                            /** Invalid path */
                            3: Internal_types.Not_compatible_with_current_license license_xcept
                            /** Generic exception */
                            4: Generic_types.Generic_error generic_xcept
                        );

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Authenticates an user using its finger against its database references
     *
     *
     * The reference templates are stored in one (and only one) specified record.<br>
     * The search template is compared to all the templates found in the specified record
     * (the number of fingerprint templates depends on the database format).
     *
     *
     * <b>Warning</b>: If enable_intermediate_replies is true, you will need a modified version of the Thrift client that supports
     *          the reception of several T_REPLY messages for one command.
     */
    Biofinger_types.Biofinger_control_operation_reply biofinger_complete_authentication(/** User ID to authenticate (UTF8 string)*/
                                                                                    1: string user_id_UTF8)
            throws (                        
                        /** The specified user ID has not been found in the database.*/
                        1: Generic_types.Inexistent_user_id_error   inexistant_user_xcept
                        /** The user database is empty.*/
                        2: Db_types.DB_empty_error                  DB_empty_xcept,
                        /** No finger was detected during process*/
                        3: Generic_types.Timeout_error              timeout_xcept,
                        /** Invalid parameter*/
                        4: Generic_types.Invalid_argument_error     invalid_argument_xcept,
                        /** Command is cancelled by distant command*/
                        5: Generic_types.Cancelled_error            cancelled_xcept,
                        /** Finger was misplaced or withdrawn during acquisition*/
                        6: Biofinger_types.Misplaced_finger_error   misplaced_finger_xcept
                        /** Generic exception */
                        7: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * This will be called when user logs into management menu
     */
    void login_management_menu()
        throws (
                /** Generic exception */
                1: Generic_types.Generic_error generic_xcept
        );

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * This will be called when user logs out from management menu
     */
    void logout_management_menu()
        throws (
                /** Generic exception */
                1: Generic_types.Generic_error generic_xcept
        );

    /**
     * This command returns terminal information structure.
     */
    Internal_types.Terminal_info terminal_get_info()
        throws (
                /** Generic exception */
                1: Generic_types.Generic_error generic_xcept
        );

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     */
    void send_event(  /**
                       * Transaction log database fields.
                       */
                       1: Db_types.Transaction_log_DB_record   tlog_data)
                throws (                            
                            /** Invalid argument exception.*/
                            1: Generic_types.Invalid_argument_error invalid_argument_xcept
                            /** Generic exception */
                            2: Generic_types.Generic_error generic_xcept
                        );

    /**
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     * Command to retrieve the list of user searched by ID prefix or first name prefix or last name .<br>
     * Return a list of the user ID or First name or last name list starts with given prefix.<br>
     * If there is no user found in DB with provided prefix then string list will be empty.
     */
    list<Db_types.User_DB_record> search_user_list( /** Prefix ID from onwards list will be returned */
                                                    1: string   prefix_ID,
                                                    /** Prefix of First name */
                                                    2: string   prefix_first_name,
                                                    /** Prefix of Last name */
                                                    3: string   prefix_last_name)
            throws (    /** Generic exception */
                        1: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;Y;;Y;;Y;;Y</product>
     * Command to search and retrieve user record list based on the User_data_fields given.<br>
     * Return a List of the user with all information .<br>
     */
    Internal_types.User_Information get_user_details(   /** Map of User_data_fields and string to search*/
                                                        1: map<Internal_types.User_data_fields, string> search_criteria,
                                                        /** Sort by ascending or descending*/
                                                        2:Internal_types.User_data_order sort_order)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept
                /** Invalid argument exception.*/
                2: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Command to set F-Key for Time and attendance.
     **/
    void set_fkey(  /** F-Key value */
                    1: Internal_types.f_key_type f_key)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept
                /** Invalid argument exception.*/
                2: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     * Convert templates
     *
     * This command converts the list of input templates to specified template type and returns the 
     * corresponding list of converted templates.<br>     
     *
     * Return a list of the templates converted to specified type.    
     */
    list<binary> convert_templates( /** List of the input templates */
                                    1: list<binary> input_templates,
                                    /** Template type of input templates */
                                    2: Biofinger_types.Biofinger_template_type input_template_type,
                                    /** Template type of output templates */
                                    3: Biofinger_types.Biofinger_template_type output_template_type)
        throws (/** Invalid parameter*/
                1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                /** No space left on device */
                2: Internal_types.No_space_left_on_device no_space_left_xcept
                /** Generic exception */
                3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y</product>
     * Convert templates Advanced
     *
     * This command converts the list of input templates to specified template type and returns the 
     * corresponding list of converted templates.<br>
     *
     * Return a list of the templates converted to specified type.    
     */
    list<binary> convert_templates_advanced( /** Template information fields */
                                    1: Internal_types.Convert_template_info template_info
                                    )
        throws (/** Invalid parameter*/
                1: Generic_types.Invalid_argument_error invalid_argument_xcept,
                /** No space left on device */
                2: Internal_types.No_space_left_on_device no_space_left_xcept
                /** Generic exception */
                3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;N;;N;;N;;Y;;Y;;N</product>
     * Run ILV commands listed in file on USB key.
     *
     * This command will run ILV commands listed in file on USB key and generate response for the same.
     */
    void execute_USB_script()
        throws (/** Invalid directory path */
                1: Internal_types.Invalid_directory_path_error  invalid_dir_xcept,
                /** Invalid file.*/
                2: Internal_types.Invalid_file invalid_file_xcept,
                /** No space left on USB device to write response in file */
                3: Internal_types.No_space_left_on_device no_space_left_xcept
                /** Generic exception */
                4: Generic_types.Generic_error generic_xcept);

   /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Keypad passthrough command to get key pressed from GUI
     * <ol>
     * <li> 0 to 9 for respective keys
     * <li> 10 for *(clear)
     * <li> 11 for #(enter)
     * <li> -1 for termination
     * </ol>
     */
    void send_keypad_passthrough(   /** Value of the key pressed */
                                    1: i32 key_value)
        throws (/** Invalid parameter*/
                1: Generic_types.Invalid_argument_error     invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Get user control trigger events 
     * Gets as per current Threat Level settings
     **/
    User_control_trigger ucc_get_trigger_events()
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * Polling commands
     **************************************************/
     
    /**
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     * Set id for external database verification.
     */
    void external_db_set_id(/** ID for external database */
                            1: string id_data)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * License command
     **************************************************/
     
    /**
     * This command will be used to check installed licenses in terminal
     */
    map<Internal_types.Module_license_code, Internal_types.Module_license_status> check_license_of_module(  /** Module id */
                                                                                                            1: set<Internal_types.Module_license_code> param_license_code)
        throws (/** Invalid argument */
                1: Generic_types.Invalid_argument_error invalid_argument_xcept
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**************************************************
     * cls command
     **************************************************/

    /**
     * <product>Y;;N;;N;;N;;Y;;Y;;N</product>
     * Command to update contactless keys
     * 
     * This command will be used to generate crypto keys and to import crypto keys from USB
     * based on provided option
     */
    void  cls_keys_update(  /** card type */
                            1: Cls_types.Cls_card_type  card_type,
                            /** Keys update type */
                            2: Internal_types.Cls_keys_update_type  update_type,
                            /** Passphrase */
                            3: string passphrase
                            /** File path to import */
                            4: string filepath_UTF8)
        throws( /** Invalid path */
                1: Internal_types.Invalid_directory_path_error  invalid_dir_ex,
                /** Invalid file */
                2: Internal_types.Invalid_file invalid_file_ex,
                /** Invalid argument */
                3: Generic_types.Generic_error generic_xcept,
                /** Generic exception */
                4: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * <product>Y;;Y;;N;;Y;;Y;;N;;N</product>
     * Command to set sensor low consumption state
     */
    void  set_sensor_low_consumption_state( /** Sensor state */
                                            1: bool disable_sensor)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
    
	/**
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     * Command to set sensor low consumption mode
     */
    void  set_sensor_low_consumption_mode( /** Sensor mode */
                                            1: Internal_types.Sensor_mode sensor_mode)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**

     * <product>Y;;N;;N;;N;;Y;;Y;;N</product>
     * Command to reset the smartcard
     */
    void  cls_reset()
        throws (/** No smartcard detected*/
                1: Generic_types.Timeout_error timeout_xcept,
                /** Invalid card detected*/
                2: Generic_types.Invalid_argument_error invalid_argument_xcept,
                /** Generic exception */
                3: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;Y;;Y;;N;;N;;N;;Y</product>
     * Set one or more MMI configurations.
     */
    void MMI_set_configs(   /** Mapping of MMI IDs to their config structure fields*/
                            1: map<Factory_types.MMI_ID, Factory_types.MMI> MMIs)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept,
                /** Invalid argument */
                2: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * <product>N;;Y;;Y;;N;;N;;N;;Y</product>
     * Return a map of MMI IDs and their corresponding configuration.<br>
     */
    map<Factory_types.MMI_ID, Factory_types.MMI> MMI_get_configs(   /** List of MMI IDs to return*/
                                                                    1: set<Factory_types.MMI_ID> MMI_IDs)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept,
                /** Invalid argument */
                2: Generic_types.Invalid_argument_error invalid_argument_xcept);

    /**
     * <product>Y;;Y;;Y;;Y;;Y;;Y;;Y</product>
     * Return a list of details corresponding to requested parameters.
     */
    list<Internal_types.Param_details> config_get_param_details(/** List of UTF8 string representing names of requested parameters */
                                                                1: list<string> param_list_UTF8)
        throws (/** A requested parameter does not exist in terminal */
                1: Config_types.Config_inexistent_parameter_error inexistant_param_xcept,
                /** Generic exception */
                2: Generic_types.Generic_error generic_xcept);

    /**
     * <product>N;;N;;N;;N;;Y;;N;;N</product>
     * Command to set video play brightness state to auto (false : Auto brightness configuration can be applied) or manual (true : Auto brightness configuration will not applied)
     */
    void  set_video_play_auto_brightness_state( /**Auto brightness configuration during video play */
						1: bool video_play_brightness_status)
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);
				
    /**
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     * Perform biometric capture with live display on local UI.
     */
    void  bio_capture()
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);                
                
    /**
     * <product>N;;Y;;N;;N;;N;;N</product>
     * Command to play beep sound on LCD menu/button touch.
     */
    void  play_touch_sound()
        throws (/** Generic exception */
                1: Generic_types.Generic_error generic_xcept);                
                
}
