
include "Biofinger_types.thrift"
include "Generic_types.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd


/**
 * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
 * Jobcode list datatype
 *
 * This will be used for the job code list load/retrieve operation
 */
struct Jobcode_list
{
    /** List number (1 to 128) */
    1: i16 list_number;
    /** List name, max name length is 128 */
    2: string list_name;
    /** Array of Job code values (0 to (2^32-1)), value is unsigned but it is declared as singed in command due to thrift data type limitation */
    3: optional list<i32> jobcode_values;
}

/** 
 * <ps_fields>
 * <ps>tlac_duress_finger_detected;;Y;;Y;;Y;;N;;Y;;N;;Y</ps> 
 * <ps>tlac_pin_mismatch;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>tlac_black_listed_card;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps> 
 * <ps>tlac_face_not_detected;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>tlac_job_code_check_failure;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>tlac_management_menu_login;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>tlac_management_menu_logout;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>tlac_cls_card_encoded;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>tlac_cls_card_reset;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>tlac_cls_card_security_key_reset;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>tlac_usr_timed_anti_passback;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>tlac_triggers_blocked;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>tlac_triggers_back_to_normal;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>tlac_sensor_recovery_start;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>tlac_sensor_recovery_end;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>tlac_scratched_sensor;;N;;N;;N;;N;;N;;Y;;N</ps> 
 * <ps>tlac_dirty_sensor;;N;;N;;N;;N;;N;;Y;;N</ps> 
 * <ps>tlac_tamper_suspended;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * <ps>tlac_tamper_back_to_monitor;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * <ps>tlac_protected_data_cleared_on_corruption;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>tlac_invalid_qr_code;;N;;N;;N;;N;;N;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration for transaction log action code
 */
 
enum Transaction_log_action_code
{
    tlac_duress_finger_detected,
    tlac_fake_finger_detected,
    tlac_user_control_successful,
    tlac_biometric_mismatch,
    tlac_pin_mismatch,
    tlac_user_id_not_in_db,
    tlac_control_timed_out,
    tlac_rejected_by_schedule,
    tlac_temp_validity_expired,
    tlac_user_not_in_white_list,
    tlac_black_listed_card,
    tlac_face_not_detected,
    tlac_transaction_log_full,
    tlac_controller_feedback_action,
    tlac_job_code_check_failure,
    tlac_user_rule_check_failure,
    tlac_door_opened_for_too_long,
    tlac_door_forced_open,
    tlac_door_closed_after_alarm,
    tlac_door_unlocked,
    tlac_door_locked_back,
    tlac_management_menu_login,
    tlac_management_menu_logout,
    tlac_database_deleted,
    tlac_enrollment_completed,
    tlac_user_deleted,
    tlac_user_modification_completed,
    tlac_cls_card_encoded,
    tlac_cls_card_reset,
    tlac_settings_changed,
    tlac_cls_card_security_key_reset,
    tlac_security_policy_changed,
    tlac_tamper_detected,
    tlac_tamper_cleared,
    tlac_terminal_boot_completed,
    tlac_firmware_upgrade,
    tlac_add_user,
    tlac_reboot_initiated,
    tlac_multi_user_intermediate_id,
    tlac_triggers_blocked,
    tlac_triggers_back_to_normal,
    tlac_usr_timed_anti_passback,
    tlac_sensor_recovery_start,
    tlac_sensor_recovery_end,
    tlac_scratched_sensor,
    tlac_dirty_sensor,
    tlac_tamper_suspended,
    tlac_tamper_back_to_monitor,
    tlac_protected_data_cleared_on_corruption,
    tlac_invalid_qr_code,
    reserved7,
    reserved8,
    reserved9,
    reserved10,
    reserved11,
    reserved12,
    tlac_max_code,
}

struct DB_status
{
    /** Maximum number of records.*/
    1: i32      capacity;
    /** Current number of records.*/
    2: i32      size;
}

struct Transaction_log_status
{
    /** Log status */
    1: DB_status log_status;
}

/** Enumeration representing all the action status */
enum Transaction_log_action_status
{
    action_fail,
    action_pass
    // More to be defined here
}

/** Enumeration representing the read status */
enum Transaction_log_read_status
{
    all_logs,
    unread_logs,
    read_logs
    // More to be defined here
}

/**
* GPS information structure
* <product>N;;N;;N;;N;;N;;N;;N</product>
*/

struct GPS_info
{
    /** latitude information,  format of data is in Degrees, Minutes, Seconds [N,S]dd°mm'ss.ssss" e.g. N40°45'36" */
    1: required string      latitude;
    /** longitude information, format of data is in Degrees, Minutes, Seconds [W,E]dd°mm'ss.ssss" e.g. W73°59'2.4"*/
    2: required string      longitude;
    /** altitude information format of data is in meter MMMMM.m e.g. 8809.6 */
    3: required string      altitude;
}

/** 
 * <ps_fields>
 * <ps>jobcode;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>tna_key;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>photo;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>user_role;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>usr_ctrl_ref;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps> 
 * <ps>gps_data;;N;;N;;N;;N;;N;;N;;N</ps> 
 * </ps_fields>
 *
 * <table_fields>
 * <tb>action_data;;Y;;N</tb>
 * <tb>duration;;Y;;N</tb>
 * <tb>usr_ctrl_ref;;Y;;N</tb>
 * <tb>usr_ctrl_checks_to_do;;Y;;N</tb>
 * <tb>usr_ctrl_checks_done;;Y;;N</tb>
 * <tb>unique_id;;N;;Y</tb>
 * </table_fields>
 * Enumeration representing all the fields contained in a transaction log record.
 */
enum Transaction_log_DB_fields
{
    action_status,
    date_time,
    name,
    first_name,
    channel,
    action_data,
    action,
    userid_csn,
    jobcode,
    duration,
    matched_finger,
    tna_key,
    photo,
    error_code,
    matching_score,
    user_role,
    usr_ctrl_ref,
    usr_ctrl_checks_to_do,
    usr_ctrl_checks_done,
    /** GPS information specific for MIMS */
    gps_data,
    unique_id
}

struct Usr_ctrl_ref 
{
    /** Control mode reference */
    1: required bool ctrl_mode_ref_in_DB;
    /** user record reference */
    2: required bool user_record_ref_in_DB;
    /** Terminal ID check reference */
    3: required bool id_check_ref_in_DB;
    /** Smartcard ID check reference */
    4: required bool id_check_ref_on_SC;
    /** Finger data reference */
    5: required bool finger_ref_in_DB;
    /** PIN data reference
     * <product>Y;;Y;;N;;N;;Y;;Y;;N</product>
     */
    6: optional bool pin_ref_in_DB;
    /** Expiry date reference */
    7: required bool expiry_date_ref_in_DB;
    /** Access schedule data reference */
    8: required bool access_schedule_ref_in_DB;
    /** Holiday schedule data reference */
    9: required bool holiday_schedule_ref_in_DB;
}

struct Usr_ctrl_check 
{
     1: required bool id_check_DB;
     /** <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product> */
     2: optional bool id_check_SC;
     3: required bool bio_check;
     /** <product>Y;;Y;;N;;Y;;Y;;Y;;N</product> */
     4: optional bool pin_check;
     5: required bool usr_rule_check;
     /** <product>Y;;Y;;N;;N;;Y;;Y;;N</product> */
     6: optional bool biopin_check;
     /** <product>Y;;N;;N;;Y;;Y;;N;;N</product> */
     7: optional bool face_detection_check;
     /** <product>Y;;N;;N;;Y;;Y;;N;;N</product> */
     8: optional bool face_detection_mandatory;
     /** <product>Y;;N;;N;;Y;;Y;;N;;N</product> */
     9: optional bool face_detection_photo_check;
    10: required bool white_list_check;
    11: required bool VIP_list_check;
    /** <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product> */
    12: required bool stolen_SC_list_check;
    13: required bool multi_usr_check;
    /** <product>Y;;Y;;Y;;N;;Y;;N;;N</product> */
    14: required bool duress_check;
    15: required bool access_schedule_check;
    16: required bool holiday_schedule_check;
    17: required bool expiry_date_check;
    /** <product>Y;;N;;N;;Y;;Y;;Y;;N</product> */
    18: optional bool tna_ext_mode_check;
    /** <product>Y;;N;;N;;Y;;Y;;Y;;N</product> */
    19: optional bool job_code_list_check;
    /** <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product> */
    20: optional bool timed_anti_passback_check;
}

struct Transaction_log_DB_record
{
    /** User control status */
    1: optional Transaction_log_action_status action_status;
    /** Transaction time stamp */
    2: optional Generic_types.Date_time date_time;
    /** Name */
    3: optional string name;
    /** First name */
    4: optional string first_name;
    /** Channel */
    5: optional byte channel;
    /** Action data (Action specific data) values */
    6: optional binary action_data;
    /** Action */
    7: optional Transaction_log_action_code action;
    /** User ID or CSN */
    8: optional string userid_csn;
    /** Jobcode
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     */
    9: optional i32 jobcode;
    /** Duration */
    10: optional i16 duration;
    /** Matched finger index of user */
    11: optional byte matched_finger;
    /** Time and Attendance key pressed by user (valid value 0(No Key) to 16(pressed key) )
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     */
    12: optional byte tna_key;
    /** Photo data of user
     * <product>Y;;N;;N;;Y;;Y;;N;;N</product>
     */
    13: optional binary photo;
    /** Error code */
    14: optional i32 error_code;
    /** Matching score */
    15: optional i64 matching_score;
    /** User role
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     */
    16: optional byte user_role;
    /** Referenced used during user control 
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     */
    17: optional Usr_ctrl_ref usr_ctrl_ref_used;
    /** Controls to do during user control */
    18: optional Usr_ctrl_check usr_ctrl_checks_to_do;
    /** Controls performed during user control */
    19: optional Usr_ctrl_check usr_ctrl_checks_done;
    /**  Flag indicating whether the response is final or intermediate */
    20: optional bool is_final_response = false;
    /**  GPS data (specific for MIMS) 
     * <product>N;;N;;N;;N;;N;;N;;N</product>
     */
    21: optional GPS_info gps_data;
    /** Transaction log Unique ID : unique ID on 64 bits incremented for each transaction log, reset only in case of reset factory of the transaction logs */
    22: optional string unique_id;
}

/** 
 * <ps_fields>
 * <ps>photo_status;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * </ps_fields>
 *
 * Enumeration representing all the filters available for retrieving transaction logs.
 */
enum Transaction_log_filter_type
{
    all,
    action_status,
    date_time,
    user_id,
    log_action,
    photo_status,
    unique_id
}

struct Transaction_log_retrieve_stats
{
    /** Offset from where log is required */
    1: optional i32 offset;
    /** Number of logs required from above offset */
    2: optional i32 num_tlogs;
}

enum Order_by
{
    asc,
    desc,
}

struct Transaction_log_sort_by_field
{
    /** Field to sort */
    1: Transaction_log_DB_fields sort_field;
    /** Sort by ascending or descending */
    2: Order_by order;
}

struct Transaction_log_filter
{
    /** Filter type required */
    1: required Transaction_log_filter_type filter_type;
    /** User control status */
    2: optional Transaction_log_action_status action_status;
    /** Start time stamp */
    3: optional Generic_types.Date_time start_time_stamp;
    /** End time stamp */
    4: optional Generic_types.Date_time end_time_stamp;
    /** User ID */
    5: optional string user_id;
    /** Action */
    6: optional Transaction_log_action_code log_action;
    /** Photo status
     * <product>Y;;N;;N;;Y;;Y;;N;;N</product>
     */
    7: optional bool photo_status;
    /** Transaction log retrieve stats (for paging) */
    8: optional Transaction_log_retrieve_stats tlog_stats;
    /** Sort order if required */
    9: optional Transaction_log_sort_by_field sort_field;
    /** Start Unique ID - For Identified Transaction log format only*/
    10: optional string start_unique_id;
    /** End Unique ID - For Identified Transaction log format only*/
    11: optional string end_unique_id;
    /** Transaction log read status : Additional filter type to provide filter based on Transaction log read status*/
    12: optional Transaction_log_read_status read_status;
}

/**
 * Enumeration representing the type of users in database.
 *
 * See user_DB_get_user_IDs command.
 */
enum User_type
{
    enrolled,
    white_list,
    VIP_list,
    all,
 }

/**
 * <ps_fields>
 * <ps>first_finger_qual;;Y;;Y;;Y;;Y;;Y;;N;;Y</ps> 
 * <ps>second_finger_qual;;Y;;Y;;Y;;Y;;Y;;N;;Y</ps> 
 * <ps>third_finger_qual;;Y;;Y;;Y;;Y;;Y;;N;;N</ps> 
 * <ps>first_finger_nb;;Y;;Y;;Y;;Y;;Y;;N;;Y</ps> 
 * <ps>second_finger_nb;;Y;;Y;;Y;;Y;;Y;;N;;Y</ps> 
 * <ps>third_finger_nb;;Y;;Y;;Y;;Y;;Y;;N;;N</ps> 
 * <ps>duress_finger_index;;Y;;Y;;Y;;N;;Y;;N;;N</ps> 
 * <ps>PIN_code;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>user_card_sn;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>job_code_list;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>legacy_timemask;;Y;;Y;;Y;;N;;Y;;N;;Y</ps> 
 * <ps>legacy_extended_id;;Y;;Y;;Y;;N;;N;;N;;Y</ps> 
 * <ps>user_role;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>photo;;N;;N;;N;;N;;N;;N;;N</ps>
 * </ps_fields>
 *
 * Enumeration representing all the fields contained in a user record.
 */
enum User_DB_fields
{
    name,
    first_name,
    /**
     * <note> templates field is updated to include list template(finger as of now) quality, template number.<br>
     * Separate fields of finger number (X_finger_nb), quality (X_finger_qual) is still supported for backward compatibility of three finger use case.<br>
     * If quality and number are present in templates structure then individual fields first_finger_qual, second_finger_qual, third_finger_qual, <br>
     * first_finger_nb, second_finger_nb, third_finger_nb will not be used. </note>
     */
    templates,
    /**
     * <note> See comments of templates field. </note>
     */
    first_finger_qual,
    /**
     * <note> See comments of templates field. </note>
     */
    second_finger_qual,
    /**
     * <note> See comments of templates field. </note>
     */
    third_finger_qual,
    /**
     * <note> See comments of templates field. </note>
     */
    first_finger_nb,
    /**
     * <note> See comments of templates field. </note>
     */
    second_finger_nb,
    /**
     * <note> See comments of templates field. </note>
     */
    third_finger_nb,
    duress_finger_index,
    schedule_nb,
    PIN_code,
    user_card_sn,
    job_code_list,
    apply_holiday_schedule,
    specific_relay_duration_in_sec,
    expiry_date,
    user_rule,
    white_list_flag,
    legacy_timemask,
    legacy_extended_id,
    user_role,
    VIP_list_flag,
    additional_data,
    /**
     * Photo field specific for MIMS
     */
    photo
}

/** The requested field does not exist in user database.*/
exception User_DB_unavailable_field
{
    1: required Generic_types.Generic_error_code    err_code,
    2: required User_DB_fields                      missing_fields
}

/**
 * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
 */
enum Dynamic_msg_type
{
    dynamic_msg_none,
    dynamic_msg_text,
    dynamic_msg_image
}

/**
 * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
 */
struct Dynamic_msg_DB_record
{
    1: Generic_types.Date_time start_date; ///< to hold dynamic message start date
    2: Generic_types.Date_time end_date; ///< to hold dynamic message end date
    3: string message_or_filename; ///< to hold dynamic message string
    4: Dynamic_msg_type type; ///< to hold dynamic message type
    5: bool attach_flag; ///< to hold dynamic message attach flag
}

/**
 * <product>Y;;N;;N;;Y;;Y;;N;;N</product>
 */
enum User_access_face_auth_rule
{
    disabled,
    photo_taking,
    face_detection_optional,
    face_detection_mandatory,
}

struct User_rule_control_check
{
    /** PIN control <br> if enabled PIN control is required for user
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product> */
    1: optional bool PIN_control;

    /** Finger bio reference check */
    2: required bool finger_bio_control;

    /** user face auth rule control
      * <product>Y;;N;;N;;Y;;Y;;N;;N</product> */
    3: optional User_access_face_auth_rule user_face_auth_rule_control;

    /** Job code control check
     * <product>Y;;N;;N;;N;;Y;;N;;N</product> */
    4: optional bool job_code_control;
}

struct User_rule_reference_check
{
    /** Terminal reference check <br> if enabled reference for user must be terminal */
    1: required bool terminal;

    /** Smartcard reference check */
    2: required bool smart_card;
}

struct User_rule_trigger_check
{
    /** Finger bio trigger check<br> if enabled Finger BIO trigger is valid source of trigger for user */
    1: required bool finger_bio;

    /** Contactless card trigger check 
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     */
    2: optional bool contactless;

    /**
     * keyboard trigger check
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product> */
    3: optional bool keyboard;

    /** external_port trigger check */
    4: required bool external_port;

    /** QR_code trigger check */
    5: optional bool QR_code;
}

struct User_access_rule
{
    /** Trigger check parameter */
    1: required User_rule_trigger_check trigger_check;

    /** reference check parameter 
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     */
    2: optional User_rule_reference_check reference_check;

    /** Control check parameter */
    3: required User_rule_control_check control_check;

    /**
     * BIO substitution
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     */
    4: optional bool allow_bio_substitution;
}
 
struct User_DB_record
{
    /** User name */
    1: optional string                  name_UTF8;
    /** First name */
    2: optional string                  first_name_UTF8;
    /**
     * Non compressed templates.<br>
     * Max number of templates per user is 8 for MorphoWave terminal and 3 for other MorphoAccess terminals.
     */
    3: optional list<Biofinger_types.User_templates>    templates;
    /** 
     * Quality of the first enrolled finger. <br>
     * <note> Not used if quality field is found in corresponding User_templates structure. </note>
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    4: optional i16                                     first_finger_qual;
    /** 
     * Quality of the second enrolled finger. <br>
     * <note> Not used if quality field is found in corresponding User_templates structure. </note>
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    5: optional i16                                     second_finger_qual;
    /** 
     * Quality of the third enrolled finger. <br>
     * <note> Not used if quality field is found in corresponding User_templates structure. </note>
     * <product>Y;;Y;;Y;;N;;Y;;N;;N</product>
     */
    6: optional i16                                     third_finger_qual;
    /**
     * Number that represents the first finger enrolled.<br>
     * 1 for left little finger, up to 10 for right little finger.<br>
     * <note> Not used if number field is found in corresponding User_templates structure. </note>
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    7: optional byte                    first_finger_nb;
    /**
     * Number that represents the second finger enrolled.<br>
     * 1 for left little finger, up to 10 for right little finger.<br>
     * <note> Not used if number field is found in corresponding User_templates structure. </note>
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    8: optional byte                    second_finger_nb;
    /**
     * Number that represents the third finger enrolled.<br>
     * 1 for left little finger, up to 10 for right little finger.<br>
     * <note> Not used if number field is found in corresponding User_templates structure. </note>
     * <product>Y;;Y;;Y;;N;;Y;;N;;N</product>
     */
    9: optional byte                    third_finger_nb;
    /**
     * Index of the duress finger among the enrolled finger.</be>
     * From 1 to 3.
     * <product>Y;;Y;;Y;;N;;Y;;N;;N</product>
     */
    10: optional byte                   duress_finger_index;
    /** Schedule number that applies to the user.*/
    11: optional byte                   schedule_nb;
    /** UTF8 string that represents user PIN code.
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     */
    12: optional string                 PIN_code_UTF8;
    /** Card Serial Number (CSN) of the user card.
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    13: optional binary                 user_card_sn;
    /** Job code list associated to a user.
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     */
    14: optional i16                   job_code_list;
    /** Flag to indicate if holiday schedule apply to the user.*/
    15: optional bool                   apply_holiday_schedule;
    /**
     * Specific duration for relay triggering.<br>
     * 0 for N/A (i.e. used duration from terminal configuration), else duration in seconds (from 1 to 3600 seconds).
     */
    16: optional i16                    specific_relay_duration_in_sec;
    /**
     * User expiration data.<br>
     * If according feature is enabled, after that date the user will be rejected if control is successful.
     */
    17: optional Generic_types.Date_time    expiry_date;
    /** User specific access rule.*/
    18: optional User_access_rule           access_rule;
    /** Flag to indicate if user is part of the white list.*/
    19: optional bool                       white_list_flag;
    /** MorphoAccess 2G legacy timemask.
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    20: optional binary                     legacy_timemask;
    /** L1/Bioscrypt legacy extended ID.
     * <product>Y;;Y;;Y;;N;;N;;N;;Y</product>
     */
    21: optional binary                     legacy_extended_id;
    /** Role of the user
     * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
     */
    22: optional byte                       user_role;
    /** Flag to indicate if user is part of the VIP list.*/
    23: optional bool                       VIP_list_flag;
    /** Customer additional data (Not used by terminal)*/
    24: optional binary                     additional_data;
    /** Photo field (specific for MIMS)
     * <product>N;;N;;N;;N;;N;;N;;N</product>
     */
    25: optional binary                     photo;
}

/** No more space to create a new record. Can be because of license.*/
exception DB_full_error
{
    1: required Generic_types.Generic_error_code err_code
}

/** The database is empty.*/
exception DB_empty_error
{
    1: required Generic_types.Generic_error_code err_code
}

/** A similar record already exists in the database.*/
exception DB_duplicate_record_error
{
    /** error code */
    1: required Generic_types.Generic_error_code err_code
    
    /** last user failed id */
    2: optional string user_id_in_error_UTF8; 
}


struct Access_schedule_slots
{
    /** Time slots of 15 mins of interval. value of slot should be between 0-96 */
    1: byte                   start_interval_1;
    /** Time slots of 15 mins of interval. value of slot should be between 0-96 */
    2: byte                   stop_interval_1;
    /** Time slots of 15 mins of interval. value of slot should be between 0-96 */
    3: byte                   start_interval_2;
    /** Time slots of 15 mins of interval. value of slot should be between 0-96 */
    4: byte                   stop_interval_2;
}

struct Access_schedule_slots_v2
{
    /** Time slots of 15 mins of interval. value of slot should be between 0-672 */
    1: i16                   start_interval;
    /** Time slots of 15 mins of interval. value of slot should be between 0-672 */
    2: i16                   stop_interval;
}

struct Access_schedule
{
    /** name of defined schedule */
    1:string                  schedule_name;
    /** Schedule index value should be between 1 to 58.
    *   Schedule 0 is never allowed schedule.Value of this schedule can not be modified.
    *   Schedule 63 is always allowed schedule.Value of this schedule can not be modified.
    *   Schedule 59 to 62 are reserved schedule.Value of this schedule can not be modified.
    */        
    2:byte                    schedule_index;
    /**
     * Schedule weekly slots</br>
     * This field is deprecated. It permits to define up to 2 slots per day, considering week starts on Sunday.
     * The start slot defines the start point/time to allow access.
     * The stop slot defines the stop point/time to not allow access. 
     * Use this field before Firmware version 4.5.0.
     */
    3:list<Access_schedule_slots> schedule_slots;
    /**
     * Schedule weekly slots v2</br>
     * This field permits to define unlimited number of 15 minutes slots within a week, considering week starts on Sunday.
     * The start slot defines the start point/time to allow access.
     * The stop slot defines the stop point/time to not allow access.
     * Use this field from Firmware version 4.5.0.
     */
    4:optional list<Access_schedule_slots_v2> schedule_slots_v2;
}

struct Door_open_schedule_interval
{
    /** Time slots of 15 mins of interval. value of slot should be between 0-96 */
    1: byte                   start_interval;
    /** Time slots of 15 mins of interval. value of slot should be between 0-96 */
    2: byte                   stop_interval;
}

struct Door_open_schedule
{
    /** Door open schedule slots. Maximum 10 slots can be defined **/
    1: list<Door_open_schedule_interval> schedule_interval;
}

struct Holiday_schedule
{
    /** name of defined schedule */
    1:string                  schedule_name;
    /** Schedule index value should be between 0 to 63 */
    2:byte                    schedule_index;
    /** Start date for holiday schedule */
    3:Generic_types.Date_time  start_date;
    /** End date for holiday schedule */
    4:Generic_types.Date_time  end_date;
}

/** Invalid schedule index exception */
exception Invalid_schedule_index
{
    1: required Generic_types.Generic_error_code err_code
}

/** Invalid schedule data exception */
exception Invalid_schedule_data
{
    1: required Generic_types.Generic_error_code err_code
}

/** Schedule store exception */
exception Schedule_store_error
{
    1: required Generic_types.Generic_error_code err_code
}

/** Schedule retrieve exception */
exception Schedule_retrieve_error
{
    1: required Generic_types.Generic_error_code err_code
}

/**
 * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
 * Job code list array full.*/
exception Job_code_list_array_full
{
    1: required Generic_types.Generic_error_code err_code
}

/**
 * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
 * Invalid job code array length.*/
exception Invalid_job_code_array_length
{
    1: required Generic_types.Generic_error_code err_code;
    2: required i16 list_number;
}

/**
 * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
 * Job code list data invalid.*/
exception Job_code_list_data_invalid
{
    1: required Generic_types.Generic_error_code err_code;
    2: required i16 list_number;
}
/**
 * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
 * Job_code_validation_failed.*/
exception Job_code_validation_failed
{
    1: required Generic_types.Generic_error_code err_code
}
