
namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

/**
 * <ps_fields>
 * <ps>max_file_size_video;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>max_file_size_audio;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>max_file_size_picture;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>max_file_size_stolen_card_list;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>max_file_size_language;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>file_ext_video;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>file_ext_audio;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>file_ext_picture;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>file_ext_language;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration for capabilities
 */
enum Capability
{
    // File sizes
    max_file_size_video = 0,
    max_file_size_audio,
    max_file_size_picture,
    max_file_size_stolen_card_list,
    max_file_size_language,
    max_file_size_SSL_component,

    // File extensions
    file_ext_video,
    file_ext_audio,
    file_ext_picture,
    file_ext_language,
    file_ext_SSL_component

    // File upload chunk size
    max_file_upload_chunk_size

    // Users
    max_user_DB_records,
}

/**
 * <ps_fields>
 * <ps>verify_result_ko_user_reference_mismatch;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>verify_result_ko_user_pin_timed_out;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_user_pin_not_exists;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_user_pin_mismath;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_face_not_detected;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>verify_result_ko_job_code_list_not_exists;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_job_code_check_failure;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_job_code_timed_out;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_bio_pin_timed_out;;Y;;Y;;N;;N;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_bio_pin_not_exists;;Y;;Y;;N;;N;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_bio_pin_mismatch;;Y;;Y;;N;;N;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_F_key_timeout;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_F_key_canceled;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>verify_result_ko_acquisition_error;;N;;N;;N;;N;;N;;Y;;N</ps>

 * </ps_fields>
 *
 * Enumeration representing verify result codes
*/
enum Verify_result_code
{
   verify_result_ok = 0,
   verify_result_ok_multi_user_intermediate_id,
   verify_result_ok_multi_user_duplicate_id,
   verify_result_ko_user_trigger_mismatch,
   verify_result_ko_user_reference_mismatch,
   verify_result_ko_user_pin_timed_out,
   verify_result_ko_user_pin_not_exists,
   verify_result_ko_user_pin_mismath,
   verify_result_ko_holiday_scheduled,
   verify_result_ko_holiday_schedule_data_not_found,
   verify_result_ko_user_not_scheduled,
   verify_result_ko_access_schedule_data_not_found,
   verify_result_ko_user_bio_expired,
   verify_result_ko_user_expiry_data_not_found,
   verify_result_ko_user_bio_not_detected,
   verify_result_ko_user_bio_canceled,
   verify_result_ko_user_bio_mismatch,
   verify_result_ko_user_bio_not_found,
   verify_result_ko_user_bio_fake_finger_detected,
   verify_result_ko_user_bio_moist_finger,
   verify_result_ko_user_bio_misplaced_finger,
   verify_result_ko_user_bio_unsupported_format,
   verify_result_ko_user_bio_incompatible_ref_error,
   verify_result_ko_user_not_in_white_list,
   verify_result_ko_face_not_detected,
   verify_result_ko_job_code_list_not_exists,
   verify_result_ko_job_code_check_failure,
   verify_result_ko_job_code_timed_out,
   verify_result_ko_bio_pin_timed_out,
   verify_result_ko_bio_pin_not_exists,
   verify_result_ko_bio_pin_mismatch,
   verify_result_ko_F_key_timeout,
   verify_result_ko_F_key_canceled,
   verify_result_ko_sensor_error,
   verify_result_ko_acquisition_error
}

/**
 * <product>Y;;N;;N;;Y;;Y;;Y;;N</product>
 * Structure for holding video phone related parameters
 */
struct Video_phone_params
{
    /**
     * Name of the profile
     */
    1: required string name,

    /**
     * The IP address to call to
     */
    2: required string address_IP,

    /**
     * The port to connect to at the given IP address
     */
    3: required i16 port
}

/**
 * Enumeration representing data-types of parameters
 */
enum Param_data_types
{
    db_integer = 0,
    db_unsigned_int = 1,
    db_binary = 5,
    db_string = 6
}

/**
 * Structure containing minimum and maximum values for integer
 * and unsigned integer parameters.
 */
struct Integer_range
{
    /**
     * Minimum value in case if range is continuous for eg: "0:10" (min_value = 0)
     */
    1: i32 min_value,

    /**
     * Maximum value in case if range is continuous for eg: "0:10" (max_value = 10)
     */
    2: i32 max_value
}

/**
 * Range shall be defined by the data type of the parameter.
 */
struct Parameter_range
{
    /** Data type of the parameter */
    1: required Param_data_types data_type,

    /**
     * Available only if value of Param_data_types is "db_integer" and "db_unsigned_int"
     */
    2: optional Integer_range continuous_range,

    /**
     * Length of the buffer
     * Available only if value of Param_data_types is "db_binary" and "db_string"
     */
    3: optional i32 max_length,

    /**
     * List of discontinuous values. For eg: If range is "1,3,5,7,9"
     * Available only if value of Param_data_types is "db_integer" and "db_unsigned_int"
     */
    4: optional list<i32> discontinuous_range
}

/**
 * A generic datatype
 *
 *
 * It is the generic datatype that might be used, for instance, as MA5G configuration parameter.<br>
 * Note that only one of the values of interest shall be filled in.
 */
struct Variant
{
    1: optional byte byte_value,
    2: optional i16 int16_value,
    3: optional i32 int32_value,
    4: optional i64 int64_value,
    5: optional double double_value,
    6: optional bool bool_value,
    7: optional string UTF8string_value,
    8: optional binary binary_value
}

/**
 * A structure to deal with date and time
 */
struct Date_time
{
    /** Year */
    1: i16  year;
    /** Month */
    2: i16  month;
    /** Day */
    3: i16  day;
    /** Hour */
    4: i16  hour;
    /** Minute */
    5: i16  minute;
    /** Second */
    6: i16  second;
}

/**
 * <ps_fields>
 * <ps>err_job_code_validation_failed;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>err_job_code_list_data_invalid;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>err_invalid_job_code_array_length;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>err_job_code_list_array_full;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>err_sd_card_not_found;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * </ps_fields>
 *
 * Generic error codes
 */
enum Generic_error_code
{
    err_unknown,
    err_size_invalid,
    err_size_negative,
    err_size_zero,
    err_buffer_overflow,
    err_buffer_underflow,
    err_config_inexistent_parameter,
    err_config_invalid_value,
    err_core_inexistent_user_id,
    err_core_timeout,
    err_core_out_of_memory,
    err_security_default_password,
    err_WIFI_module_not_connected,
    err_no_space_left_on_device,
    err_not_compatible_with_current_license,
    err_ethernet_config_not_found,
    err_ethernet_configuration_failed,
    err_job_code_validation_failed,
    err_job_code_list_data_invalid,
    err_invalid_job_code_array_length,
    err_job_code_list_array_full,
    err_wiegand_invalid_wiegand_string,
    err_wiegand_output_disabled,
    err_host_not_found,
    err_license_not_found,
    err_sd_card_not_found,
    err_sensor_incompatible_version
    err_name_too_long_for_sc
}

/** Enumeration representing the month in a year */
enum Month_in_year
{
    january = 1,
    february,
    march,
    april,
    may,
    june,
    july,
    august,
    september,
    october
    november,
    december
}

/** Enumeration representing the day in week */
enum Day_in_week
{
    monday = 1,
    tuesday,
    wednesday,
    thursday,
    friday,
    saturday,
    sunday
}

struct Time_customed_time_zone
{
    /** Offset from UTC in quarter of hour*/
    1: required i32 UTC_offset_in_quarter,
    /**
     * Number representing the month that starts the daylight saving time.<br>
     * Not set if terminal uses predefined timezone.
     */
    2: optional Month_in_year daylight_start_month,
    /**
     * Number representing the weekday that starts the daylight saving time.<br>
     * Not set if terminal uses predefined timezone.
     */
    3: optional Day_in_week daylight_start_day_in_week,
    /**
     * Number representing the week number in month (1 to 4) where daylight saving time starts.<br>
     * Not set if terminal uses predefined timezone.
     */
    4: optional byte daylight_start_week_in_month,
    /**
     * Number representing the hour of day (0 to 23) where daylight saving time starts.<br>
     * Not set if terminal uses predefined timezone.
     */
    5: optional i16 daylight_start_hour_of_day,
    /**
     * Number representing the month that ends the daylight saving time.<br>
     * Not set if terminal uses predefined timezone.
     */
    6: optional Month_in_year daylight_end_month,
    /**
     * Number representing the weekday that ends the daylight saving time.<br>
     * Not set if terminal uses predefined timezone.
     */
    7: optional Day_in_week daylight_end_day_in_week,
    /**
     * Number representing the week number in month (1 to 4) where daylight saving time ends.<br>
     * Not set if terminal uses predefined timezone.
     */
    8: optional byte daylight_end_week_in_month,
    /**
     * Number representing the hour of day (0 to 23) where daylight saving time ends.<br>
     * Not set if terminal uses predefined timezone.
     */
    9: optional i16 daylight_end_hour_of_day
}

struct Time_time_zone
{
    /** Set to true if daylight saving time shall be handled*/
    1: required bool observe_daylight_saving_time,
    /**
     * Name of a predefined timezone (Europe/Paris for example).<br>
     * Not set if terminal uses customized time zone.
     */
    2: optional string predefined_time_zone_UTF8,
    /**
     * Customized time zone.<br>
     * Not set if terminals uses predefined time zone.
     */
    3: optional Time_customed_time_zone customed_time_zone
}

/** <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
 * Enumeration representing the date display type
 */
enum Date_display_format_type
{
    mm_slash_dd_slash_yyyy,// MM/DD/YYYY
    dd_slash_mm_slash_yyyy,// DD/MM/YYYY
    mmm_dash_dd_dash_yy,// MMM-DD-YY
    dd_dash_mmm_dash_yy,// DD-MMM-YY
    yyyy_slash_mm_slash_dd // YYYY/MM/DD
}

/** <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
 * Enumeration representing the time display type
 */
enum Time_display_format_type
{
    hh_colon_mm_colon_ss,//HH:MM:SS
    hh_colon_mm_dot_ss //HH:MM.SS
}

/** <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
 * Enumeration representing the time meridian type
 */
enum Time_hour_display_type
{
    time_12_hour,
    time_24_hour
}

struct Time_local_date_time_settings
{
    /** Date and time in DDMMYY format*/
    1: optional Date_time date_time,
    /**
     * Structure defining the terminal time zone<br>
     * Can use predefined or customized time zone
     */
    2: optional Time_time_zone time_zone,
    /** Number representing date display format type
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     */
    3: optional Date_display_format_type date_display_type,
    /** Number representing time display format type
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     */
    4: optional Time_display_format_type time_display_type,
    /** Number representing time meridian type
     * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
     */
    5: optional Time_hour_display_type hour_display_type
}

/** The types of the communication system */
enum Communication_system_type
{
    /** Half-duplex communication */
    half_duplex,
    /** Full-duplex communication */
    full_duplex,
}

/** The baud rates supported */
enum Baud_rate
{
    baud_9600 = 9600,
    baud_19200 = 19200,
    baud_38400 = 38400,
    baud_57600 = 57600,
    baud_115200 = 115200
}

/** The data bits supported */
enum Data_bits
{
    /** 7 characters */
    data_bits_7 = 7,
    /** 8 characters */
    data_bits_8 = 8
}

/** Parity supported */
enum Parity_bits
{
    /** no parity bit */
    no_parity = 0,
    /** odd parity bit */
    odd_parity = 1,
    /** even parity bit */
    even_parity = 2
}

/** Stop bits supported */
enum Stop_bits
{
    /** 1 stop bit */
    stop_bits_1 = 1,
    /** 2 stop bits */
    stop_bits_2 = 2
}

/** Settings for serial channel */
struct Serial_params_settings
{
    /** The baud rate for the serial protocol */
    1: optional Baud_rate baud,

    /** Data bits or character size */
    2: optional Data_bits char_size,

    /** Parity supported */
    3: optional Parity_bits parity,

    /** Stop bits supported */
    4: optional Stop_bits stop_bit,

    /** The kind of communication system used by the serial channel this instance represents */
    5: optional Communication_system_type communication_system,

    /** The net ID for the terminal. If remote message over serial feature is enabled, net ID value considered as 1 byte(0 to 255) internally */
    6: optional i16 net_id,
}

/**
 * <ps_fields>
 * <ps>wifi;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>gprs;;Y;;N;;N;;N;;N;;Y;;N</ps>
 * </ps_fields>
 *
 * Different IP channels
 */
enum IP_channel
{
    ethernet,
    wifi,
    gprs
}

enum Ip_version_type
{
    ip_v4,
    ip_v6
}

struct IP_settings
{
    /** Specify for which IP channel the configuration apply for*/
    1: required IP_channel channel,
    /** IPV4 address or IPV6 address configuration*/
    2: required Ip_version_type ip_version,
    /** Set to true if dynamic IP configuration else set to false for static IP configuration*/
    3: required bool is_dhcp_mode,
    /** If static IP configuration then value of IP address to be set*/
    4: optional string ip_address,
    /** If static IP configuration then value of netmask(IPV4)/prefix_length(IPV6) to be set*/
    5: optional string net_mask_or_prefix_len,
    /** If static IP configuration then value of default gateway to be set*/
    6: optional string gateway_address
    /** Device host name*/
    7: optional string host_name
    /** preferred DNS server address*/
    8: optional string preferred_DNS_address
    /** alternate DNS server address*/
    9: optional string alternate_DNS_address
}

struct Sdac_status
{
    /** SDAC mode : 0=Off, 1=On*/
    1: required i32 mode;
    /** Relay state : 0=Door unlocked, 1=Door locked */
    2: optional i32 relay_state;
    /** Door status : 0=Open, 1=Closed, -1=Error */
    3: optional i32 door_status;
    /** Request to exit state : 0=Off, 1=On, -1=Error */
    4: optional i32 req_to_exit_state;
}

/**
 * <ps_fields>
 * <ps>wifi_cfg;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>gprs_cfg;;Y;;N;;N;;N;;N;;Y;;N</ps>
 * </ps_fields>
 *
 * Types of terminal settings
 */
enum Terminal_settings_type
{
    date_time,
    ip,
    wifi_cfg,
    gprs_cfg,
    serial_params_cfg
}

/**
 * <ps_fields>
 * <ps>L1_legacy;;Y;;Y;;Y;;N;;N;;Y;;N</ps>
 * <ps>MA_legacy;;Y;;Y;;Y;;N;;Y;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration of terminal protocol
 */
enum Terminal_protocol
{
   L1_legacy = 0,
   MA_legacy,
   standard,

   get_current
}

/** Enumeration for WIFI encryption type */
enum WIFI_encryption_type
{
    enc_none,
    enc_wep,
    enc_wpa_psk,
    enc_wpa2_psk
}

struct WIFI_network_info
{
    /** SSID address */
    1: optional string SSID,

    /** BSSID address*/
    2: optional string BSSID,

    /** Frequency of ssid */
    3: optional i16 frequency,

    /** Signal level of ssid */
    4: optional i16 signal_level,

    /** Encryption type Enumeration */
    5: optional WIFI_encryption_type enc_type
}

/**
 * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
 * Structure for WIFI settings
 */
struct WIFI_settings
{
    1: WIFI_network_info wifi_info

    /**
     * Security key (password) of WIFI SSID.
     * Write Only value (i.e. can not be read)
     */
    2: optional string password,

    /**
     * WEP key index of SSID.
     * Write Only value (i.e. can not be read)
     */
    3: optional byte wep_key_index,
}

/**
 * <product>Y;;Y;;Y;;N;;N;;Y;;Y</product>
 * Structure for Mobile network settings
 */
struct Mobile_network_settings
{
    /** APN string as per network */
    1: optional string apn,

    /** User Name */
    2: optional string username,

    /** Network PIN */
    3: optional string pin,

    /** Network dial Access number */
    4: optional string access_number
}
/** Enumeration of available distant session state*/
enum IP_protocol_type
{
    ipv4,
    ipv6,
    all
}

struct Terminal_configuration_type
{
    1: required Terminal_settings_type settings_type,
    2: optional Ip_version_type ip_version,
    3: optional IP_channel ip_channel,
}

/**
 * <ps_fields>
 * <ps>wifi_settings;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>mobile_nw_settings;;Y;;Y;;Y;;N;;N;;Y;;Y</ps>
 * </ps_fields>
 */
struct Terminal_configuration
{
    1: optional Time_local_date_time_settings date_time_settings,
    2: optional IP_settings ip_settings
    3: optional WIFI_settings wifi_settings
    4: optional Serial_params_settings serial_params
    5: optional Mobile_network_settings mobile_nw_settings
}

/**
 * <ps_fields>
 * <ps>wifi;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>video_phone;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>crypto_keys;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>stolen_card_list;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>job_code_lists;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>languages;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>multimedia_files;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>dynamic_msg;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * </ps_fields>
 *
 * List of the settings that can be reset.
 */
enum Terminal_reset_settings_type
{
    /** Configuration parameters modified by config_set_params command */
    configuration_parameters,
    /** Date and Time settings: date, time, timezone, display format */
    date_time,
    /** Ethernet settings */
    ethernet,
    /** Serial settings */
    serial,
    /** Wifi settings */
    wifi,
    /** video_phone_params settings */
    video_phone,
    /** crypto keys (includes contactless and RSA keys) */
    crypto_keys,
    /** Passwords */
    passwords,
    /** SSL components */
    ssl_components,
    /** Stolen card list */
    stolen_card_list,
    /** Job code lists */
    job_code_lists,
    /** Access schedules */
    access_schedules,
    /** Holiday schedules */
    holiday_schedules,
    /** Door schedule */
    door_schedule,
    /** authorized IP list */
    authorized_IP_list,
    /** Logs */
    logs,
    /** Passphrases */
    passphrases,
    /** Language files */
    languages,
    /** Multimedia files (video, audio, pictures...) */
    multimedia_files,
    /** User database */
    user_db,
    /** White list */
    white_list,
    /** Dynamic message */
    dynamic_msg,
    /** VIP list */
    vip_list,
    /** Event setting */
    events,
   /** Error Log reset */
    error_log,
   /** Mobile Network settings */
    mobile_network_settings
}

/**
 * <ps_fields>
 * <ps>screen;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>keyboard;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>camera;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>microphone;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>speaker;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>contactless_reader_mifare_desfire;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>contactless_reader_iclass;;Y;;Y;;Y;;N;;Y;;N;;N</ps>
 * <ps>contactless_reader_prox;;Y;;Y;;Y;;N;;Y;;N;;N</ps>
 * <ps>sensor_cbi;;Y;;Y;;Y;;N;;N;;N;;N</ps>
 * <ps>wifi;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>sd_card;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>gprs;;Y;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>barcode_reader;;N;;N;;N;;N;;N;;N;;N</ps>
 * <ps>LED;;N;;N;;Y;;N;;N;;N;;Y</ps>
 * <ps>buzzer;;N;;Y;;Y;;N;;N;;N;;Y</ps>
 * <ps>sensor_fotf;;N;;N;;N;;Y;;N;;Y;;N</ps>
 * <ps>sensor_msi;;N;;N;;N;;N;;Y;;N;;N</ps>
 * <ps>sensor_msi_ffd;;N;;N;;N;;N;;Y;;N;;N</ps>
 * <ps>contactless_reader_all_in_one;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>sensor_ext_fvp;;N;;N;;N;;N;;N;;N;;Y</ps>
 * </ps_fields>
 *
 * List of the terminal peripherals.
 */
enum Terminal_peripherals
{
    screen,
    keyboard,
    camera,
    microphone,
    speaker,
    contactless_reader_mifare_desfire,
    contactless_reader_iclass,
    contactless_reader_prox,
    sensor_cbi,
    wifi,
    sd_card,
    gprs,
    barcode_reader,
    LED,
    buzzer,
    sensor_fotf,
    sensor_msi,
    sensor_msi_ffd,
    contactless_reader_all_in_one,
    sensor_ext_fvp,
}

/**
 * <ps_fields>
 * <ps>front_camera;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>screen;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * </ps_fields>
 * Enumeration of available picture interface for capture feature.
 */
enum Picture_interface
{
    front_camera,
    screen
}

/**
 * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
 * Enumeration of available picture format for capture and display features.
 */
enum Picture_format
{
    JPEG,
    BMP,
    PNG
}

/**
 * <product>Y;;Y;;N;;Y;;Y;;Y;;N</product>
 */
struct XY_coordinates
{
    /** Horizontal coordinate of a point*/
    1: required i16 x,
    /** Vertical coordinate of a point*/
    2: required i16 y
}

/**
 * <product>Y;;Y;;N;;N;;Y;;Y;;N</product>
 * structure to display text on terminal gui.
 */
struct Display_text_optional_param
{
    /** Background colour of terminal GUI*/
    1: optional i32 bg_color,
    /** Text colour of displayed text*/
    2: optional i32 text_color,
    /** Font size of displayed text*/
    3: optional i32 font_size
}

/** Enumeration for protocol type */
enum Wiegand_protocol_type
{
    wiegand,
    clock_and_data
}

/** Enumeration of available distant session state*/
enum Distant_session_state
{
    opened,
    closed
}

/**
 * Structure of previous and current distant session state after "distant_session_set_state_advanced" execution.
 */
struct Get_distant_session_state
{
    /** Enumeration of previous distant session state */
    1: required Distant_session_state previous_distant_session_state,
   /** Enumeration of current distant session state */
    2: required Distant_session_state current_distant_session_state
    
}

/** Enumeration of available distant session message*/
enum Display_message
{
   OFF = 0,
   ON
}

/**
 * Optional parameters for distant_session_state_advance
 */
struct Distant_session_optional_param
{
    /** Enumeration for value of Display_message (OFF,ON) */
    1: optional Display_message display_message

}

/** Enumeration of board type*/
enum Generic_firmware_type
{
    terminal_firmware,
    sensor_firmware
}

/**
 * Product information
 */
struct Product_info
{
    /** UTF8 string that represents 10 digit terminal part number of packaged product */
    1: optional string terminal_packaged_part_number_UTF8,
    /** UTF8 string that represents 14 digit terminal serial number of packaged product */
    2: optional string terminal_packaged_serial_number_UTF8,
    /** UTF8 string that represents terminal commercial name */
    3: optional string terminal_packaged_comm_name_UTF8,
    /** UTF8 string that represents 10 digit sensor part number of packaged product */
    4: optional string sensor_packaged_part_number_UTF8,
    /** UTF8 string that represents 14 digit sensor serial number of packaged product */
    5: optional string sensor_packaged_serial_number_UTF8,
    /** UTF8 string that represents 10 digit product specific part number */
    6: optional string specific_part_number_UTF8,
    /** UTF8 string that represents license identifier */
    7: optional string license_identifier_UTF8,
    /** UTF8 string that represents license name */
    8: optional string license_name_UTF8,
    /** UTF8 string that represents MAC address of Ethernet */
    9: optional string mac_address_ethernet_UTF8,
    /** UTF8 string that represents MAC address of WiFi
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     */
    10: optional string mac_address_wifi_UTF8,
    /** UTF8 string that represents MAC address of 3G modem
     * <product>Y;;N;;N;;N;;N;;Y;;N</product>
     */
    11: optional string mac_address_3g_modem_UTF8
    /** UTF8 string that represents RFID Board type of Smartcard reader */
    12: optional string RFID_Board_type_UTF8,
    /** UTF8 string that represents CIE part number of terminal */
    13: optional string terminal_cie_part_number_UTF8,
}

/**
 * <ps_fields>
 * <ps>mac_address_wifi;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * <ps>mac_address_3g_modem;;Y;;N;;N;;N;;N;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration of product information type
 */
enum Product_info_type
{
    /** 10 digit number that represents terminal part number of packaged product */
    terminal_packaged_part_number,
    /** 14 digit number that represents terminal serial number of packaged product */
    terminal_packaged_serial_number,
    /** String that represents terminal descriptive name of packaged product */
    terminal_packaged_comm_name,
    /** 10 digit number that represents sensor part number of packaged product */
    sensor_packaged_part_number,
    /** 14 digit number that represents sensor serial number of packaged product */
    sensor_packaged_serial_number,
    /** 10 digit number that represents product specific part number */
    specific_part_number,
    /** String that represents license identifier */
    license_identifier,
    /** String that represents license name */
    license_name,
    /** String that represents MAC address of Ethernet */
    mac_address_ethernet,
    /** String that represents MAC address of WiFi */
    mac_address_wifi,
    /** String that represents MAC address of 3G modem */
    mac_address_3g_modem
    /** String that represents RFID board type; 0(SIGMA old ), 1(SIGMA New), 2(LITE) */
    RFID_board_type
    /** String that represents terminal CIE part number */
    terminal_cie_part_number
}

/**
 * Structure representing per event configuration
 */
struct Event_config
{
    /** Event enable/disable flag */
    1: required bool enable,
    /** Event sending to controller flag */
    2: required bool send_to_controller,
    /** Event ID for clock n data */
    3: optional string clock_n_data_id,
    /** Event sending over TTL0 enable/disable flag */
    4: optional bool trigger_TTL0,
    /** Event sending over TTL1 enable/disable flag */
    5: optional bool trigger_TTL1,
    /** Event sending over TTL2 enable/disable flag
     * <product>Y;;N;;N;;N;;Y;;Y;;N</product>
     */
    6: optional bool trigger_TTL2,
}

/** Enumeration host protocol */
enum Protocol_host
{
    /** normal TCP channel */
    prot_TCP,
    /** UDP protocol */
    prot_UDP,
    /** secure TCP channel */
    prot_SSL
}

/**
 * Structure contains IP host configuration data
 */
struct IP_host_config
{
    /** controller IP address */
    1: required string ip_address,

    /** controller port number, range (1:65000) */
    2: required i32 port,

    /** timeout for connection, read and write */
    3: required i32 timeout,

    /** protocol to use for IP channel, range (1:65000) */
    4: required Protocol_host protocol
}

/**
 * Structure contains serial host configuration data
 */
struct Serial_host_config
{
    /** The baud rate for the serial protocol */
    1: required Baud_rate baud,

    /** The kind of communication system used by the serial channel */
    2: required Communication_system_type communication_system,

    /** Databits to use */
    3: required Data_bits char_size,

    /** Parity bits to use */
    4: required Parity_bits parity,

    /** Stop bits to use */
    5: required Stop_bits stopbits
}

/**
 * Configurations of the host on which the data needs to be send
 */
struct Host_configuration
{
    /** IP host configuration if data needs to be sent on IP channel*/
    1: optional IP_host_config ip_host_conf,

    /** serial host configuration if data needs to be sent on serial channel*/
    2: optional Serial_host_config serial_host_conf
}

/**
 * A map that translates error code into string.
 *
 * 
 * <note>For product support, see <code>Generic_error_code</code> Enumeration</note>
 * 
 */
const map< Generic_error_code, string > Generic_error_msg_map =
{
        Generic_error_code.err_unknown                               : "Unknown error",
        Generic_error_code.err_size_invalid                          : "Invalid size",
        Generic_error_code.err_size_negative                         : "Size is negative"
        Generic_error_code.err_size_zero                             : "Size is zero",
        Generic_error_code.err_buffer_overflow                       : "Buffer Underflow",
        Generic_error_code.err_buffer_underflow                      : "Buffer Overflow",
        Generic_error_code.err_config_inexistent_parameter           : "Inexistent configuration parameter",
        Generic_error_code.err_config_invalid_value                  : "Invalid configuration parameter value",
        Generic_error_code.err_core_inexistent_user_id               : "Inexistent User ID",
        Generic_error_code.err_core_timeout                          : "Operation timed out",
        Generic_error_code.err_core_out_of_memory                    : "Out of memory",
        Generic_error_code.err_security_default_password             : "Default password detected",
        Generic_error_code.err_WIFI_module_not_connected             : "WIFI module not connected",
        Generic_error_code.err_no_space_left_on_device               : "No space left on device",
        Generic_error_code.err_not_compatible_with_current_license   : "Not compatible with current license",
        Generic_error_code.err_ethernet_config_not_found             : "Ethernet configuration not found",
        Generic_error_code.err_ethernet_configuration_failed         : "Failed to configure ethernet settings",
        Generic_error_code.err_job_code_validation_failed            : "job code validation failed",
        Generic_error_code.err_job_code_list_data_invalid            : "job code list data invalid",
        Generic_error_code.err_invalid_job_code_array_length         : "invalid job code array length",
        Generic_error_code.err_job_code_list_array_full              : "job code list array full",
        Generic_error_code.err_host_not_found                        : "Send to host failed",
        Generic_error_code.err_license_not_found                     : "License not found",
        Generic_error_code.err_sd_card_not_found                     : "SD card not found",
        Generic_error_code.err_sensor_incompatible_version           : "Incompatible Biometric software version found"
}

/**
 * Printer error codes and/or printer status
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
enum Printer_error_code
{
    printer_noerror = 0x00000000,  //!< Success
    printer_paper_low = 0x00000001,  //!< Error for paper low
    printer_paper_out = 0x00000002,  //!< Error for paper out or no paper
    printer_temp_toohot = 0x00000004,  //!< Error for printer temperature too hot
    printer_cover_open = 0x00000008,  //!< Error for printer cover open
    printer_cutter_err = 0x00000010,  //!< Error in cutter
    printer_buffer_full = 0x00000020,  //!< Error for printer buffer full
    printer_send_err = 0x00000040,  //!< Error for sending command to printer
    printer_not_plugged = 0x00000080,  //!< Error for printer not plugged
    printer_bad_serial = 0x00000100,  //!< Error for bad serial number
    printer_download_err = 0x00000200,  //!< Error in Firmware download
    printer_error = 0x00000400,  //!< Printer error
    printer_memory_err = 0x00000800,  //!< Error in printer memory
    printer_connected = 0x00001000,  //!< Printer connected (used for signal handling)
    printer_bad_file_format = 0x00002000,  //!< Error for bad image file format
    printer_initialization = 0x00004000,  //!< Successful initialization
    printer_not_complete = 0x00008000,  //!< Error for printing not complete
    printer_send_timeout = 0x00010000,  //!< Error for timeout
    printer_cover_close = 0x00020000,  //!< Signal for cover close
    printer_print_bitmap = 0x01000000,  //!< Printer status for printing BMP file
}

/**
 * Enumeration for message type
 * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
 */
enum MMI_message_type
{
	access_granted = 0,
	access_denied
}

/**
 * Structure contains MMI configuration data
 */
struct MMI_config
{
	/** MMI message type Enumeration*/
    1: required MMI_message_type message_type,
    /** Display message timeout
	 * <product>Y;;Y;;N;;N;;Y;;N;;N</product>
	 */
	2: optional i32 message_timeout
}

/**
 * Enumeration for Sound Activation State
 * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
 */
enum Sound_activation_state
{
    no_sound = 0,
    access_denied,
    access_granted
}

/**
 * Enumeration for Relay State
 * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
 */
enum Relay_state
{
    no_action = 0,
    trigger_relay
}

/**
 * Enumeration for Display Activation State
 * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
 */
enum Display_activation_state
{
    led_text_off = 0,
    led_text_red,
    led_text_green
}

/**
 * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
 * Structure contains Sound configuration parameters
 */
struct Sound_config
{
    /** Sound Activation Enumeration */
    1: optional Sound_activation_state sound_activation_state,
    /** Sound Activation timeout */
    2: optional i32 sound_activation_duration
}

/**
 * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
 * Structure contains Relay configuration data
 */
struct Relay_config
{
    /** Relay State Enumeration*/
    1: optional Relay_state relay_state,
    /** Relay timeout */
    2: optional i32 relay_state_duration
}

/**
 * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
 * Structure contains Display configuration data
 */
struct Display_config
{
    /** Display Activation Enumeration*/
    1: optional Display_activation_state display_activation_state,
    /** Display message timeout */
    2: optional i32 display_activation_duration
}

/**
 * <product>Y;;Y;;N;;N;;Y;;Y;;Y</product>
 * Structure contains Text Window configuration parameters
 */
struct Text_window_config
{
    /** Text Message 1
     */
    1: optional string text_line_1,
    /** Text Message 2
     */
    2: optional string text_line_2,
    /** Text Message 3
     */
    3: optional string text_line_3,
    /** Font Size
     */
    4: optional i32 font_size
}

/**
 * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
 * Structure contains the parameters for enhanced MMI play command.
 */
struct MMI_play_params
{
    /** specifies the Sound configuration for Enhanced MMI play
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     */
    1: optional Sound_config sound_config,
    /** specifies the Relay configuration for Enhanced MMI play
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     */
    2: optional Relay_config relay_config,
    /** specifies the Display configuration for Enhanced MMI play
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     */
    3: optional Display_config display_config,
    /** specifies the Text Window configuration for Enhanced MMI play
     * <product>Y;;Y;;N;;N;;Y;;Y;;Y</product>
     */
    4: optional Text_window_config text_window_config
}

/**
 * <product>Y;;N;;N;;N;;Y;;Y;;N</product>
 * Keyboard type.
 */
enum Keyboard_type
{
   alphanumeric = 0,
   numeric
}

/**
 * <product>Y;;N;;N;;N;;Y;;Y;;N</product>
 * Keypad configuration.
 */
struct Keypad_optional_param
{
    /** Duration in second for getting input (valid range is from 0 to 2147483) */
    1: required i32 timeout_in_sec,
    /** Select keyboard type for inputting value */
    2: required Keyboard_type keyboard_type,
    /** Set text line 1 string of the keypad */
    3: optional string text_line_1,
    /** Set text line 2 string of the keypad */
    4: optional string text_line_2,
    /** Set text line 3 string of the keypad */
    5: optional string text_line_3
}

/**
 * <product>Y;;N;;N;;N;;Y;;Y;;N</product>
 * Retrieve Keypad input entered from terminal.
 */
struct Keypad_input
{
    /** Input String entered from terminal */
    1: required string input_text
}

/** The specified user ID does not exist.*/
exception Inexistent_user_id_error
{
    1: required Generic_error_code err_code,
    2: required string user_id_UTF8
}

exception Out_of_memory_error
{
    1: required Generic_error_code err_code
}

/** Command reaches the timeout.*/
exception Timeout_error
{
    1: required Generic_error_code err_code
}

/** One command argument can not be interpreted.*/
exception Invalid_argument_error
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

/** The command was cancelled by distant command.*/
exception Cancelled_error
{
    1: required Generic_error_code err_code
}

exception Security_default_password
{
    1: required Generic_error_code err_code
}

exception File_inexistent_error
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

exception File_name_missing_error
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

exception Invalid_file_extension_error
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

exception File_too_large_error
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

exception WIFI_module_not_connected
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

exception Ethernet_config_not_found
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

exception Ethernet_configuration_failed
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

exception Invalid_IP_error
{
    1: required string what_UTF8
}

exception Invalid_size_error
{
    1: required string what_UTF8
}

exception Invalid_wiegand_string
{
    1: required Generic_error_code err_code
}

exception Wiegand_output_disabled
{
    1: required Generic_error_code err_code
}

exception Host_not_found
{
    1: required Generic_error_code err_code
}
exception License_not_found
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;Y;;N;;N;;N</product>
 */
exception Biometric_sensor_incompatible_version_error
{
    1: required string what_UTF8
}

/**
 * <product>Y;;N;;N;;Y;;Y;;N;;N</product>
 */
exception SD_card_not_found
{
    1: required Generic_error_code err_code
}

/** The exception encompassing other general exceptions */
exception Generic_error
{
    1: required Generic_error_code err_code;
    2: optional string what_UTF8;
}

exception Bad_file_format
{
    1: required Generic_error_code err_code
}

/** Printer exceptions */
/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Printer_paper_out
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Printer_temperature_too_hot
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Printer_cover_open
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Printer_cutter_error
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Printer_not_plugged
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Printer_error
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Printer_bad_serial
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Printer_general_error
{
    1: required Printer_error_code err_code;
    2: optional string what_UTF8;
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception fw_integrity_error
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception VFD_not_connected
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception VFD_error
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception Anti_tamper_not_found
{
    1: required Generic_error_code err_code
}

/**
 * <product>N;;N;;N;;N;;N;;N;;N</product>
 */
exception BCR_not_connected
{
    1: required Generic_error_code err_code
}
