// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2012 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Biofinger_types.thrift
 *
 * CHANGELOG
 * 23 nov 2012  - Initiate header
 * 03 dec 2012  - In Biofinger_compression_type, add _algo (because null is interpreted by SWIG)
 *              - Change "set" to "list" in structure because std::set is a mess in SWIG
 * 06 dec 2012  - Change standard member in Biofinger_sensor_matching_strategy and Biofinger_sensor_coder
 *                so that files generated with pure_enums option can compile
 * 10 dec 2012  - Change namespace to comply MPH coding rules.
 * 13 dec 2012  - Erase useless Biofinger_mode
 *              - Add comments
 *              - Add Biofinger_enroll_operation_reply for reply of Biofinger_enroll command
 *              - Change Biofinger_control_operation_reply, no need for enrollment steps
 *              - Change members' names of Biofinger_sensor_image to indicate the type
 *              - Correct name of Biofinger_control_optional_param required_matching_score
 * 18 dec 2012  - Change returned image of enroll command into best_image
 * 08 jan 2013  - Modify enroll distant command to be able to export FP templates and FV at same time.
 *              - Rename some variables for better understanding
 * 09 jan 2013  - Change images requirement in enroll command
 *              - Enroll now returns a security compatibility per template
 * 15 may 2013  - Changed 'enrol' word(UK english) to 'enroll' (US english change)
 *              - Response of enrollment command changed to include template type field as well
 * 10 july 2013 - Removed 'cfv' as template type from Biofinger_template_type enumeration.
 * 20 feb 2014  - Add namespace for java and C# generation
 * 05 jun 2014  - Updated enumeration 'Biofinger_sensor_security_level', enumeration name updated as well as new value added for FFD_disabled
 * 16 jan 2015  - Added optional parameter for fail user id having duplicate finger in exception 'Duplicated_finger_error'
 * 29 jan 2015  - MorphoWave Product support changes
 * 11 feb 2015  - Biofinger_template_type.bioscrypt not supported in MorphoWave, added finger id (image_index) in Biofinger_sensor_image 
 * 11 feb 2015  - Biofinger_template_type minex_a, din_v66400_cs, din_v66400_cs_aa, multimodal not supported in MorphoWave 
 * 24 mar 2015  - coder, detection_mode, require_security_compatibilty not supported in all products 
 * 25 mar 2015  - Rename image_index to finger_number
 * 31 mar 2015  - Biofinger_events boolean replaced with Biofinger_event enumeration list.
 * 06 apr 2015  - Enumeration Biofinger_command not supported on MorphoWave.
 * 10 apr 2015  - remove "temperature_mask" from Biofinger_event
 * 21 apr 2015  - added notes "left and right thumb not supported" for MorphoWave product
 * 19 may 2015  - added product specific table for Biofinger_event
 * 04 nov 2015  - Change java namespace for coherency
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 08 aug 2016  - Add "nb_capture" for one capture acquisition
 * 01 mar 2017  - Update product support table for 'Biofinger_event' and 'Biofinger_command'
 * 27 Mar 2017  - PRGCA Product support added 
 * 14 sep 2017  - Update product support table for 'Biofinger_matching_moist_finger' and 'low_resol_live_images'
 * 29 mar 2018 -  Update for MorphoWave Compact enrollment
 * 28 may 2018  - Update comment for "template_number" field of "User_templates" structure for card encoding feature.  
 * 12 june 2018 - Add "low_quality_fingers" event enum under "Biofinger_event"
 * 03 Jul 2018  - MA_VP_MD Product support added 
 * 05 Jul 2019  - Description updated for Biofinger_template_type
 * 21 Nov 2019  - Added "no_bio_template" template type to Biofinger_template_type
 */
// </doc_history>

include "Generic_types.thrift"

// Access Control (authenticate, identify ,enrollment) types

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

/**
 * <ps_fields>
 * <ps>Bio_sensor_internal_error;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>Biofinger_acquisition_failure;;N;;N;;N;;N;;N;;Y;;N</ps> 
 * <ps>Biofinger_matching_moist_finger;;Y;;Y;;Y;;Y;;Y;;N;;Y</ps> 
 * </ps_fields>
 *
 * Enumeration representing of the biometric operation failure. 
 */
enum Biofinger_failure_cause
{
    Biofinger_matching_no_hit,
    Biofinger_matching_FFD,
    Biofinger_matching_moist_finger,
    Biofinger_enrollment_bad_quality,
    Bio_sensor_internal_error,
    Biofinger_acquisition_failure,
}


/** 
* Enumeration representing swipe status of hand.
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
enum Biofinger_hand_swipe_status 
{
    swipe_ok,
    swipe_too_fast,
    swipe_too_inclined,
    swipe_too_slow,
}

/**
* Enumeration representing depth status of the hand.
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
enum Biofinger_hand_depth_status 
{
    deep_enough,
    too_deep,
    not_deep_enough,
}

/**
* Enumeration representing status of the hand wave (expectation and direction).
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
enum Biofinger_hand_wave_status 
{
    expected_hand_and_good_direction,
    unexpected_hand_and_good_direction,
    expected_hand_and_wrong_direction,
    unexpected_hand_and_wrong_direction,
}

/**
* Enumeration representing status of Inter-matching controls (Unicity and coherency).
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
enum Biofinger_inter_matching_status 
{
    inter_matching_ok,
    unicity_ko,
    coherency_ko,    
}

/**
* Enumeration representing status of device temperature.
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
enum Biofinger_device_temperature_status 
{
    temperature_ok,
    temperature_high,
    temperature_too_high,
}

/**
* Acquisition status structure
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
struct Biofinger_acquisition_status
{
    /** Dirty status of sensor glass.
    */
    1: optional bool dirty_glass;
    
    /** Scratched status of sensor glass.
    */
    2: optional bool scratched_glass;    
    
    /** Hand swipe status
    */
    3: optional Biofinger_hand_swipe_status swipe_status,
    
    /** Hand depth status
    */    
    4: optional Biofinger_hand_depth_status depth_status,

    /** Hand wave status
    */        
    5: optional Biofinger_hand_wave_status wave_status,

    /** Captured fingers number status (expected or not expected).<br>
    * True if expected number of fingers are captured
    */     
    6: optional bool nb_captured_finger_status,
    
    /** Inter matching status
    */         
    7: optional Biofinger_inter_matching_status inter_matching_status,
    
    /** Device temperature status
    */             
    8: optional Biofinger_device_temperature_status temperature_status,
}

/** 
 * <b>Warning:</b> From version <b>1.5.0</b>, the field 3: matching_score won’t be returned anymore when matching score is requested in command.
 * Instead the field biometric_matching_score will be returned.
 */
struct Biofinger_control_final_result
{
    /**
     * Indicate matching result.<br>
     * True for hit, false for error or no hit.
     */
    1: required bool success,
    /**
     * Indicate the cause of the error.<br>
     * Present if success field is false.
     */
    2: optional Biofinger_failure_cause cause,
    /**
     * If success field if true, score of the matching operation. <br>
     * It shall have been requested in the request.
     */
    3: optional byte matching_score,
    /** If success field is true, number of the finger that matched with captured one (see Biofinger_control_optional_param::require_matching_score).*/
    4: optional byte matching_template_number,
    /**
     * User ID.<br>
     * It is filled once the user ID is known, whatever the final result is.
     */
    5: optional string user_ID_UTF8
    /**
     * If success field if true, score of the matching operation. <br>
     * It shall have been requested in the request.
     */
    6: optional i32 biometric_matching_score

    /**
     * Status information of last acquisition
     * Present if success is true or failure cause is Biofinger_acquisition_failure.<br>    
     * <product>N;;N;;N;;N;;N;;Y;;N</product>     
     */
    7: optional Biofinger_acquisition_status        acquisition_status,            
}

/**
 * <ps_fields>
 * <ps>biofinger_move_no_finger;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_move_finger_up;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_move_finger_down;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_move_finger_left;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_move_finger_right;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_press_finger_harder;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_latent;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_remove_finger;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_finger_ok;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_finger_detected;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>biofinger_finger_misplaced;;N;;N;;N;;N;;N;;N;;N</ps>
 * <ps>biofinger_live_ok;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * </ps_fields>
 *
 * Enumeration representing the finger position/command event sent by the sensor.
 */
enum Biofinger_command
{
    biofinger_move_no_finger = 0,
    biofinger_move_finger_up = 1,
    biofinger_move_finger_down = 2,
    biofinger_move_finger_left = 3,
    biofinger_move_finger_right = 4,
    biofinger_press_finger_harder = 5,
    biofinger_latent = 6,
    biofinger_remove_finger = 7,
    biofinger_finger_ok = 8,
    biofinger_finger_detected = 9,
    biofinger_finger_misplaced = 10,
    biofinger_live_ok = 11,
}

/**
 * <ps_fields>
 * <ps>no_finger;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>move_finger_up;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>move_finger_down;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>move_finger_left;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>move_finger_right;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>press_finger_harder;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>latent_detected;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>remove_finger;;Y;;Y;;Y;;N;;Y;;N;;Y</ps> 
 * <ps>finger_misplaced;;N;;N;;N;;N;;N;;N;;N</ps> 
 * <ps>live_ok;;Y;;Y;;Y;;N;;Y;;N;;Y</ps> 
 * <ps>capture_started;;N;;N;;N;;Y;;N;;Y;;N</ps>
 * <ps>coding_matching_done;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>device_dirty;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>duplicate_finger;;N;;N;;N;;Y;;N;;Y;;N</ps>
 * <ps>fake_finger_detected;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>glass_scratched;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>hand_too_fast;;N;;N;;N;;Y;;N;;Y;;N</ps>
 * <ps>hand_too_inclined;;N;;N;;N;;Y;;N;;Y;;N</ps>
 * <ps>inconsistent_fingers;;N;;N;;N;;Y;;N;;Y;;N</ps>
 * <ps>low_quality_left_index;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_left_little;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_left_middle;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_left_ring;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_left_thumb;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_right_index;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_right_little;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_right_middle;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_right_ring;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>low_quality_right_thumb;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>pre_proc_done;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>temperature_high;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>temperature_too_high;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>wrong_finger_number;;N;;N;;N;;Y;;N;;Y;;N</ps> 
 * <ps>presence_detected;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * <ps>capture_ended;;N;;N;;N;;N;;N;;Y;;N</ps>   
 * <ps>force_finger_ok;;N;;N;;N;;N;;N;;Y;;N</ps> 
 * <ps>bad_hand;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * <ps>bad_direction;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * <ps>hand_too_slow;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * <ps>hand_not_deep_enough;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * <ps>hand_too_deep;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * <ps>low_quality_fingers;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * </ps_fields>
 *
 * Enumeration representing MorphoWave Tower and MorphoWave Compact terminal specific events.
 */
enum Biofinger_event
{
    no_finger = 0,
    move_finger_up = 1,
    move_finger_down = 2,
    move_finger_left = 3,
    move_finger_right = 4,
    press_finger_harder = 5,
    latent_detected = 6,
    remove_finger = 7,
    finger_ok = 8, // same as image_captured
    finger_detected = 9, // same as presence_detected
    finger_misplaced = 10,
    live_ok = 11,
    hand_too_fast = 12,
    hand_too_inclined = 13,
    low_quality_left_index = 14,
    low_quality_left_little = 15,
    low_quality_left_middle = 16,
    low_quality_left_ring = 17,
    low_quality_left_thumb = 18,
    low_quality_right_index = 19,
    low_quality_right_little = 20,
    low_quality_right_middle = 21,
    low_quality_right_ring = 22,
    low_quality_right_thumb = 23,
    duplicate_finger = 24,
    inconsistent_fingers = 25,
    fake_finger_detected = 26,
    wrong_finger_number = 27,
    device_dirty = 28,
    glass_scratched = 29,
    capture_started = 30,
    pre_proc_done = 31,
    coding_matching_done = 32,
    temperature_high = 33,
    temperature_too_high = 34,
    presence_detected  = 35,
    capture_ended  = 36,
    force_finger_ok = 37,
    bad_hand = 38,
    bad_direction = 39,
    hand_too_slow = 40,
    hand_not_deep_enough = 41,
    hand_too_deep = 42,
    low_quality_fingers = 43,  
}

struct Biofinger_callback_command
{
    /**
    * Finger placement modification required, or finger data acquisition completed.<br>
    * Sent as intermediate reply of biometric command.
    * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
    */
    1: optional Biofinger_command command;
    /**
    * Finger capture events and feedback messages.<br>
    * Sent as intermediate reply of biometric command.
    * <product>N;;N;;N;;Y;;N;;Y;;N</product>
    */
    2: optional list < Biofinger_event > events;
}

/** Enumeration representing compression used in an exported image.*/
enum Biofinger_compression_type
{
    null_algo,
    v1_algo,
    wsq_algo,
}

/**
* Enumeration representing status of a finger during consolidated swipe.<br>
* no_status: finger has not been captured, no template available.<br>
* poor_quality and good_quality: template is available and shall be stored.<br>
* <product>N;;N;;N;;N;;N;;Y;;N</product> 
*/
enum Biofinger_consolidation_status
{
    no_status,
    poor_quality,
    good_quality
}

struct Biofinger_sensor_image
{
    /** Width of the image in pixels.*/
    1: required i32 width_in_pix,
    /** Height of the image in pixels.*/
    2: required i32 height_in_pix,
    /** Horizontal resolution of the image in dot per inch.*/
    3: required i32 horizontal_resol_dpi,
    /** Vertical resolution of the image in dot per inch.*/
    4: required i32 vertical_resol_dpi,
    /** Compression algorithm used on the image.*/
    5: required Biofinger_compression_type  compression_type,
    /** Image's binary data.*/
    6: required binary image_data,
    /**
     * Number of bits per pixel for the image.<br>
     * Present in intermediate replies' images of biometric commands.
     */
    7: optional byte bits_per_pixel,
    /**
     * Number that represents the finger captured.<br>
     * 1 for left little finger, up to 10 for right little finger.<br>
     * <product>N;;N;;N;;Y;;N;;Y;;N</product>
     * <note> For MorphoWave Tower and MorphoWave Compact left and right thumbs are not supported. </note>
     */
    8: optional byte finger_number,
    /**
     * Consolidation status of the finger.<br>
     * Only after each swipe, not in single swipe (see @a Biofinger_async_event::high_resol_captures_image).
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     */    
    9: optional Biofinger_consolidation_status consolidation_status,
}



/**
 * <info_fields>
 * <itb>Template on device;;pklite;;Best accuracy</itb>
 * <itb>Templates on card;;pkcompv2;;Best trade-off between memory size and accuracy</itb>
 * </info_fields>
 *
 *Other template formats can be used to leverage existing databases, it is recommended not to use them for new enrollment.
 *
 * <ps_fields_custom>
 * <ps_custom>pkcompv2;;Template on card;;Y;;Y;;Y;;N;;Y;;Y;;N</ps_custom>
 * <ps_custom>pkmat;;Legacy installations;;Y;;Y;;Y;;N;;Y;;Y;;N</ps_custom>
 * <ps_custom>ansi378_2004;;Legacy installations;;Y;;Y;;Y;;N;;Y;;Y;;N</ps_custom>
 * <ps_custom>iso19794_2_fmc_cs;;Legacy installations;;Y;;Y;;Y;;N;;Y;;Y;;N</ps_custom>
 * <ps_custom>iso19794_2_fmc_ns;;Legacy installations;;Y;;Y;;Y;;N;;Y;;Y;;N</ps_custom>
 * <ps_custom>iso19794_2_fmr;;Legacy installations;;Y;;Y;;Y;;N;;Y;;Y;;N</ps_custom>
 * <ps_custom>iso19794_2_fmc_cs_aa;;Legacy installations;;Y;;Y;;Y;;N;;Y;;N;;N</ps_custom>
 * <ps_custom>minex_a;;Legacy installations;;Y;;Y;;Y;;N;;Y;;N;;N</ps_custom>
 * <ps_custom>din_v66400_cs;;Legacy installations;;Y;;Y;;Y;;N;;Y;;N;;N</ps_custom>
 * <ps_custom>din_v66400_cs_aa;;Legacy installations;;Y;;Y;;Y;;N;;Y;;N;;N</ps_custom>
 * <ps_custom>multimodal (fingerprint part only);;Legacy installations;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps_custom>
 * <ps_custom>bioscrypt;;Legacy installations;;Y;;Y;;Y;;N;;Y;;N;;N</ps_custom>
 * <ps_custom>pklite;;Template on device;;Y;;Y;;Y;;N;;Y;;Y;;N</ps_custom>
 * <ps_custom>ansi378_2009;;Legacy installations;;N;;N;;N;;N;;N;;Y;;N</ps_custom>  
 * <ps_custom>iso19794_2_fmr_2011;;Legacy installations;;N;;N;;N;;N;;N;;Y;;N</ps_custom> 
 * <ps_custom>no_bio_template;;No template;;N;;N;;N;;N;;N;;Y;;N</ps_custom> 
 * </ps_fields>
 *
 * Enumeration representing the format handled by the sensor in biometric operations. 
 *
 */
enum Biofinger_template_type
{
    /** No biometric template
     * It means no template is used as input or output parameter 
     * Morpho template format which will not return any bio template. It will return only bio images.
     */
    no_bio_template = -1,
    /** Morpho legacy native fingerprint template format.*/
    pkcompv2 = 0,
    /** Morpho private fingerprint template format dedicated to AFIS.*/
    pkmat,
    /** ANSI INCITS 378-2004 public finger minutiae record format.*/
    ansi378_2004,
    /**
     * Finger Minutiae Card Record, Compact Size.<br>
     * Public fingerprint template format, dedicated to smart card, defined by the ISO organization.
     */
    iso19794_2_fmc_cs,
    /**
     * Finger Minutiae Card Record, Normal Size.<br>
     * Public fingerprint template format, dedicated to smart card, defined by the ISO organization.
     */
    iso19794_2_fmc_ns,
    /**
     * Finger Minutiae Record.<br>
     * Public fingerprint template format, defined by the ISO organization.
     */
    iso19794_2_fmr,
    /**
     * Finger Minutiae Card Record, Compact Size, minutiae ordered by Ascending Angle.<br>
     * Public fingerprint template format, dedicated to smart card, defined by the ISO organization, with minutiae ordered by ascending angle.
     */
    iso19794_2_fmc_cs_aa,
    /** Public fingerprint template format, defined by the NIST organisation for MINEX testing.*/
    minex_a,
    /**
     * DIN V66400 Finger Minutiae Card Record.<br>
     * Public fingerprint template format, dedicated to use with Smart Cards to perform a Match On Card.
     */
    din_v66400_cs,
    /**
     * DIN V66400 Finger Minutiae Card Record, Compact Size, minutiae ordered by Ascending Angle.<br>
     * Public fingerprint template format, dedicated to use with Smart Cards to perform a Match On Card, with minutiae ordered by ascending angle.
     */
    din_v66400_cs_aa,
    /**
     * Morpho private multimodal template format which contains the fingerprint data and the vein data of a finger.<br>
     * This is the format that shall be used for best performance, on FingerVein terminals.
     */
    multimodal,
    /**
     * L1/Bioscrypt legacy native fingerprint template format.<br>
     * This shall be used only for L1/Bioscrypt legacy mode for user_DB_set_users command.
     */
    bioscrypt,
    /**
     * PKLITE is newly added template type in Sensor SDK.
     */
    pklite,
    /** ANSI INCITS 378-2009 public finger minutiae record format.*/
    ansi378_2009,
    /**
     * Finger Minutiae Record.<br>
     * Public fingerprint template format, defined by the ISO organization (19794-2 2011 standard).
     */
    iso19794_2_fmr_2011,
}

/**
* Enumeration representing status of biometric data for the enrollment process.
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
enum Biofinger_status 
{
    safe,
    amputated,    
    bandaged,
}

struct User_templates
{
    /** Type of the template.*/
    1: required Biofinger_template_type template_type;
    /** Template data.*/
    2: required binary template_data;
    /** Number that represents the enrolled template number <br>
     * i.e. for finger biometric data, this number represents finger enrolled. <br>
     * 1 for left little finger, up to 10 for right little finger. <br>
     * <note> 
     * - For MorphoWave Tower and MorphoWave Compact terminals, left and right thumbs are not supported. <br>
     * - For MorphoWave Compact terminals, this field need to be provided during card encoding. <br>
     * - Either this field is present or not, following fields in User_DB_record structure fields are present <br>
     * first_finger_nb, second_finger_nb, third_finger_nb.<br>
     * Both can not be present at the same time.</note>
     */
    3: optional byte template_number;
    /** Quality of the enrolled template
     * <note> Either this field is present or not, following fields in User_DB_record structure fields are present <br>
     * first_finger_qual, second_finger_qual, third_finger_qual.<br>
     * Both can not be present at the same time.</note>
     * <product>Y;;Y;;Y;;Y;;Y;;N;;Y</product>     
     */
    4: optional i16 template_quality;

    /** Status of template to be enrolled
     * i.e. for finger biometric data, this represents status of finger to be enrolled. <br>  
     * <note> Used as input for enrollment process only .<br>
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     */
    5: optional Biofinger_status finger_status;
}

struct Biofinger_enroll_final_result
{
    /**
     * Indicate enrollment result.<br>
     * True for success, false for error.
     */
    1: required bool                            success,
    /**
     * Indicate the cause of the error.<br>
     * Present if success field is false.
     */
    2: optional Biofinger_failure_cause         cause,
    /**
     * Binary fingerprint templates of the user.<br>
     * Present if success is true and templates are required in the command.
     */
    3: optional list<User_templates>                    fp_templates,
    /**
     * Binary fingervein templates of the user.<br>
     * Present if success is true and templates are required in the command.
     * <product>N;;N;;N;;N;;N;;N;;Y</product>
     */
    4: optional list<User_templates>                    fv_templates,
    /**
     * Binary result best images of the fingers.<br>
     * Present if success is true and images are required in the command (see Biofinger_enroll_optional_param::require_best_fingers_images).<br>
     * 1 image per finger.
     */
    5: optional list<Biofinger_sensor_image>    best_finger_images,
    /**
     * Result qualities of the captured fingers.<br>
     * Present if success is true and final qualities are required in the command (see Biofinger_enroll_optional_param::require_fingers_qualities).<br>
     * 1 quality per finger.
     * <product>Y;;Y;;Y;;Y;;Y;;N;;Y</product>
     */
    6: optional list<byte>                      finger_qualities,
    /**
     * Indicate if the enrolled templates complies with sensor security level.<br>
     * Present if requested in the command (see Biofinger_enroll_optional_param::require_security_compatibilty).<br>
     * 1 parameter per finger.
     * <product>N;;N;;N;;N;;N;;N;;N</product>
     */
    7: optional list<i16>                       security_compatibilities,
    /**
     * Status information of last acquisition
     * Present if success is true or failure cause is Biofinger_acquisition_failure.<br>    
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     */
    8: optional Biofinger_acquisition_status                  acquisition_status,
    /**
     * Per finger information, about consolidated swipe.<br>
     * Forefinger first up to little finger.
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     */
    9: optional list<Biofinger_consolidation_status>          fingers_consolidation_status,
}

struct Biofinger_callback_enrollment_step
{
    /** Current finger number for the enrollment process (starting to 1).
    * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
    */
    1: required byte current_finger,
    /** Total number of fingers for the enrollment process.
    * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
    */
    2: required byte total_finger,
   /** Current capture of the current enrollment (starting to 1).<br>
    * For MorphoWave Compact, this is a percentage of capture progress.
    */
    3: required byte current_capture,
   /** Total number of captures for enrollment.<br>
    * For MorphoWave Compact, this is 100%.
    */
    4: required byte total_capture
}

struct Biofinger_enroll_operation_reply
{
    /**
     * Final reply of authenticate/identify command. <br>
     * If final_result is present, none of the other fields shall be
     * present.
     */
    1: optional Biofinger_enroll_final_result        final_result,
    /**
     * Intermediate reply that is used to:
     * <ul>
     * <li>Indicate that there is a wrongly placed finger on the sensor, and then the action that the user has to perform.</li>
     * <li>Provide information about the state of the fingerprint process (fingerprint wait started, fingerprint acquisition completed).</li>
     * </ul>
     * It shall have been requested in the command (see @a Biofinger_async_event::finger_positions).
     */
    2: optional Biofinger_callback_command          cb_bio_command,
    /**
     * Intermediate reply that is used to transmit images from the sensor whatever there is a fingerprint or not.<br>
     * The terminal sends live images in low resolution until the end of the fingerprint acquisition process, with a reduced size to provide a convenient image throughput.<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::low_resol_live_images).
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    3: optional Biofinger_sensor_image              cb_low_resol_live_image,
    /**
     * Intermediate reply that is used to indicate the enrollment process steps.<br>
     * For example, during the enrollment of one finger with three images, the terminal sends three messages (enrollment of finger 1 of 1, acquisition 1, 2 and 3).<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::enrollment_steps).
     */
    4: optional Biofinger_callback_enrollment_step   cb_enroll_step,
    /**
     * Intermediate reply that is used to transmit only one image: the image of the current capture (May be several capture per finger).<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::high_resol_captures_image).
     */
    5: optional Biofinger_sensor_image              cb_high_resol_capture_image,
    /**
     * Intermediate reply that is used to provide the quality value of the capture (May be several capture per finger).<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::captures_quality).
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    6: optional i32                                 cb_capture_quality,
    /**
     * Intermediate reply that is used to provide the quality note of the live fingerprint image, which is calculated by the presence detection function.<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::live_quality).
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    7: optional i32                                 cb_live_quality,
}

struct Biofinger_control_operation_reply
{
    /**
     * Final reply of authenticate/identify command. <br>
     * If final_result is present, none of the other fields shall be
     * present.
     */
    1: optional Biofinger_control_final_result      final_result,
    /**
     * Intermediate reply that is used to:
     * <ul>
     * <li>Indicate that there is a wrongly placed finger on the sensor, and then the action that the user has to perform.</li>
     * <li>Provide information about the state of the fingerprint process (fingerprint wait started, fingerprint acquisition completed).</li>
     * </ul>
     * It shall have been requested in the request (see @a Biofinger_async_event::finger_positions).
     */
    2: optional Biofinger_callback_command          cb_bio_command,
    /**
     * Intermediate reply that is used to transmit images from the sensor whatever there is a fingerprint or not.<br>
     * The terminal sends live images in low resolution until the end of the fingerprint acquisition process, with a reduced size to provide a convenient image throughput.<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::low_resol_live_images).
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    3: optional Biofinger_sensor_image              cb_low_resol_live_image,
    /**
     * Intermediate reply that is used to transmit only one image: the image of the current capture (May be several capture per finger).<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::high_resol_captures_image).
     */
    4: optional Biofinger_sensor_image              cb_high_resol_capture_image,
    /**
     * Intermediate reply that is used to provide the quality value of the acquired fingerprint image.<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::captures_quality).
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    5: optional i32                                 cb_capture_quality,
    /**
     * Intermediate reply that is used to provide the quality note of the live fingerprint image, which is calculated by the presence detection function.<br>
     * It shall have been requested in the command (see @a Biofinger_async_event::live_quality).
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product>
     */
    6: optional i32                                 cb_live_quality,
}

/**
 * <ps_fields>
 * <ps>low_resol_live_images;;Y;;Y;;Y;;N;;Y;;N;;Y</ps> 
 * <ps>captures_quality;;Y;;Y;;Y;;N;;Y;;N;;Y</ps> 
 * <ps>live_quality;;Y;;Y;;Y;;N;;Y;;N;;Y</ps> 
 * </ps_fields>
 *
 * Enumeration representing the events that can be monitored.*/
enum Biofinger_async_event
{
    finger_positions,
    low_resol_live_images,
    enrollment_steps,
    high_resol_captures_image,
    captures_quality,
    live_quality,
}

/**
 * <ps_fields>
 * <ps>FFD_low;;Y;;Y;;Y;;Y;;Y;;Y;;N</ps> 
 * <ps>FFD_medium;;Y;;Y;;Y;;Y;;Y;;Y;;N</ps>  
 * <ps>FFD_high;;Y;;Y;;Y;;Y;;Y;;Y;;N</ps>  
 * <ps>FVP_standard;;N;;N;;N;;N;;N;;N;;Y</ps> 
 * <ps>FVP_medium;;N;;N;;N;;N;;N;;N;;Y</ps>  
 * <ps>FVP_high;;N;;N;;N;;N;;N;;N;;Y</ps>  
 * </ps_fields>
 *
 * Enumeration representing the FFD/FVP security level to use.*/
enum Biofinger_sensor_security_level
{
    FFD_low,
    FFD_medium,
    FFD_high,
    FVP_standard,
    FVP_medium,
    FVP_high,
    FFD_disabled,
}

/** Enumeration representing the matching strategy to apply.*/
enum Biofinger_sensor_matching_strategy
{
    standard_strategy,
    advanced_strategy
}

/** Enumeration representing the hand (left or right).
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
enum Biofinger_hand
{
    right_hand,
    left_hand,
}

struct Biofinger_control_optional_param
{
    /** List of event to subscribe to.*/
    1: optional list<Biofinger_async_event>         events,
    /** Matching score is required in final answer.*/
    2: optional bool                                require_matching_score,
    /** On MSI or VP equipped terminals, indicate the security level to use for the operation.*/
    3: optional Biofinger_sensor_security_level     security_level,
    /** Shall not be used.
     * <product>N;;N;;N;;N;;N;;N;;N</product> 
     */
    4: optional i32                                 coder,
    /** Shall not be used.
     * <product>N;;N;;N;;N;;N;;N;;N</product> 
     */
    5: optional i32                                 detection_mode,
    /** Shall not be used, right now.
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product>
     */
    6: optional Biofinger_sensor_matching_strategy  matching_strategy,
    /** First hand to match (to speed up matching time).<br>
     * Default is right hand    
     * <product>N;;N;;N;;N;;N;;Y;;N</product>
     */
    7: optional Biofinger_hand first_hand_to_match,    
}

/**
* Enumeration representing acquisition threshold for the enrollment process.
* <product>N;;N;;N;;N;;N;;Y;;N</product>
*/
enum Biofinger_acquisition_threshold 
{
    standard_quality_check,
    secure_quality_check,
    very_secure_quality_check,
}

struct Biofinger_enroll_optional_param
{
    /** List of event to subscribe to.*/
    1: optional list<Biofinger_async_event>         events,
    /** Specify the format of the fingerprint templates to be exported.*/
    2: optional Biofinger_template_type             fp_template_format,
    /** Specify the format of the fingervein templates to be exported. (FingerVein-equipped terminals only)
     * <product>N;;N;;N;;N;;N;;N;;Y</product> */
    3: optional Biofinger_template_type             fv_template_format,
    /** Best image of each captured fingers is required in the reply.*/
    4: optional bool                                require_best_fingers_images,
    /**
     * Require latent detection.<br>
     * Not recommended for standard enrollment.
     * <product>Y;;Y;;Y;;N;;Y;;N;;Y</product> */
    5: optional bool                                require_latent,
    /** On MSI or VP equipped terminals, indicate the security level to use for the operation.
     *
     * maxt_ffd_level_note */
    6: optional Biofinger_sensor_security_level     security_level,
    /** Shall not be used.
     * <product>N;;N;;N;;N;;N;;N;;N</product> */
    7: optional i32                                 coder,
    /** Shall not be used.
     * <product>N;;N;;N;;N;;N;;N;;N</product> */
    8: optional i32                                 detection_mode,
    /** Quality of each captured fingers is required in the reply.
    * <product>Y;;Y;;Y;;Y;;Y;;N;;Y</product> */
    9: optional bool                                require_fingers_qualities,
    /** Compliance with sensor security level is required in the reply.
     * <product>N;;N;;N;;N;;N;;N;;N</product> */
    10: optional bool                               require_security_compatibilty,
    /** Number of capture per finger.<br>
     * For MorphoWave Compact, 1 for single swipe, other for consolidated swipe
     * <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product> */
    11: optional byte                               nb_capture,
    /** Acquisition threshold to use for the enrollment.<br>
     * Default value is standard
     * <product>N;;N;;N;;N;;N;;Y;;N</product> */
    12: optional Biofinger_acquisition_threshold          acquisition_threshold,    
}

/** Enumeration representing the operation of the enrollment process.*/
enum Enrollment_type
{
    store,
    transfer,
    both
}

/**
 * The finger was misplaced or has been withdrawn during finger acquisition performed.<br>
 * On FingerVein-equipped terminals only.
 */
exception Misplaced_finger_error
{
    /** Error code that represents the error.*/
    1: required Generic_types.Generic_error_code err_code
}

/** The format of the fingerprint template is not supported, or the fingerprint data does not match its type.*/
exception Unsupported_format_error
{
    /** Error code that represents the error.*/
    1: required Generic_types.Generic_error_code err_code
}

/** A reference template does not match sensor security policy.*/
exception Incompatible_ref_error
{
    /** Error code that represents the error.*/
    1: required Generic_types.Generic_error_code err_code
}

/** User uses a finger more than once.*/
exception Duplicated_finger_error
{
    /** Error code that represents the error.*/
    1: required Generic_types.Generic_error_code err_code
    /** last user failed id */
    2: optional string user_id_in_error_UTF8; 
}
