// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2012 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Factory_types.thrift
 *
 * CHANGELOG
 * 23 nov 2012  - Initiate header
 * 10 dec 2012  - Change namespace to comply MPH coding rules.
 * 19 dec 2012  - Add OEM parameter to Factory_info struct and factory_info_type enumeration
 *              - Create Factory_sensor_parameter_type enumeration
 * 30 jan 2013  - Commented "Factory_info_type" enumeration
 *              - Renamed "MMI_led_state" to "MMI_LED_state"
 * 12 feb 2013  - Create Factory_error_codes enumeration
 * 14 feb 2013  - Update Info_type comments for sensor board
 * 18 feb 2013  - Add defect into Factory_sensor_parameter_type enumeration
 *              - Create Factory_external_device enumeration
 *              - Create Fac_device_unmounted_error, Fac_read_file_error and Fac_write_file_error enums
 *              - Update Factory_error_code enumeration
 * 22 feb 2013  - Create Fac_file_not_found exception for Factory_sensor_read_parameter command.
 * 25 feb 2013  - Create Factory_GPIO_index and Factory_GPIO_state enums.
 * 27 feb 2013  - Create Fac_timeout and Fac_out_of_memory exception for Factory_check_port command.
 * 18 mar 2013  - Remove duplicates stuff (exceptions, errors)
 * 29 mar 2013  - Create Fac_alsa_error exception for factory_read_microphone command.
 * 22 apr 2013  - Update Factory_info_type comments.
 *              - Create Anti_instrusion_switch_state enums.
 * 29 may 2013  - Create Factory_Wiegand_LED_out and Factory_Wiegand_LED_state enums.
 * 24 jun 2013  - Create Factory_fuse_imx6 enumeration.
 * 26 jun 2013  - Uncomment Factory_info_type enumeration.
 * 23 jul 2013  - Remove obsoleted iMX6 fuse from Factory_fuse_imx6 enumeration.
 * 14 aug 2013  - Added Firmware upgrade command
 * 09 oct 2013  - Rename USB member of Factory_external_device enumeration.
 *              - Rename member of Factory_Wiegand_LED_state enumeration.
 * 18 oct 2013  - Create snd_pcm_format enumeration.
 * 10 jan 2014  - Create Touchpoint_coordinate structure.
 * 20 feb 2014  - Add namespace for java and C# generation
 * 24 mar 2014  - Added product support notes  
 * 03 jul 2014  - Added MMI_ID and MMI  
 * 10 jul 2014  - GPI_2, GPO_2 now supported now supported by all products   
 * 03 nov 2014  - GPI_2, GPO_2 not supported by MALite and MALiteS
 * 17 dec 2014  - Added parameter min_dwngd_version_UTF8 in struct Factory_info and min_dwngd_fw_version in Factory_info_type.
 * 24 dec 2014  - Removed unsed enumeration 'Sensor_camera_parameter'
 * 31 dec 2014  - Added parameter min_dwngd_version_UTF8 in struct Factory_info and min_dwngd_fw_version in Factory_info_type.
 * 29 jan 2015  - MorphoWave Product support changes  
 * 28 apr 2015  - Added exception for 'factory_clear_pairing_data'
 * 04 nov 2015  - Change java namespace for coherency
 *              - Merge with CIM
 * 09 dev 2015  - Add new field RFID_BOARD_TYPE to thrift types.
 * 11 may 2016  - Remove enum "Factory_sensor_parameter_type"  , not be use in factory and new sensor manager will not handle 
 * 17 june 2016 - Added parameter ITO_prism_version_UTF8 in struct Factory_info and ITO_prism_version in Factory_info_type.
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 30 Sept 2016 - Removed parameter ITO_prism_version_UTF8 in struct Factory_info and ITO_prism_version in Factory_info_type.
 * 24 Mar 2017  - PRGCA Product support added 
 * 24 Oct 2017  - Create Factory_temperature_sensors enumeration.
 * 02 Jul 2018  - MA_VP_MD Product support added 
 */
// </doc_history>

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

struct Factory_info
{
    /** UTF8 string that represents 10 digit CIE part number */
    1: optional string cie_part_number_UTF8,
    /** UTF8 string that represents 14 digit CIE serial number */
    2: optional string cie_serial_number_UTF8,
    /** UTF8 string that represents 4 digit CIE revision number */
    3: optional string cie_revision_UTF8,
    /** UTF8 string that represents 10 digit part number of packaged product */
    4: optional string packaged_part_number_UTF8,
    /** UTF8 string that represents 14 digit serial number of packaged product */
    5: optional string packaged_serial_number_UTF8,
    /** UTF8 string that represents 4 digit revision number of packaged product */
    6: optional string packaged_revision_UTF8,
    /** UTF8 string that represents MAC address of terminal Ethernet interface in XX:XX:XX:XX:XX:XX style */
    7: optional string ethernet_MAC_address_UTF8,
    /** UTF8 string that represents 10 digit product specific part number */
    8: optional string specific_part_number_UTF8,
    /** UTF8 string that represents 10 digit OEM part number */
    9: optional string oem_part_number_UTF8,
    /** UTF8 string that represents 14 digit OEM serial number */
    10: optional string oem_serial_number_UTF8,
    /** UTF8 string that represents 4 digit OEM revision number */
    11: optional string oem_revision_UTF8
    /** UTF8 string that represents 4 digit Pfuse version */
    12: optional string pfuse_version_UTF8
    /** UTF8 string that represents the minimal Firmware version that the board should accept */
    13: optional string min_fw_version_UTF8
    /** UTF8 string that represents the minimal downgrade version that the board should accept */
    14: optional string min_dwngd_version_UTF8
    /** RFID board type that represents the RFID card type embedded in terminal*/
    15: optional string RFID_board_type_UTF8
    /** UTF8 string that represents CIE part number of RFID board
    * <product>N;;N;;N;;N;;N;;Y;;N</product>
    */
    16: optional string RFID_board_cie_part_number_UTF8
    /** UTF8 string that represents CIE serial number of RFID board
    * <product>N;;N;;N;;N;;N;;Y;;N</product>
    */
    17: optional string RFID_board_cie_serial_number_UTF8
    /** UTF8 string that represents CIE revision number of RFID board
    * <product>N;;N;;N;;N;;N;;Y;;N</product>
    */
    18: optional string RFID_board_cie_revision_UTF8
}

struct Touchpoint_coordinate
{
    /** integer that represents X co-ordinate of touch point */
    1: i32 X,
    /** integer that represents Y co-ordinate of touch point */
    2: i32 Y
}

/**
 * <ps_fields>
 * <ps>sensor_board;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * </ps_fields>
 *
 * Enumeration for factory board type.
 */
enum Factory_board_type
{
    terminal_board,
    sensor_board
}

/**
 * <ps_fields>
 * <ps>RFID_board_cie_part_number;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>RFID_board_cie_serial_number;;N;;N;;N;;N;;N;;Y;;N</ps> 
 * <ps>RFID_board_cie_revision;;N;;N;;N;;N;;N;;Y;;N</ps>  
 * </ps_fields>
 *
 * Enumeration of Factory type */
enum Factory_info_type
{
    /** 10 digit number that represents CIE part number */
    cie_part_number,
    /** 14 digit number that represents CIE serial number */
    cie_serial_number,
    /** 4 digit number that represents CIE revision number */
    cie_revision,
    /** 10 digit number that represents part number of packaged product */
    packaged_part_number,
    /** 14 digit number that represents serial number of packaged product */
    packaged_serial_number,
    /** 4 digit number that represents revision number of packaged product */
    packaged_revision,
    /** MAC address of terminal Ethernet interface in XX-XX-XX-XX-XX-XX style */
    ethernet_MAC_address,
    /** 10 digit number that represents product specific part number */
    specific_part_number,
    /** 10 digit number that represents OEM part number */
    oem_part_number,
    /** 14 digit number that represents OEM serial number */
    oem_serial_number,
    /** 4 digit number that represents OEM revision number */
    oem_revision,
    /** 4 digit number that represents pfuse version number */
    pfuse_version,
    /** minimal Firmware version that the board should accept, example: 1.0.5 */
    min_fw_version,
    /** minimal downgrade Firmware version that the board should accept */
    min_dwngd_fw_version,
    /** RFID board type that represents the RFID card type embedded in terminal*/
    RFID_board_type,
    /** RFID board CIE part number */
    RFID_board_cie_part_number,
    /** RFID board CIE serial number */
    RFID_board_cie_serial_number,
    /** RFID board CIE revision number */    
    RFID_board_cie_revision,
}

/** Enumeration of available LEDs state */
enum MMI_LED_state
{
    led_ON,
    led_OFF
}

/**
 * <ps_fields>
 * <ps>nSD;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>USB_sdb1;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps>
 * </ps_fields>
 *
 * Enumeration of external device */
enum Factory_external_device
{
    nSD = 0,
    USB_sda1,
    USB_sdb1
}

/**
 * <ps_fields>
 * <ps>CPU_temperature;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>GPU_temperature;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>MEM_temperature;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>PLL_temperature;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>tmp451_temperature;;N;;N;;N;;N;;N;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration of temperature sensors */
enum Factory_temperature_sensors
{
    CPU_temperature = 0,
    GPU_temperature,
    MEM_temperature,
    PLL_temperature,
    tmp451_temperature,
    max_sensor_code
}

/**
 * <ps_fields>
 * <ps>GPI_2;;Y;;N;;N;;N;;Y;;Y;;Y</ps>
 * <ps>GPO_2;;Y;;N;;N;;N;;Y;;Y;;Y</ps>
 * </ps_fields>
 *
 * Enumeration of GPIO index */
enum Factory_GPIO_index
{
    GPI_0 = 0,
    GPI_1,
    GPI_2,
    GPO_0,
    GPO_1,
    GPO_2
}

/** Enumeration of GPIO state */
enum Factory_GPIO_state
{
    GPIO_state_low = 0,
    GPIO_state_high
}

/** Enumeration of Anti intrusion switch state */
enum Anti_intrusion_switch_state
{
    state_OK = 0,
    state_KO
}

/**
 * Enumeration of iMX SRK fuse
 */
enum Factory_fuse_imx6
{
    HW_OCOTP_SRK0 = 0,
    HW_OCOTP_SRK1,
    HW_OCOTP_SRK2,
    HW_OCOTP_SRK3,
    HW_OCOTP_SRK4,
    HW_OCOTP_SRK5,
    HW_OCOTP_SRK6,
    HW_OCOTP_SRK7,
    HW_OCOTP_MEM0,
    HW_OCOTP_CFG4,
    HW_OCOTP_CFG5
}

/** Enumeration of Wiegand LED state */
enum Factory_Wiegand_LED_state
{
    WIEG_LED_state_rejected = 0,
    WIEG_LED_state_accepted
}

/**
 * <ps_fields>
 * <ps>upgrade_policy_ok_and_copy_on_sd_card;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * </ps_fields>
 *
 * Enumeration of Upgrade policy return status */
enum Factory_upgrade_policy_status
{
    /** Upgrade policy ok, backup Firmware waits for new package on Ethernet or USB connection  */
    upgrade_policy_ok_and_reboot = 0,
    /** Upgrade policy ok, backup Firmware searches new package on sd card  */
    upgrade_policy_ok_and_copy_on_sd_card
}

/**
 * Factory error codes
 */
enum Factory_error_code
{
    err_inexistent_parameter = 0,
    err_invalid_value,
    err_device_unmounted,
    err_read_file,
    err_write_file,
    err_file_not_found,
    err_alsa_error,
    err_fuse_already_burned,
    err_firmware_upgrade_incompatible_firmware_version,
    err_firmware_upgrade_incompatible_product,
    err_firmware_upgrade_incompatible_hardware_revision,
    err_firmware_upgrade_policy_not_uploaded,
    err_firmware_upgrade_signature_failure,
    err_fac_clear_pairing_host_data_fail,
    err_fac_clear_pairing_sensor_data_fail,
}

/**
 * <product>N;;N;;Y;;N;;N;;N;;Y</product>
 * LED color enumeration.
 */
enum MMI_LED_color
{
    no_color,
    red,
    green,
    blue,
    white,
    yellow,
    purple,
    cyan
}

/**
 * <product>N;;Y;;Y;;N;;N;;N;;Y</product>
 * MMI IDs for different events.
 */
enum MMI_ID
{
  access_granted,
  access_denied,
  default_trigger,
  finger_only_trigger
}

/**
 * <product>N;;N;;Y;;N;;N;;N;;Y</product>
 * MMI LED configuration.
 */
struct MMI_LED
{
    /** LED color. If 'no_color', all other parameters ignored */
    1: required MMI_LED_color color,
    /** The duration for which the LED shall remain on. 0 means infinite duration */
    2: required i32 LED_on_duration_in_ms,
    /** The duration for which the LED shall remain off. Ignored if LED_on_duration_in_ms = 0 */
    3: required i32 LED_off_duration_in_ms,
    /** The number of flashes of LED. 0 means infinite loop of flashes. Ignored if LED_on_duration_in_ms = 0 */
    4: required i32 num_LED_flashes,
}

/**
 * <product>N;;Y;;Y;;N;;N;;N;;Y</product>
 * MMI buzzer configuration.
 */
struct MMI_buzzer
{
    /** Flag to indicate whether buzzer is on or off. If false, all other parameters are ignored */
    1: required bool buzzer_on,
    /** The duration for which the buzzer shall remain on. 0 means infinite duration */
    2: required i32 buzzer_on_duration_in_ms,
    /** The duration for which the buzzer shall remain off. Ignored if buzzer_on_duration_in_ms = 0 */
    3: required i32 buzzer_off_duration_in_ms,
    /** The number of the beeps of the buzzer. 0 means infinite loop of beeps. Ignored if buzzer_on_duration_in_ms = 0 */
    4: required i32 num_buzzer_beeps,
    /** The frequency of the buzzer */
    5: required i32 buzzer_frequency_in_Hz
}

/**
 * Configuration for the MMI.
 */
struct MMI
{
    /** LED configuration
     * <product>N;;N;;Y;;N;;N;;N;;Y</product>
     */
    1: optional MMI_LED LED_config,
    /** Buzzer configuration
     * <product>N;;Y;;Y;;N;;N;;N;;Y</product>
     */
    2: optional MMI_buzzer buzzer_config,
}

/** Select components to retrofit */ 
enum Component_type
{
    Bootloader = 0,
    Mapping,
    DTB,
    Kernel_A,
    Kernel_B,
    UBIFS,
    Application,
    Application_key,
}

/** A map that translates error code into string.*/
const map< Factory_error_code, string > Factory_error_msg_map =
{
    Factory_error_code.err_inexistent_parameter        : "Inexistent parameter",
    Factory_error_code.err_invalid_value               : "Invalid parameter value",
    Factory_error_code.err_device_unmounted            : "External device isn't mount",
    Factory_error_code.err_read_file                   : "Read file fail",
    Factory_error_code.err_write_file                  : "Write file fail",
    Factory_error_code.err_file_not_found              : "File not found",
    Factory_error_code.err_fuse_already_burned         : "Fuse already burned",
    Factory_error_code.err_firmware_upgrade_incompatible_firmware_version         : "Upgrade Firmware incompatible Firmware version",
    Factory_error_code.err_firmware_upgrade_incompatible_product         : "Upgrade Firmware incompatible product",
    Factory_error_code.err_firmware_upgrade_incompatible_hardware_revision         : "Upgrade Firmware incompatible hardware revision",
    Factory_error_code.err_firmware_upgrade_policy_not_uploaded         : "Upgrade Firmware policy not uploaded",
    Factory_error_code.err_firmware_upgrade_signature_failure         : "Upgrade Firmware signature failed",
    Factory_error_code.err_fac_clear_pairing_host_data_fail : "Clear pairing data failed for host",
    Factory_error_code.err_fac_clear_pairing_sensor_data_fail : "Clear pairing data failed for sensor",
}

/** One command argument can not be interpreted.*/
exception Fac_invalid_argument_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** External device isn't mounted.*/
exception Fac_device_unmounted_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** Cannot read target file.*/
exception Fac_read_file_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** Cannot write target file.*/
exception Fac_write_file_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** File not found.*/
exception Fac_file_not_found_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** ALSA error */
exception Fac_alsa_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

typedef binary Upgrade_policy_file

/** Firmware upgrade version incompatible */
exception Fac_firmware_upgrade_incompatible_version_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** Firmware upgrade product incompatible */
exception Fac_firmware_upgrade_incompatible_product_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** Firmware upgrade hardware revision incompatible */
exception Fac_firmware_upgrade_incompatible_hardware_revision_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** Firmware upgrade policy not uploaded */
exception Fac_firmware_upgrade_policy_not_uploaded_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** Firmware upgrade signature failure */
exception Fac_firmware_upgrade_signature_failure_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** factory clear pairing host data error*/
exception Fac_clear_pairing_host_data_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** factory clear pairing sensor data error*/
exception Fac_clear_pairing_sensor_data_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}

/** File not loaded.*/
exception Fac_file_not_loaded_error
{
    1: required Factory_error_code err_code;
    2: optional string what_UTF8;
}
