// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2012 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file File_types.thrift
 *
 * CHANGELOG
 * 23 nov 2012  - Initiate header
 * 10 dec 2012  - Change namespace to comply MPH coding rules.
 * 07 feb 2013  - Added 'File_subtype' enumeration and 'File_details' structure.
 * 16 feb 2013  - Renamed 'File_details.name' to 'File_details.name_UTF8'
 * 20 feb 2013  - Added 'stolen_card_list' in 'File_type' enumeration
 *              - Make subtype optional in File_details
 * 05 mar 2013  - Added 'diagnostic_info' in 'File_type' enumeration
 * 05 mar 2013  - Added 'ssl_component' in 'File_type' enumeration
 * 29 mar 2013  - Renamed 'ssl_component' to 'SSL_component' in 'File_type' enumeration
 *              - Changed 'File_subtype' enumeration
 * 31 mar 2013  - Added 'language' entry to 'File_type' enumeration
 * 14 aug 2013  - Added comment for enumeration play in 'File_action' 
 * 07 oct 2013  - Removed 'picture_slideshow' element from enumeration 'File_subtype'
 * 08 oct 2013  - Added 'firmware_upgrade' to enumeration 'File_type'
 * 20 feb 2014  - Add namespace for java and C# generation  
 * 24 mar 2014  - Added product support notes  
 * 05 nov 2014  - Added File_subtype.SSL_webserver enumerator
 * 29 jan 2015  - MorphoWave Product support changes 
 * 03 aug 2015  - 'biometric_sensor' is added in enumeration File_type and 'firmware' as its subtype for MorphoWave PC software upgrade 
 * 04 nov 2015  - Change java namespace for coherency
 *              - Merge with CIM
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 27 Mar 2017  - PRGCA Product support added
 * 05 oct 2017  - 'font' is added in enumeration File_type
 * 02 Jul 2018  - MA_VP_MD Product support added
 * 18 Jul 2019  - Added File subtype for Access Granted and Denied Logo for file type picture
                - Added File subtypes for File Type picture for customize animations for Card, Bio, QR Code trigger events
 */
// </doc_history>

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

/**
 * <ps_fields>
 * <ps>video;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>audio;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>picture;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>stolen_card_list;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps> 
 * <ps>language;;Y;;Y;;Y;;Y;;Y;;Y;;N</ps>
 * <ps>firmware_upgrade;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>biometric_sensor;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>font;;Y;;Y;;N;;N;;Y;;N;;N</ps>
 * </ps_fields>
 *
 * Enumeration for file types for file operations.
 */
enum File_type
{
    video,
    audio,
    picture,
    stolen_card_list,
    diagnostic_info,
    language,
    SSL_component,
    firmware_upgrade,
    biometric_sensor,
    font,
}

/**
 * <ps_fields>
 * <ps>Sensor;;N;;N;;N;;Y;;N;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration for file types for file operations.
 */
enum Upgrade_type
{
    Sensor,
}

/**
 * <ps_fields>
 * <ps>audio_verification_failed;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>audio_verification_succeeded;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>audio_message_attention;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>audio_tamper;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>audio_test;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>picture_dynamic_message;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>picture_wallpaper;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>customer_application;;N;;N;;N;;N;;N;;N;;N</ps>
 * <ps>customer_key;;N;;N;;N;;N;;N;;N;;N</ps>
 * <ps>SSL_webserver;;Y;;Y;;Y;;Y;;Y;;Y;;Y</ps>
 * <ps>firmware;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>localization_gui;;Y;;Y;;Y;;N;;Y;;Y;;N</ps>
 * <ps>localization_webserver;;Y;;Y;;Y;;N;;Y;;Y;;N</ps>
 * <ps>picture_access_granted;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>picture_access_denied;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>picture_card_animation;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>picture_biometric_animation;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>picture_QR_code_animation;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>picture_card_bio_animation;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>picture_bio_QR_animation;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>picture_card_QR_animation;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>picture_card_bio_QR_animation;;N;;N;;N;;N;;N;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration for file sub-types for file operations.
 */
enum File_subtype
{
    // Video file subtypes
    no_subtype,

    // Audio file subtypes
    audio_verification_failed,
    audio_verification_succeeded,
    audio_message_attention,
    audio_tamper,
    audio_test,

    // Picture file subtypes
    picture_dynamic_message,
    picture_wallpaper,
    
    // SSL_component subtypes
    SSL_profile_0,
    SSL_profile_1,
    // Customer application subtypes
    customer_application,
    // Customer key subtypes
    customer_key,
    
    SSL_webserver,
    
    firmware,

    localization_gui,
    localization_webserver,
    //Picture File Subtype Custom GUI Logo
    picture_access_granted,
    picture_access_denied,
    //Picture File Subtype for custom GUI animation
    picture_card_animation,
    picture_biometric_animation,
    picture_QR_code_animation,
    picture_card_bio_animation,
    picture_bio_QR_animation,
    picture_card_QR_animation,
    picture_card_bio_QR_animation,
}

struct File_details
{
    1: File_type type;
    2: optional File_subtype subtype;
    3: optional string name_UTF8;
}

/**
 * <ps_fields>
 * <ps>play;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration for file action after upload
 */
enum File_action
{
    /** Do nothing after upload*/
    nothing,
    /** Play file after upload, only applicable for audio and picture files */
    play
}

struct File_chunk
{
    /** File index of the data present in this chunk */
    1: i32      index;
    /** Set to true if this chunk is the last, false otherwise */
    2: bool     is_last;
    /** File data */
    3: binary   data;
    /** Action performed after transfer */
    4: optional File_action action;
}
