// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2013 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Internal_types.thrift
 *
 * CHANGELOG
 * 02 aug 2013  - Initiate header
 * 02 aug 2013  - Change namespace to comply MPH coding rules.
 * 02 aug 2013  - Added total and free memory parameter in terminal_info structure
 * 05 aug 2013  - Renamed variable to sd_card_capacity_in_bytes in terminal_info structure
 * 29 aug 2013  - Updated enumeration 'Data_export_type', added member 'crypto_keys'
 * 29 aug 2013  - Updatd structure 'Data_export_options', added member 'key_type'
 * 20 feb 2014  - Add namespace for java and C# generation  
 * 24 mar 2014  - Added product support notes
 * 25 jun 2014  - Added MALITE product enums in 'Product_type'
 * 30 jun 2014  - Added MMI_ID and MMI
 * 03 jul 2014  - Moved MMI_ID and MMI to factory
 * 10 jul 2014  - MALITE and MALITE/S is WR product
 * 21 jul 2014  - Non WR MALITE and MALITE/S products added
 * 17 nov 2014  - Non WR MALITE/S products removed
 * 29 jan 2015  - MorphoWave Product support changes 
 * 11 feb 2015  - Added struct User_DB_record_ext
 * 11 feb 2015  - Added enumeration User_data_fields
 * 04 mar 2015  - Added structure 'User_Information' and modified enumeration 'User_data_fields'
 * 24 mar 2015  - Rename fotf to ext_fotf,facp_wr removed 
 * 08 jun 2015  - Added structure 'Param_details' 
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 11 July 2016 - Added 'msi_ffd' sensor in enum 'Sensor_type'
 * 21 Sept 2016 - Added new product types for maxt and maxt_ffd
 * 27 Mar 2017  - PRGCA Product support added 
 * 10 Jul 2017 - Added new product types ans Sensor type for malite-vp-edf
 * 31 Oct 2017 - Remove PRGCA specific enum 'int_otf' from Sensor_type
 * 19 jan 2018  - Added enumeration Sensor_mode
 * 28 may 2018  - Added "users_20K" license enumeration
 * 03 Jul 2018  - MA_VP_MD Product support added 
 * 28 Sep 2018  - Added "Convert_template_info" structure
 */
// </doc_history>

include "Generic_types.thrift"
include "Cls_types.thrift"
include "Db_types.thrift"
include "Biofinger_types.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

/**
 * <ps_fields>
 * <ps>TNA_no_fkey;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F1;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F2;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F3;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F4;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F5;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F6;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F7;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F8;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F9;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F10;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F11;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F12;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F13;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F14;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F15;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>TNA_F16;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * </ps_fields>
 * Enumeration of F-Key values */
enum f_key_type
{
    TNA_no_fkey = 0,
    TNA_F1,
    TNA_F2,
    TNA_F3,
    TNA_F4,
    TNA_F5,
    TNA_F6,
    TNA_F7,
    TNA_F8,
    TNA_F9,
    TNA_F10,
    TNA_F11,
    TNA_F12,
    TNA_F13,
    TNA_F14,
    TNA_F15,
    TNA_F16
}

/**
 * <ps_fields>
 * <ps>msi;;N;;N;;N;;N;;Y;;N;;N</ps>
 * <ps>cbi;;Y;;Y;;Y;;N;;N;;N;;N</ps>
 * <ps>fvp;;N;;N;;N;;N;;N;;N;;Y</ps>
 * <ps>ext_fotf;;N;;N;;N;;Y;;N;;N;;N</ps>
 * <ps>msi_ffd;;N;;N;;N;;N;;Y;;N;;N</ps>
 * <ps>int_otf;;N;;N;;N;;N;;N;;Y;;N</ps> 
 * </ps_fields>
 *
 * Enumeration of Sensor type values.
 */
enum Sensor_type
{
    msi,
    cbi,
    fvp,
    ext_fotf,
    msi_ffd,
	int_otf,
	fvp_ext,
}

/**
 * <ps_fields>
 * <ps>Normal_mode;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>ECO_mode;;N;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>standby_mode;;N;;N;;N;;N;;N;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration of Sensor Mode.
 */
enum Sensor_mode
{
	normal_mode,
	ECO_mode,
	standby_mode,
}

/** Enumeration of product type*/
enum Product_type
{
    ma1000 = 0, //!< MA1000 Touch, 5” color + Touch screen, CBI-E, None, VGA face camera
    ma1001, //!< MA1001 Touch, 5” color + Touch screen, MSI with FFD, None, VGA face camera
    ma1010, //!< MA1010 Touch, 5” color + Touch screen, CBI-E, iClass, VGA face camera
    ma1011, //!< MA1011 Touch, 5” color + Touch screen, MSI with FFD, iClass, VGA face camera
    ma1020, //!< MA1020 Touch, 5” color + Touch screen CBI-E   Mifare/Desfire  VGA face camera
    ma1021, //!< MA1021 Touch, 5” color + Touch screen MSI with FFD    Mifare/Desfire  VGA face camera
    ma1030, //!< MA1030 Touch, 5” color + Touch screen CBI-E   Prox    VGA face camera
    ma1031, //!< MA1031 Touch, 5” color + Touch screen MSI with FFD    Prox    VGA face camera
    ma1000_fvp, //!< MA1000 VP Touch, 5” color + Touch screen FVP, None, VGA face camera
    ma1010_fvp, //!< MA1010 VP Touch, 5” color + Touch screen FVP, iClass, VGA face camera
    ma1020_fvp, //!< MA1020 VP Touch, 5” color + Touch screen FVP, Mifare/Desfire, VGA face camera
    ma1030_fvp, //!< MA1030 VP Touch, 5” color + Touch screen FVP, Prox, VGA face camera
    ma1000_wr,  //!< MA1000 WR Touch, 5” color + Touch screen CBI-E, None, VGA face camera
    ma1010_wr,  //!< MA1010 WR Touch, 5” color + Touch screen CBI-E, iClass, VGA face camera
    ma1020_wr,  //!< MA1020 WR Touch, 5” color + Touch screen CBI-E, Mifare/Desfire, VGA face camera
    ma1030_wr,   //!< MA1030 WR Touch, 5” color + Touch screen CBI-E, Prox, VGA face camera
    malite_wr, //!< MALITE WR, CBI-E
    malite_iclass_wr, //!< MALITE WR, CBI-E, iClass
    malite_multi_wr, //!< MALITE WR, CBI-E, Mifare/Desfire
    malite_prox_wr //!< MALITE WR, CBI-E, Prox
    malite_s_wr, //!< MALITE/S WR, CBI-E
    malite_s_iclass_wr, //!< MALITE/S WR, CBI-E, iClass
    malite_s_multi_wr, //!< MALITE/S WR, CBI-E, Mifare/Desfire
    malite_s_prox_wr //!< MALITE/S WR, CBI-E, Prox
    malite, //!< MALITE, CBI-E
    malite_iclass, //!< MALITE, CBI-E, iClass
    malite_multi, //!< MALITE, CBI-E, Mifare/Desfire
    malite_prox, //!< MALITE, CBI-E, Prox
    maxt_iclass, //!< MAXT, MSI iClass
    maxt_multi,  //!< MAXT, MSI Mifare/Desfire
    maxt_prox, //!< MAXT, MSI, Prox
    maxt_iclass_ffd, //!< MAXT, MSI_ITO iClass
    maxt_multi_ffd,  //!< MAXT, MSI_ITO Mifare/Desfire
    maxt_prox_ffd, //!< MAXT, MSI_ITO, Prox
    maxt, //!< MAXT, MSI
    maxt_ffd, //!< MAXT, MSI_ITO
    prgca_multi, //!< PRGCA Mifare/Desfire
    prgca_all_in_one, //!< PRGCA All In One
    ma_vp_md, //!< MALITE_VP
}

/** Terminal Info structure */
struct Terminal_info
{
    1: optional Sensor_type sensor_type,
    2: optional string firmware_version,
    3: optional Product_type product_type,
    4: optional string product_name,
    /** <product>Y;;Y;;Y;;N;;Y;;N;;N</product> */
    5: optional bool is_CBI_supported,
    /** <product>Y;;Y;;Y;;N;;Y;;N;;N</product> */
    6: optional bool is_MSI_supported,
    /** <product>Y;;Y;;Y;;N;;Y;;N;;Y</product> */
    7: optional bool is_FVP_supported,
    /** <product>Y;;Y;;Y;;N;;Y;;Y;;N</product> */
    8: optional bool is_iclass_supported,
    /** <product>Y;;Y;;Y;;N;;Y;;Y;;Y</product> */
    9: optional bool is_mifare_desfire_supported,
    /** <product>Y;;Y;;Y;;N;;Y;;Y;;N</product> */
    10: optional bool is_prox_supported,
    /** <product>Y;;N;;N;;Y;;Y;;N;;N</product> */
    11: optional bool is_face_camera_supported,
    /** <product>Y;;Y;;N;;Y;;Y;;Y;;N</product> */
    12: optional bool is_screen_supported,
    /** <product>Y;;N;;N;;Y;;Y;;N;;N</product> */
    13: optional bool is_sd_card_detected,
    /** <product>Y;;N;;N;;Y;;Y;;N;;N</product> */
    14: optional i64 sd_card_capacity_in_bytes,
    /** <product>Y;;N;;N;;Y;;Y;;N;;N</product> */
    15: optional i64 sd_card_free_space_in_bytes,
    16: optional i64 internal_memory_capacity_in_bytes,
    17: optional i64 internal_memory_free_space_in_bytes,
    /** <product>N;;N;;N;;Y;;N;;Y;;N</product> */
    18: optional bool is_FOTF_supported
	/** <product>N;;N;;N;;Y;;N;;N;;Y</product> */
    19: optional bool is_external_fvp_supported
}

/**
 * <ps_fields>
 * <ps>time_and_attendance;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>transaction_log_250K;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>transaction_log_500K;;Y;;N;;N;;Y;;Y;;N;;N</ps>
 * <ps>WIFI_communication;;Y;;Y;;Y;;N;;Y;;Y;;Y</ps> 
 * <ps>GSM_3g_GPRS_communication;;Y;;N;;N;;N;;N;;Y;;N</ps>
 * <ps>users_3K;;Y;;Y;;Y;;Y;;N;;N;;Y</ps> 
 * <ps>users_50K;;Y;;N;;N;;N;;Y;;Y;;N</ps>
 * <ps>users_100K;;Y;;N;;N;;N;;Y;;Y;;N</ps>
 * <ps>users_40K;;N;;N;;N;;Y;;N;;N;;N</ps> 
 * </ps_fields>
 *
 * Enumeration of module license */
enum Module_license_code
{
   time_and_attendance = 0,
   transaction_log_100K,
   transaction_log_250K,
   transaction_log_500K,
   transaction_log_1000K,
   wiegand_output_channel,
   clock_and_data_output_channel,
   IP_output_channel,
   WIFI_communication,
   GSM_3g_GPRS_communication,
   serial_output_channel,
   sdac,
   users_3K,
   users_10K,
   users_50K,
   users_100K,
   users_40K,
   users_20K 
}

/** Structure of module license status */
struct Module_license_status
{
    1: optional bool license_status,
    2: optional i32 capacity
}


/** Convert Template Info structure */
struct Convert_template_info
{
    1: required list<binary> input_templates,
    2: required Biofinger_types.Biofinger_template_type input_template_type,
    3: required Biofinger_types.Biofinger_template_type output_template_type,
    4: optional i32 truncate_size,
}


/**
 * <ps_fields>
 * <ps>transaction_log_csv;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>transaction_log_binary;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>user_database;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>crypto_keys;;Y;;N;;N;;N;;Y;;Y;;N</ps>
 * </ps_fields>
 *
 * Enumeration representing types of data to be exported */
enum Data_export_type
{
        transaction_log_csv,
        transaction_log_binary,
        error_log,
        user_database,
        crypto_keys
}

struct Data_export_options
{
        /** Type of data to be exported */
        1: Data_export_type data_export_type;
        /** File path where the data will be exported */
        2: string filepath_UTF8;
        /** Passphrase used for specific files only */
        3: optional string passphrase_UTF8;
        /** Crypto key type */
        4: optional Cls_types.Cls_card_type key_type;
}

/**
 * Contactless keys update type.
 */
enum Cls_keys_update_type
{
    generate_key,
    import_key
}

/**
 * Structure for user information.
 */
struct User_DB_record_ext
{
   1: Db_types.User_DB_record user_db_record;
   2: string user_id;
}

struct User_Information
{
   1: list<User_DB_record_ext> records;
   2: i32 total_num_of_records;
}

/**
 * <ps_fields>
 * <ps>user_role;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>duress_finger_index;;Y;;Y;;Y;;N;;Y;;N;;N</ps> 
 * <ps>PIN_code_UTF8;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>user_card_csn;;Y;;Y;;Y;;N;;N;;N;;Y</ps> 
 * <ps>job_code_list;;Y;;N;;N;;Y;;Y;;Y;;N</ps>
 * <ps>legacy_timemask;;Y;;Y;;Y;;N;;Y;;N;;Y</ps>
 * <ps>user_rule_BIO_substition;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>
 * <ps>extendedID;;Y;;Y;;Y;;N;;N;;N;;Y</ps>
 * <ps>first_finger_nb;;Y;;Y;;N;;Y;;Y;;Y;;Y</ps>
 * <ps>second_finger_nb;;Y;;Y;;N;;Y;;Y;;Y;;Y</ps>
 * <ps>third_finger_nb;;Y;;Y;;N;;Y;;Y;;Y;;N</ps>  
 * </ps_fields>
 *
 * Enumeration representing user information fields */
enum User_data_fields
{
    user_id,
    name,
    first_name,
    user_role,
    duress_finger_index,
    PIN_code_UTF8,
    user_card_csn,
    job_code_list,
    apply_holiday_schedule,
    legacy_timemask,
    specific_relay_duration_in_sec,
    expiry_date,
    user_rule_trigger_check,
    user_rule_ref_check,
    user_rule_control_check,
    user_rule_BIO_substition,
    extendedID,
    scheduleNumber,
    VIPList_User,
    WhiteList_User,
    first_finger_nb,
    second_finger_nb,
    third_finger_nb,
    records_offset,
    records_size,
}

struct User_data_order
{
    1: User_data_fields user_field,
    2: optional Db_types.Order_by sort_by
}

/**
 * Details of a particular configuration parameter.
 */
struct Param_details
{
    /** Value of the parameter */
    1: required Generic_types.Variant value,
    /** Default value of the parameter */
    2: required Generic_types.Variant default_value,
    /** Range of the parameter */
    3: Generic_types.Parameter_range range
}

exception No_space_left_on_device
{
    1: required Generic_types.Generic_error_code err_code
}

exception Not_compatible_with_current_license
{
    1: required Generic_types.Generic_error_code err_code
}

/** Invalid directory path exception */
exception Invalid_directory_path_error
{
    1: required Generic_types.Generic_error_code err_code
}

/** The passphrase specified for import is not correct.*/
exception DB_invalid_passphrase_error
{
    1: required Generic_types.Generic_error_code err_code
}

exception Invalid_file
{
    1: required Generic_types.Generic_error_code err_code
}
