// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2013 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file QA_types.thrift
 *
 * CHANGELOG
 * 28 may 2013  - Add 'Cbi_simulation_files' structure
 * 05 Aug 2013  - Change 'Cbi_simulation_files' structure to add file name
 * 06 Aug 2013  - Updated comments in 'Cbi_simulation_files'
 * 20 feb 2014  - Add namespace for java and C# generation
 * 29 jan 2015  - MorphoWave Product support changes
 * 27 June 2016 - Added MA-Extreme product for distant command guide
 * 20 july 2016 - Changed simulation thrift command for new simulation changes in CBI/MSI SDK
 *                renamed Cbi_simulation_files to Simulation_files 
 * 09 Aug 2016  - added new field 'link_name' in structure 'Simulation_files'
 * 30 Sept 2016 - added new structure 'Simulation_files' 
 * 24 Mar 2017  - PRGCA Product support added 
 * 17 June 2019  - Added new enum 'Life_cycle_counter_types' for PRGCA
 */
// </doc_history>

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd

/**
 * Structure for sensor simulation mode
 */
struct Simulation_files
{
    /**
     * scfg ini file data (required to change parameter like target, resetrecordidxeachstart) 
     * <note>Terminal reboot required after change of ini file data.</note>
     */
    1: optional binary scfg_ini_file_data,

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y</product>
     * simu record directory path (absolute directory path on terminal). Terminal creates softlink 
     * of this directory inside directory specified by pathsimurecords parameter in scfg ini file.
     * <note>Name of softlink is specified by field 'Simulation_files.link_name'</note>
     */
    2: optional string simu_record_path,

    /**
     * <product>Y;;Y;;Y;;N;;Y;;Y</product>
     * simu record link name. Terminal creates softlink with this name inside directory specified by 
     * pathsimurecords parameter in scfg ini file
     */
    3: optional string link_name,
}

/**
 * Structure for sensor simulation mode
 */
struct Cbi_simulation_files
{
    /**
     * raw image file path (absolute file path on terminal)
     */
    1: optional string raw_image_file_name,

    /**
     * <product>Y;;Y;;Y;;N;;N;;N</product>
     * eeprom file path (absolute file path on terminal)
     */
    2: optional string eeprom_file_name,

    /**
     * <product>Y;;Y;;Y;;N;;N;;N</product>
     * uniformy file path (absolute file path on terminal)
     */
    3: optional string uniformity_file_name,

    /**
     * <product>Y;;Y;;Y;;N;;N;;N</product>
     * deconvolution file path (absolute file path on terminal)
     */
    4: optional string deconvolution_file_name
}

 /**
 * Enumeration of life cycle counter type.
 */
enum Life_cycle_counter_types
{
    lcc_num_hands_detected_by_cam,
    lcc_num_hands_detected_by_proximity,
    lcc_num_biometric_hit_auth,
    lcc_num_biometric_nohit_auth,
    lcc_num_biometric_hit_identify,
    lcc_num_biometric_nohit_identify,
    lcc_num_success_enroll,
    lcc_num_failed_enroll,
    lcc_num_cancelled_enroll
}

